`timescale 1ns / 1ps
//////////////////////////////////////////////////////////////////////////////////
// Company: 
// Engineer: 
// 
// Create Date: 11/16/2017 08:55:54 AM
// Design Name: 
// Module Name: cpu_tb
// Project Name: 
// Target Devices: 
// Tool Versions: 
// Description: 
// 
// Dependencies: 
// 
// Revision:
// Revision 0.01 - File Created
// Additional Comments:
// 
//////////////////////////////////////////////////////////////////////////////////

`define SIM

module cpu_tb
	();

	`include "tasks.v"
	
    reg clk, rst_n;
	

	////////////////////////////////////////////////////////////////////////////
    // Instantiate the 64k of BlockRAM for main memory
    ////////////////////////////////////////////////////////////////////////////
    wire	[15:0]			memAB;				// Address bus
    wire	[7:0]			memDO;				// data-read bus
    wire	[7:0]			memDI;				// data-write bus
    wire					memWE;				// Write-enable
    
    bram mem_inst
    	(
    	.clk(clk),
    		
    	.addr(memAB),
    	.we(memWE),
    	.din(memDI),
    	.dout(memDO)
    	);

	////////////////////////////////////////////////////////////////////////////
    // Instantiate the CPU
    ////////////////////////////////////////////////////////////////////////////
    reg						irq_n;				// External in: IRQ
    reg						nmi_n;				// External in: Non-maskable IRQ
    
	
	cpu cpu_inst
		(
		.clk(clk),								// System clock
		.reset(~rst_n),							// Active high reset
		
		.DI(memDO),								// Read:  data @ (address)
		.DO(memDI),								// Write)
		.AB(memAB),								// Address bus
		.WE(memWE),								// Write-enable
		
		.RDY(1'b1),								// Pause the CPU if low
		.IRQ(irq_n),							// external IRQ if low
		.NMI(nmi_n)								// non-maskable IRQ if low
		);
		
	////////////////////////////////////////////////////////////////////////////
	// Clock count
	////////////////////////////////////////////////////////////////////////////
	reg	[31:0] counter;

	
	////////////////////////////////////////////////////////////////////////////
	// Initialise everything
	////////////////////////////////////////////////////////////////////////////
	initial
		begin
            $dumpfile("/tmp/6502.vcd");
            $dumpvars(0, cpu_tb);

			clk			<= 1'b1;
			rst_n		<= 1'b1;
			irq_n		<= 1'b1;
			nmi_n		<= 1'b1;
			counter 	<= 32'b0;

			#10	// Allow the above to "take"

			testAddrAcc();
			/*testAddrImplicit();
			testAddrJump();
			testAddrImmediate();
			testAddrZp();
			testAddrZpX();
			testAddrAbs();
			testAddrAbsX();
			testAddrAbsY();
          	testZpIndxX();
            testZpIndxY();			
			testBranch();
*/


			#80 
				$finish;
		end
		
	////////////////////////////////////////////////////////////////////////////
	// Toggle the clock indefinitely
	////////////////////////////////////////////////////////////////////////////
    always 
    	begin
	        #5 clk = ~clk;
			if (clk)
				counter = counter + 1;
		end

endmodule
