
	////////////////////////////////////////////////////////////////////////////
	// Setters for the CPU registers and state
	////////////////////////////////////////////////////////////////////////////
	function automatic [7:0] getA;
		input dummy;
		getA = cpu_inst.AXYS[0];
	endfunction


	task setA;
		input 	[7:0]		a;
		cpu_inst.AXYS[0]	= a;
	endtask
	
	
	task setX;
		input 	[7:0]		x;
		cpu_inst.AXYS[2]	= x;
	endtask
	
	function [7:0] getX;
		input dummy;
		begin
			getX = cpu_inst.AXYS[2];
		end
	endfunction
	
	task setY;
		input 	[7:0]		y;
		cpu_inst.AXYS[3]	= y;
	endtask
	
	function [7:0] getY;
		input dummy;
		begin
			getY = cpu_inst.AXYS[3];
		end
	endfunction
	
	task setSP;
		input 	[7:0]		sp;
		cpu_inst.AXYS[1]	= sp;
	endtask
	
	function [7:0] getSP;
		input dummy;
		begin
			getSP = cpu_inst.AXYS[1];
		end
	endfunction
	
	task setPS;
		input 	[7:0]		ps;
		begin
			cpu_inst.N	= ps[7];
			cpu_inst.V	= ps[6];
			cpu_inst.D	= ps[3];
			cpu_inst.I	= ps[2];
			cpu_inst.Z 	= ps[1];
			cpu_inst.C	= ps[0];
		end
	endtask
	
	function [7:0] getPS;
		input dummy;
		begin
			getPS = {cpu_inst.N, cpu_inst.V, 1'b1,       1'b1,
					 cpu_inst.D, cpu_inst.I, cpu_inst.Z, cpu_inst.C};
		end
	endfunction
	
	task setPC;
		input 	[15:0]		pc;
		cpu_inst.PC	= pc;
	endtask
	
	function [15:0] getPC;
		input dummy;
		begin
			getPC = cpu_inst.PC;
		end
	endfunction

	task setMem;
		input 	[15:0]		addr;
		input	[7:0]		val;
		begin
				mem_inst.mem_inst.mem[addr] 	= val;
		end
	endtask

	function [7:0] getMem;
		input 	[15:0]		addr;
		begin
				getMem = mem_inst.mem_inst.mem[addr];
		end
	endfunction

	////////////////////////////////////////////////////////////////////////////
	// Set up a task to set the internal state of the CPU
	////////////////////////////////////////////////////////////////////////////
	task setState;
		input	[7:0]		sr;
		input	[15:0]		pc;
		input	[7:0]		a;
		input	[7:0]		x;
		input	[7:0]		y;
		input	[7:0]		sp;
	 
	 	integer vector;
	
		begin

            	 	rst_n 		= 1'b0;
            #10 	rst_n 		= 1'b1;

			
			vector = pc;
			 	
			setPC(pc);
			setA(a);
			setX(x);
			setY(y);
			setPS(sr);
			setSP(sp);
			counter 			= 0;
			setMem(16'hfffc, vector[7:0]);
			setMem(16'hfffd, vector[15:0]);
			
		end
	endtask

	////////////////////////////////////////////////////////////////////////////
	// Set up a task to set the internal state of the CPU, default params
	////////////////////////////////////////////////////////////////////////////
	task resetState;
		setState(8'h20,0,0,0,0,8'hff);
	endtask

	
	////////////////////////////////////////////////////////////////////////////
	// Set up a task to check the results of an opcode vs the expectation and
	// verify that a result was written to memory
	////////////////////////////////////////////////////////////////////////////
	task checkMem;
		input 	[47:0]		b;				// bytes to insert into RAM @ PC
		input	[7:0]		sr;				// Set the stack register
		input	[15:0]		pc;				// Set the program counter
		input	[7:0]		a;				// Set A
		input	[7:0]		x;				// Set X
		input	[7:0]		y;				// Set Y
		input	[7:0]		sp;				// Set the Processor status
		input	[4:0]		insns;			// Number of instructions to run
		input	[7:0]		clocks;			// Number of clocks expected
		input 	[15:0]		aStart;			// address to check
		input	[2:0]		aBytes;			// number of bytes
		input 	[47:0]		aData;			// bytes to check
		
		reg					error, ok;
		integer 			i;
		reg 	[20*4:1] 	s;

		begin
			s = "";
			setMem(cpu_inst.PC+0, b[7:0]);
			setMem(cpu_inst.PC+1, b[15:8]);
			setMem(cpu_inst.PC+2, b[23:16]);
			setMem(cpu_inst.PC+3, b[31:24]);
			setMem(cpu_inst.PC+4, b[39:32]);
			setMem(cpu_inst.PC+5, b[47:40]);
			error = 0;
			
					
			for (i=0; i<insns; i = i + 1)
				begin
					wait (cpu_inst.state == 8'h0c);		// decode
					wait (cpu_inst.state != 8'h0c);
				end

			// Wait a clock so we can see what the result is before
			// rst_n resets the values
			#20	begin
					if (getPC(0) !== pc)
						begin
							s = {s,"PC "};
							error = 1;
						end
					
					if (getPS(0) !== sr)
						begin
							error = 1;
							s = {s,"PS "};
						end

					if (getA(0) !== a)
						begin
							s = {s,"A "};
							error = 1;
						end

					if (getX(0) !== x)
						begin
							s = {s,"X "};
							error = 1;
						end

					if (getY(0) !== y)
						begin
							s = {s,"Y "};
							error = 1;
						end

					if (getSP(0) !== sp)
						begin
							s = {s,"SP "};
							error = 1;
						end

					if (counter-1 !== clocks) // -1 for the reset cycle
						begin
							s = {s,"# "};
							error = 1;
						end
					
					if (aStart != 16'hffff)
						begin
							if (aBytes == 1)
								ok 	= (getMem(aStart) === aData[7:0]);
							else if (aBytes == 2)
								ok 	= (getMem(aStart)   === aData[7:0])
									||  (getMem(aStart+1) === aData[15:8]);
							else if (aBytes == 3)
								ok 	= (getMem(aStart)   === aData[7:0])
									||  (getMem(aStart+1) === aData[15:8])
									||  (getMem(aStart+2) === aData[23:16]);

							if (ok == 1'b0)
								begin
									s = {s,"mem"};
									error = 1;
								end
						end
					
					if (error == 1'b0)
						begin
							$display("%c[32m PASS%c[30m", 27, 27);
						end
					else 
						begin
							$display("%c[31m FAIL%c[30m", 27, 27);
							$display("[%s] {%s%s..%s%s%s%s} [%h] PC=%h, A=%h, X=%h, Y=%h, SP=%h, PS=%x, #=%2d", 
								s,
								((cpu_inst.N == 1'b1) ? "n" : "-"),
								((cpu_inst.V == 1'b1) ? "v" : "-"),
								((cpu_inst.D == 1'b1) ? "d" : "-"),
								((cpu_inst.I == 1'b1) ? "i" : "-"),
								((cpu_inst.Z == 1'b1) ? "z" : "-"),
								((cpu_inst.C == 1'b1) ? "c" : "-"),
								b,
								cpu_inst.PC,
								cpu_inst.A,
								cpu_inst.X,
								cpu_inst.Y,
								getSP(0),
								getPS(0),
								counter-1); 
						end
				end
		end
	endtask
	
	////////////////////////////////////////////////////////////////////////////
	// Set up a task to check the results of an opcode vs the expectation
	////////////////////////////////////////////////////////////////////////////
	task check;
		input 	[47:0]		b;
		input	[7:0]		sr;
		input	[15:0]		pc;
		input	[7:0]		a;
		input	[7:0]		x;
		input	[7:0]		y;
		input	[7:0]		sp;
		input	[4:0]		insns;
		input	[7:0]		clocks;

		checkMem(b, sr, pc, a, x, y, sp, insns, clocks, 16'hffff, 8'h0, 48'b0);
	endtask

	
	////////////////////////////////////////////////////////////////////////////
	// Test all the accumulator-mode operations
	////////////////////////////////////////////////////////////////////////////
	task testAddrAcc;
		begin
			$display("\nTest accumulator addressing:");

		 	////////////////////////////////////////////////////////////////////
			// ASL accumulator: check result, Z, C
			////////////////////////////////////////////////////////////////////
            $display("");
            resetState();
 			setA(8'h80);
            $write("%40s", "ASL A [check Z, C] :");
            //   (bytes,  ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'h0A, 8'h23, 1,  8'h00, 8'h00, 8'h00, 8'hff, 1, 2);
		           
		 	////////////////////////////////////////////////////////////////////
			// ASL accumulator: check result, N
			////////////////////////////////////////////////////////////////////
            resetState();
 			setA(8'h40);
            $write("%40s", "ASL A [check N] :");
            //   (bytes,  ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'h0A, 8'ha0, 1,  8'h80, 8'h00, 8'h00, 8'hff, 1, 2);
            
 		 	////////////////////////////////////////////////////////////////////
			// ASL accumulator: check result, no unnecessary flags 
			////////////////////////////////////////////////////////////////////
            resetState();
 			setA(8'h10);
            $write("%40s", "ASL A [check no flags] :");
            //   (bytes,  ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'h0A, 8'h20, 1,  8'h20, 8'h00, 8'h00, 8'hff, 1, 2);

							

		 	////////////////////////////////////////////////////////////////////
			// LSR accumulator: check result, Z, C
			////////////////////////////////////////////////////////////////////
            $display("");
            resetState();
 			setA(8'h01);
            $write("%40s", "LSR A [check Z, C] :");
            //   (bytes,  ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'h4A, 8'h23, 1,  8'h00, 8'h00, 8'h00, 8'hff, 1, 2);
		                       
 		 	////////////////////////////////////////////////////////////////////
			// LSR accumulator: check result, no unnecessary flags 
			////////////////////////////////////////////////////////////////////
            resetState();
 			setA(8'h10);
            $write("%40s", "LSR A [check no flags] :");
            //   (bytes,  ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'h4A, 8'h20, 1,  8'h08, 8'h00, 8'h00, 8'hff, 1, 2);



		 	////////////////////////////////////////////////////////////////////
			// ROL accumulator: check result, Z, C
			////////////////////////////////////////////////////////////////////
            $display("");
            resetState();
 			setA(8'h80);
            $write("%40s", "ROL A [check Z, C] :");
            //   (bytes,  ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'h2A, 8'h23, 1,  8'h00, 8'h00, 8'h00, 8'hff, 1, 2);
		           
		 	////////////////////////////////////////////////////////////////////
			// ROL accumulator: check result, N
			////////////////////////////////////////////////////////////////////
            resetState();
 			setA(8'h40);
            $write("%40s", "ROL A [check N] :");
            //   (bytes,  ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'h2A, 8'ha0, 1,  8'h80, 8'h00, 8'h00, 8'hff, 1, 2);
            
 		 	////////////////////////////////////////////////////////////////////
			// ROL accumulator: check result, no unnecessary flags 
			////////////////////////////////////////////////////////////////////
            resetState();
 			setA(8'h10);
            $write("%40s", "ROL A [check no flags] :");
            //   (bytes,  ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'h2A, 8'h20, 1,  8'h20, 8'h00, 8'h00, 8'hff, 1, 2);
            
 		 	////////////////////////////////////////////////////////////////////
			// ROL accumulator: check carry-in,  
			////////////////////////////////////////////////////////////////////
            resetState();
 			setA(8'h10);
 			setPS(8'h21);
            $write("%40s", "ROL A [check carry in] :");
            //   (bytes,  ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'h2A, 8'h20, 1,  8'h21, 8'h00, 8'h00, 8'hff, 1, 2);



		 	////////////////////////////////////////////////////////////////////
			// ROR accumulator: check result, Z, C
			////////////////////////////////////////////////////////////////////
            $display("");
            resetState();
 			setA(8'h01);
            $write("%40s", "ROR A [check Z, C] :");
            //   (bytes,  ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'h6A, 8'h23, 1,  8'h00, 8'h00, 8'h00, 8'hff, 1, 2);
		           
		 	////////////////////////////////////////////////////////////////////
			// ASL accumulator: check result, N
			////////////////////////////////////////////////////////////////////
            resetState();
  			setPS(8'h21);
			setA(8'h40);
            $write("%40s", "ROR A [check N] :");
            //   (bytes,  ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'h6A, 8'ha0, 1,  8'ha0, 8'h00, 8'h00, 8'hff, 1, 2);
            
 		 	////////////////////////////////////////////////////////////////////
			// ASL accumulator: check result, no unnecessary flags 
			////////////////////////////////////////////////////////////////////
            resetState();
 			setA(8'h10);
            $write("%40s", "ROR A [check no flags] :");
            //   (bytes,  ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'h6A, 8'h20, 1,  8'h08, 8'h00, 8'h00, 8'hff, 1, 2);
            
 		 	////////////////////////////////////////////////////////////////////
			// ASL accumulator: check carry-in,  
			////////////////////////////////////////////////////////////////////
            resetState();
 			setA(8'h10);
 			setPS(8'h21);
            $write("%40s", "ROR A [check carry in] :");
            //   (bytes,  ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'h6A, 8'ha0, 1,  8'h88, 8'h00, 8'h00, 8'hff, 1, 2);	
        end
	endtask
	
	
	////////////////////////////////////////////////////////////////////////////
	// Test all the non-jump-style implicit-mode operations
	////////////////////////////////////////////////////////////////////////////
	task testAddrImplicit;
		begin
			$display("\nTest implicit addressing:");
		 	////////////////////////////////////////////////////////////////////
			// CLD: check result
			////////////////////////////////////////////////////////////////////
            $display("");
            resetState();
 			setPS(8'hff);
            $write("%40s", "CLD [check flags] :");
            //   (bytes,  ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hD8, 8'hf7, 1,  8'h00, 8'h00, 8'h00, 8'hff, 1, 2);

		 	////////////////////////////////////////////////////////////////////
			// CLI: check result
			////////////////////////////////////////////////////////////////////
            resetState();
 			setPS(8'hff);
            $write("%40s", "CLI [check flags] :");
            //   (bytes,  ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'h58, 8'hfb, 1,  8'h00, 8'h00, 8'h00, 8'hff, 1, 2);

		 	////////////////////////////////////////////////////////////////////
			// CLC: check result
			////////////////////////////////////////////////////////////////////
            resetState();
 			setPS(8'hff);
            $write("%40s", "CLC [check flags] :");
            //   (bytes,  ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'h18, 8'hfe, 1,  8'h00, 8'h00, 8'h00, 8'hff, 1, 2);

		 	////////////////////////////////////////////////////////////////////
			// CLV: check result
			////////////////////////////////////////////////////////////////////
            resetState();
 			setPS(8'hff);
            $write("%40s", "CLV [check flags] :");
            //   (bytes,  ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hb8, 8'hbf, 1,  8'h00, 8'h00, 8'h00, 8'hff, 1, 2);
					
						

		 	////////////////////////////////////////////////////////////////////
			// DEX: check  Z
			////////////////////////////////////////////////////////////////////
            $display("");
            resetState();
 			setX(8'h01);
            $write("%40s", "DEX [check Z] :");
            //   (bytes,  ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hCA, 8'h22, 1,  8'h00, 8'h00, 8'h00, 8'hff, 1, 2);

		 	////////////////////////////////////////////////////////////////////
			// DEX: check  N
			////////////////////////////////////////////////////////////////////
            resetState();
            $write("%40s", "DEX [check N] :");
            //   (bytes,  ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hCA, 8'ha0, 1,  8'h00, 8'hff, 8'h00, 8'hff, 1, 2);

		 	////////////////////////////////////////////////////////////////////
			// DEX: check no erroneous flags
			////////////////////////////////////////////////////////////////////
            resetState();
 			setX(8'h40);
            $write("%40s", "DEX [check no extra flags] :");
            //   (bytes,  ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hCA, 8'h20, 1,  8'h00, 8'h3f, 8'h00, 8'hff, 1, 2);
					
						

		 	////////////////////////////////////////////////////////////////////
			// DEY: check  Z
			////////////////////////////////////////////////////////////////////
            $display("");
            resetState();
 			setY(8'h01);
            $write("%40s", "DEY [check Z] :");
            //   (bytes,  ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'h88, 8'h22, 1,  8'h00, 8'h00, 8'h00, 8'hff, 1, 2);

		 	////////////////////////////////////////////////////////////////////
			// DEY: check  N
			////////////////////////////////////////////////////////////////////
            resetState();
            $write("%40s", "DEY [check N] :");
            //   (bytes,  ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'h88, 8'ha0, 1,  8'h00, 8'h00, 8'hff, 8'hff, 1, 2);

		 	////////////////////////////////////////////////////////////////////
			// DEY: check no erroneous flags
			////////////////////////////////////////////////////////////////////
            resetState();
 			setY(8'h40);
            $write("%40s", "DEY [check no extra flags] :");
            //   (bytes,  ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'h88, 8'h20, 1,  8'h00, 8'h00, 8'h3f, 8'hff, 1, 2);
					
						

		 	////////////////////////////////////////////////////////////////////
			// INX: check  Z
			////////////////////////////////////////////////////////////////////
            $display("");
            resetState();
 			setX(8'hff);
            $write("%40s", "INX [check Z] :");
            //   (bytes,  ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'he8, 8'h22, 1,  8'h00, 8'h00, 8'h00, 8'hff, 1, 2);

		 	////////////////////////////////////////////////////////////////////
			// INX: check  N
			////////////////////////////////////////////////////////////////////
            resetState();
 			setX(8'h7f);
            $write("%40s", "INX [check N] :");
            //   (bytes,  ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'he8, 8'ha0, 1,  8'h00, 8'h80, 8'h00, 8'hff, 1, 2);

		 	////////////////////////////////////////////////////////////////////
			// INX: check no erroneous flags
			////////////////////////////////////////////////////////////////////
            resetState();
 			setX(8'h40);
            $write("%40s", "INX [check no extra flags] :");
            //   (bytes,  ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'he8, 8'h20, 1,  8'h00, 8'h41, 8'h00, 8'hff, 1, 2);
					
						

		 	////////////////////////////////////////////////////////////////////
			// INY: check  Z
			////////////////////////////////////////////////////////////////////
            $display("");
            resetState();
 			setY(8'hff);
            $write("%40s", "INY [check Z] :");
            //   (bytes,  ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hC8, 8'h22, 1,  8'h00, 8'h00, 8'h00, 8'hff, 1, 2);

		 	////////////////////////////////////////////////////////////////////
			// INY: check  N
			////////////////////////////////////////////////////////////////////
            resetState();
 			setY(8'h7f);
            $write("%40s", "INY [check N] :");
            //   (bytes,  ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hC8, 8'ha0, 1,  8'h00, 8'h00, 8'h80, 8'hff, 1, 2);

		 	////////////////////////////////////////////////////////////////////
			// INY: check no erroneous flags
			////////////////////////////////////////////////////////////////////
            resetState();
 			setY(8'h40);
            $write("%40s", "INY [check no extra flags] :");
            //   (bytes,  ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hC8, 8'h20, 1,  8'h00, 8'h00, 8'h41, 8'hff, 1, 2);
					
						

		 	////////////////////////////////////////////////////////////////////
			// NOP: check no effect
			////////////////////////////////////////////////////////////////////
            $display("");
            resetState();
            $write("%40s", "NOP [check flags, PC] :");
            //   (bytes,  ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hEA, 8'h20, 1,  8'h00, 8'h00, 8'h00, 8'hff, 1, 2);
					
						

		 	////////////////////////////////////////////////////////////////////
			// PHA: check SP
			////////////////////////////////////////////////////////////////////
            $display("");
            resetState();
 			setA(8'h42);
            $write("%40s", "PHA [check stack] :");
            //      (bytes,  ps,    pc, a,     x,     y,     sp,    #, ck, addr,   #, val)
            checkMem(48'h48, 8'h20, 1,  8'h42, 8'h00, 8'h00, 8'hfe, 1, 3, 16'h1ff, 1, 48'h42);

		 	////////////////////////////////////////////////////////////////////
			// PHP: check SP
			////////////////////////////////////////////////////////////////////
            resetState();
 			setPS(8'h92);
            $write("%40s", "PHP [check stack] :");
            //      (bytes,  ps,    pc, a,     x,     y,     sp,    #, ck, addr,   #, val)
            checkMem(48'h08, 8'h92, 1,  8'h00, 8'h00, 8'h00, 8'hfe, 1, 3, 16'h1ff, 1, 48'h92);

		 	////////////////////////////////////////////////////////////////////
			// PLA: check SP, A
			////////////////////////////////////////////////////////////////////
            resetState();
            setMem(16'h01ff,8'h42);
            setSP(8'hfe);
            $write("%40s", "PLA [check stack] :");
            //      (bytes,  ps,    pc, a,     x,     y,     sp,    #, ck, addr,   #, val)
            checkMem(48'h68, 8'h20, 1,  8'h42, 8'h00, 8'h00, 8'hff, 1, 4, 16'h1ff, 1, 48'h42);

		 	////////////////////////////////////////////////////////////////////
			// PLP: check SP, A
			////////////////////////////////////////////////////////////////////
            resetState();
            setMem(16'h01ff,8'h62);
            setSP(8'hfe);
            $write("%40s", "PLP [check stack] :");
            //      (bytes,  ps,    pc, a,     x,     y,     sp,    #, ck, addr,   #, val)
            checkMem(48'h28, 8'h62, 1,  8'h00, 8'h00, 8'h00, 8'hff, 1, 4, 16'h1ff, 1, 48'h62);


		 	////////////////////////////////////////////////////////////////////
			// SEC: check result
			////////////////////////////////////////////////////////////////////
            $display("");
            resetState();
            $write("%40s", "SEC [check flags] :");
            //   (bytes,  ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'h38, 8'h21, 1,  8'h00, 8'h00, 8'h00, 8'hff, 1, 2);
            
		 	////////////////////////////////////////////////////////////////////
			// SED: check result
			////////////////////////////////////////////////////////////////////
            resetState();
            $write("%40s", "SED [check flags] :");
            //   (bytes,  ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hf8, 8'h28, 1,  8'h00, 8'h00, 8'h00, 8'hff, 1, 2);
            
		 	////////////////////////////////////////////////////////////////////
			// SEI: check result
			////////////////////////////////////////////////////////////////////
            resetState();
            $write("%40s", "SEI [check flags] :");
            //   (bytes,  ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'h78, 8'h24, 1,  8'h00, 8'h00, 8'h00, 8'hff, 1, 2);


		 	////////////////////////////////////////////////////////////////////
			// TAX: check N, X
			////////////////////////////////////////////////////////////////////
            $display("");
            resetState();
            setA(8'hfe);
            $write("%40s", "TAX [check N, X] :");
            //   (bytes,  ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'haa, 8'ha0, 1,  8'hfe, 8'hfe, 8'h00, 8'hff, 1, 2);
            
		 	////////////////////////////////////////////////////////////////////
			// TAX: check Z, X
			////////////////////////////////////////////////////////////////////
            resetState();
            $write("%40s", "TAX [check Z, X] :");
            //   (bytes,  ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'haa, 8'h22, 1,  8'h00, 8'h00, 8'h00, 8'hff, 1, 2);
            
		 	////////////////////////////////////////////////////////////////////
			// TAX: check no unreasonable flags
			////////////////////////////////////////////////////////////////////
            resetState();
            setA(8'h42);
            $write("%40s", "TAX [check no extra flags set] :");
            //   (bytes,  ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'haa, 8'h20, 1,  8'h42, 8'h42, 8'h00, 8'hff, 1, 2);



		 	////////////////////////////////////////////////////////////////////
			// TAY: check N, Y
			////////////////////////////////////////////////////////////////////
            $display("");
            resetState();
            setA(8'hfe);
            $write("%40s", "TAY [check N, Y] :");
            //   (bytes,  ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'ha8, 8'ha0, 1,  8'hfe, 8'h00, 8'hfe, 8'hff, 1, 2);
            
		 	////////////////////////////////////////////////////////////////////
			// TAY: check Z, X
			////////////////////////////////////////////////////////////////////
            resetState();
            $write("%40s", "TAY [check Z, Y] :");
            //   (bytes,  ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'ha8, 8'h22, 1,  8'h00, 8'h00, 8'h00, 8'hff, 1, 2);
            
		 	////////////////////////////////////////////////////////////////////
			// TAY: check no unreasonable flags
			////////////////////////////////////////////////////////////////////
            resetState();
            setA(8'h42);
            $write("%40s", "TAY [check no extra flags set] :");
            //   (bytes,  ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'ha8, 8'h20, 1,  8'h42, 8'h00, 8'h42, 8'hff, 1, 2);



		 	////////////////////////////////////////////////////////////////////
			// TSX: check N, X
			////////////////////////////////////////////////////////////////////
            $display("");
            resetState();
            $write("%40s", "TSX [check N, X] :");
            //   (bytes,  ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hba, 8'ha0, 1,  8'h00, 8'hff, 8'h00, 8'hff, 1, 2);
            
		 	////////////////////////////////////////////////////////////////////
			// TSX: check Z, X
			////////////////////////////////////////////////////////////////////
            resetState();
            setSP(8'h00);
            $write("%40s", "TSX [check Z, X] :");
            //   (bytes,  ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hba, 8'h22, 1,  8'h00, 8'h00, 8'h00, 8'h00, 1, 2);
            
		 	////////////////////////////////////////////////////////////////////
			// TSX: check no unreasonable flags
			////////////////////////////////////////////////////////////////////
            resetState();
            setSP(8'h42);
            $write("%40s", "TSX [check no extra flags set] :");
            //   (bytes,  ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hba, 8'h20, 1,  8'h00, 8'h42, 8'h00, 8'h42, 1, 2);


		 	////////////////////////////////////////////////////////////////////
			// TXA: check N, A
			////////////////////////////////////////////////////////////////////
            $display("");
            resetState();
            setX(8'hfe);
            $write("%40s", "TXA [check N, X] :");
            //   (bytes,  ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'h8a, 8'ha0, 1,  8'hfe, 8'hfe, 8'h00, 8'hff, 1, 2);
            
		 	////////////////////////////////////////////////////////////////////
			// TXA: check Z, X
			////////////////////////////////////////////////////////////////////
            resetState();
            $write("%40s", "TXA [check Z, X] :");
            //   (bytes,  ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'h8a, 8'h22, 1,  8'h00, 8'h00, 8'h00, 8'hff, 1, 2);
            
		 	////////////////////////////////////////////////////////////////////
			// TXA: check no unreasonable flags
			////////////////////////////////////////////////////////////////////
            resetState();
            setX(8'h42);
            $write("%40s", "TXA [check no extra flags set] :");
            //   (bytes,  ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'h8a, 8'h20, 1,  8'h42, 8'h42, 8'h00, 8'hff, 1, 2);



		 	////////////////////////////////////////////////////////////////////
			// TXS: check N, X
			////////////////////////////////////////////////////////////////////
            $display("");
            resetState();
            setX(8'hfe);
            $write("%40s", "TXS [check N, X] :");
            //   (bytes,  ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'h9a, 8'h20, 1,  8'h00, 8'hfe, 8'h00, 8'hfe, 1, 2);
            
		 	////////////////////////////////////////////////////////////////////
			// TXS: check Z, X
			////////////////////////////////////////////////////////////////////
            resetState();
            $write("%40s", "TXS [check Z, X] :");
            //   (bytes,  ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'h9a, 8'h20, 1,  8'h00, 8'h00, 8'h00, 8'h00, 1, 2);
            
		 	////////////////////////////////////////////////////////////////////
			// TXS: check no unreasonable flags
			////////////////////////////////////////////////////////////////////
            resetState();
            setX(8'h42);
            $write("%40s", "TXS [check no extra flags set] :");
            //   (bytes,  ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'h9a, 8'h20, 1,  8'h00, 8'h42, 8'h00, 8'h42, 1, 2);



		 	////////////////////////////////////////////////////////////////////
			// TYA: check N, A
			////////////////////////////////////////////////////////////////////
            $display("");
            resetState();
            setY(8'hfe);
            $write("%40s", "TYA [check N, X] :");
            //   (bytes,  ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'h98, 8'ha0, 1,  8'hfe, 8'h00, 8'hfe, 8'hff, 1, 2);
            
		 	////////////////////////////////////////////////////////////////////
			// TYA: check Z, X
			////////////////////////////////////////////////////////////////////
            resetState();
            $write("%40s", "TYA [check Z, X] :");
            //   (bytes,  ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'h98, 8'h22, 1,  8'h00, 8'h00, 8'h00, 8'hff, 1, 2);
            
		 	////////////////////////////////////////////////////////////////////
			// TYA: check no unreasonable flags
			////////////////////////////////////////////////////////////////////
            resetState();
            setY(8'h42);
            $write("%40s", "TYA [check no extra flags set] :");
            //   (bytes,  ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'h98, 8'h20, 1,  8'h42, 8'h00, 8'h42, 8'hff, 1, 2);
		end
	endtask

	
	////////////////////////////////////////////////////////////////////////////
	// Test all the jump-style implicit-mode operations
	////////////////////////////////////////////////////////////////////////////
	task testAddrJump;
		begin
			$display("\nTest jump-style implicit addressing:");
						
		 	////////////////////////////////////////////////////////////////////
			// BRK
			////////////////////////////////////////////////////////////////////
            $display("");
            resetState();
            setMem(16'hfffe, 8'h34);
            setMem(16'hffff, 8'h12);
            setPC(16'h5432);
            $write("%40s", "BRK:");
            //      (bytes,  ps,    pc,        a,     x,     y,     sp,    #, ck, addr,    #, val)
            checkMem(48'h00, 8'h30, 16'h1234,  8'h00, 8'h00, 8'h00, 8'hfc, 1, 7, 16'h01fd, 3, 48'h543420);

		 	////////////////////////////////////////////////////////////////////
			// RTI
			////////////////////////////////////////////////////////////////////
            resetState();
            setMem(16'h01ff, 8'h12);		// high byte
            setMem(16'h01fe, 8'h34);		// low byte
            setMem(16'h01fd, 8'h45);		// PS
            setSP(8'hfc);
            $write("%40s", "RTI:");
            //      (bytes,  ps,    pc,        a,     x,     y,     sp,    #, ck, addr,    #, val)
            checkMem(48'h40, 8'h45, 16'h1234,  8'h00, 8'h00, 8'h00, 8'hff, 1, 6, 16'h01fd, 3, 48'h123445);

		 	////////////////////////////////////////////////////////////////////
			// RTS
			////////////////////////////////////////////////////////////////////
            resetState();
            setMem(16'h01ff, 8'h67);		// high byte
            setMem(16'h01fe, 8'h89);		// low byte
            setSP(8'hfd);
            $write("%40s", "RTS:");
            //      (bytes,  ps,    pc,        a,     x,     y,     sp,    #, ck, addr,    #, val)
            checkMem(48'h60, 8'h20, 16'h6789,  8'h00, 8'h00, 8'h00, 8'hff, 1, 6, 16'h01fe, 2, 48'h6789);
		end
	endtask
	
	
	////////////////////////////////////////////////////////////////////////////
	// Test all the immediate-mode operations
	////////////////////////////////////////////////////////////////////////////
	task testAddrImmediate;
		begin
			$display("\nTest immediate-mode addressing:");
						
		 	////////////////////////////////////////////////////////////////////
			// ADC : check no flags
			////////////////////////////////////////////////////////////////////
            $display("");
            resetState();
            $write("%40s", "ADC #5 [binary, check no flags] :");
            //   (bytes,    ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'h0569, 8'h20, 2,  8'h05, 8'h00, 8'h00, 8'hff, 1, 2);	
            				
		 	////////////////////////////////////////////////////////////////////
			// ADC : check V,C
			////////////////////////////////////////////////////////////////////
            resetState();
            setA(8'h80);
            $write("%40s", "ADC #$85 [binary, check V,C] :");
            //   (bytes,    ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'h8569, 8'h61, 2,  8'h05, 8'h00, 8'h00, 8'hff, 1, 2);	
            				
		 	////////////////////////////////////////////////////////////////////
			// ADC : check V,N
			////////////////////////////////////////////////////////////////////
            resetState();
            setA(8'h50);
            $write("%40s", "ADC #$50 [binary, check V,N] :");
            //   (bytes,    ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'h5069, 8'he0, 2,  8'ha0, 8'h00, 8'h00, 8'hff, 1, 2);	
            				
		 	////////////////////////////////////////////////////////////////////
			// ADC : check Z
			////////////////////////////////////////////////////////////////////
            resetState();
            $write("%40s", "ADC #$00 [binary, check V,N] :");
            //   (bytes,    ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'h0069, 8'h22, 2,  8'h00, 8'h00, 8'h00, 8'hff, 1, 2);	
            				

		 	////////////////////////////////////////////////////////////////////
			// ADC : check decimal, no flags
			////////////////////////////////////////////////////////////////////
            $display("");
            resetState();
            setPS(8'h28);
            $write("%40s", "ADC #5 [decimal, check no flags] :");
            //   (bytes,    ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'h0569, 8'h28, 2,  8'h05, 8'h00, 8'h00, 8'hff, 1, 2);	
            				
		 	////////////////////////////////////////////////////////////////////
			// ADC : check decimal, C
			////////////////////////////////////////////////////////////////////
            resetState();
            setPS(8'h28);
            setA(8'h80);
            $write("%40s", "ADC #$85 [decimal, check V,C] :");
            //   (bytes,    ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'h8569, 8'h69, 2,  8'h65, 8'h00, 8'h00, 8'hff, 1, 2);	
            				
		 	////////////////////////////////////////////////////////////////////
			// ADC : check decimal, V,N
			////////////////////////////////////////////////////////////////////
            resetState();
            setPS(8'h28);
            setA(8'h40);
            $write("%40s", "ADC #$40 [decimal, check V,N] :");
            //   (bytes,    ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'h4069, 8'he8, 2,  8'h80, 8'h00, 8'h00, 8'hff, 1, 2);	
            				
		 	////////////////////////////////////////////////////////////////////
			// ADC : check decimal, Z
			////////////////////////////////////////////////////////////////////
            resetState();
            setPS(8'h28);
            $write("%40s", "ADC #$00 [decimal, check Z] :");
            //   (bytes,    ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'h0069, 8'h2a, 2,  8'h00, 8'h00, 8'h00, 8'hff, 1, 2);	
            	
            	
            				
		 	////////////////////////////////////////////////////////////////////
			// AND : check no flags
			////////////////////////////////////////////////////////////////////
            $display("");
            resetState();
            setA(8'h40);
            $write("%40s", "AND #$40 [check no flags] :");
            //   (bytes,    ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'h4029, 8'h20, 2,  8'h40, 8'h00, 8'h00, 8'hff, 1, 2);	
            				
		 	////////////////////////////////////////////////////////////////////
			// AND : check z
			////////////////////////////////////////////////////////////////////
            resetState();
            setA(8'h20);
            $write("%40s", "AND #$20 [check Z] :");
            //   (bytes,    ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'h4029, 8'h22, 2,  8'h00, 8'h00, 8'h00, 8'hff, 1, 2);	
            				
		 	////////////////////////////////////////////////////////////////////
			// AND : check n
			////////////////////////////////////////////////////////////////////
            resetState();
            setA(8'hff);
            $write("%40s", "AND #$80 [check N] :");
            //   (bytes,    ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'h8029, 8'ha0, 2,  8'h80, 8'h00, 8'h00, 8'hff, 1, 2);	
			

		 	////////////////////////////////////////////////////////////////////
			// CMP : check no flags
			////////////////////////////////////////////////////////////////////
            $display("");
            resetState();
            setA(5);
            $write("%40s", "CMP #0 [check C] :");
            //   (bytes,    ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'h00C9, 8'h21, 2,  8'h05, 8'h00, 8'h00, 8'hff, 1, 2);	
            				
		 	////////////////////////////////////////////////////////////////////
			// CMP : check N
			////////////////////////////////////////////////////////////////////
            resetState();
            $write("%40s", "CMP #5 [check N] :");
            //   (bytes,    ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'h05C9, 8'ha0, 2,  8'h00, 8'h00, 8'h00, 8'hff, 1, 2);	
            				
		 	////////////////////////////////////////////////////////////////////
			// CMP : check Z
			////////////////////////////////////////////////////////////////////
            resetState();
            setA(5);
            $write("%40s", "CMP #5 [check Z] :");
            //   (bytes,    ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'h05C9, 8'h23, 2,  8'h05, 8'h00, 8'h00, 8'hff, 1, 2);	
            	
            				

		 	////////////////////////////////////////////////////////////////////
			// CPX : check no flags
			////////////////////////////////////////////////////////////////////
            $display("");
            resetState();
            setX(5);
            $write("%40s", "CPX #0 [check C] :");
            //   (bytes,    ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'h00E0, 8'h21, 2,  8'h00, 8'h05, 8'h00, 8'hff, 1, 2);	
            				
		 	////////////////////////////////////////////////////////////////////
			// CPX : check N
			////////////////////////////////////////////////////////////////////
            resetState();
            $write("%40s", "CPX #5 [check N] :");
            //   (bytes,    ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'h05E0, 8'ha0, 2,  8'h00, 8'h00, 8'h00, 8'hff, 1, 2);	
            				
		 	////////////////////////////////////////////////////////////////////
			// CPX : check Z
			////////////////////////////////////////////////////////////////////
            resetState();
            setX(5);
            $write("%40s", "CPX #5 [check Z] :");
            //   (bytes,    ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'h05E0, 8'h23, 2,  8'h00, 8'h05, 8'h00, 8'hff, 1, 2);	
            	
            				

		 	////////////////////////////////////////////////////////////////////
			// CPY : check no flags
			////////////////////////////////////////////////////////////////////
            $display("");
            resetState();
            setY(5);
            $write("%40s", "CPY #0 [check C] :");
            //   (bytes,    ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'h00C0, 8'h21, 2,  8'h00, 8'h00, 8'h05, 8'hff, 1, 2);	
            				
		 	////////////////////////////////////////////////////////////////////
			// CPY : check N
			////////////////////////////////////////////////////////////////////
            resetState();
            $write("%40s", "CPY #5 [check N] :");
            //   (bytes,    ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'h05C0, 8'ha0, 2,  8'h00, 8'h00, 8'h00, 8'hff, 1, 2);	
            				
		 	////////////////////////////////////////////////////////////////////
			// CPY : check Z
			////////////////////////////////////////////////////////////////////
            resetState();
            setY(5);
            $write("%40s", "CPY #5 [check Z] :");
            //   (bytes,    ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'h05C0, 8'h23, 2,  8'h00, 8'h00, 8'h05, 8'hff, 1, 2);	
             	
            	
            				
		 	////////////////////////////////////////////////////////////////////
			// EOR : check no flags
			////////////////////////////////////////////////////////////////////
            $display("");
            resetState();
            setA(8'h40);
            $write("%40s", "EOR #$40 [check Z] :");
            //   (bytes,    ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'h4049, 8'h22, 2,  8'h00, 8'h00, 8'h00, 8'hff, 1, 2);	
            				
		 	////////////////////////////////////////////////////////////////////
			// EOR : check z
			////////////////////////////////////////////////////////////////////
            resetState();
            setA(8'h20);
            $write("%40s", "EOR #$40 [check no flags] :");
            //   (bytes,    ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'h4049, 8'h20, 2,  8'h60, 8'h00, 8'h00, 8'hff, 1, 2);	
            				
		 	////////////////////////////////////////////////////////////////////
			// EOR : check n
			////////////////////////////////////////////////////////////////////
            resetState();
            setA(8'hff);
            $write("%40s", "EOR #$7F [check N] :");
            //   (bytes,    ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'h7f49, 8'ha0, 2,  8'h80, 8'h00, 8'h00, 8'hff, 1, 2);	
             	
            	
            				
		 	////////////////////////////////////////////////////////////////////
			// LDA : check no flags
			////////////////////////////////////////////////////////////////////
            $display("");
            resetState();
            $write("%40s", "LDA #$40 [check no flags] :");
            //   (bytes,    ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'h40a9, 8'h20, 2,  8'h40, 8'h00, 8'h00, 8'hff, 1, 2);	
            				
		 	////////////////////////////////////////////////////////////////////
			// LDA : check z
			////////////////////////////////////////////////////////////////////
            resetState();
            setA(8'h20);
            $write("%40s", "LDA #$00 [check Z] :");
            //   (bytes,    ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'h00a9, 8'h22, 2,  8'h00, 8'h00, 8'h00, 8'hff, 1, 2);	
            				
		 	////////////////////////////////////////////////////////////////////
			// LDA : check n
			////////////////////////////////////////////////////////////////////
            resetState();
            $write("%40s", "LDA #$80 [check N] :");
            //   (bytes,    ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'h80a9, 8'ha0, 2,  8'h80, 8'h00, 8'h00, 8'hff, 1, 2);	
             	
            	
            				
		 	////////////////////////////////////////////////////////////////////
			// LDX : check no flags
			////////////////////////////////////////////////////////////////////
            $display("");
            resetState();
            $write("%40s", "LDX #$40 [check no flags] :");
            //   (bytes,    ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'h40a2, 8'h20, 2,  8'h00, 8'h40, 8'h00, 8'hff, 1, 2);	
            				
		 	////////////////////////////////////////////////////////////////////
			// LDX : check z
			////////////////////////////////////////////////////////////////////
            resetState();
            setX(8'h20);
            $write("%40s", "LDX #$00 [check Z] :");
            //   (bytes,    ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'h00a2, 8'h22, 2,  8'h00, 8'h00, 8'h00, 8'hff, 1, 2);	
            				
		 	////////////////////////////////////////////////////////////////////
			// LDX : check n
			////////////////////////////////////////////////////////////////////
            resetState();
            $write("%40s", "LDX #$80 [check N] :");
            //   (bytes,    ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'h80a2, 8'ha0, 2,  8'h00, 8'h80, 8'h00, 8'hff, 1, 2);	
             	
            	
            				
		 	////////////////////////////////////////////////////////////////////
			// LDY : check no flags
			////////////////////////////////////////////////////////////////////
            $display("");
            resetState();
            $write("%40s", "LDY #$40 [check no flags] :");
            //   (bytes,    ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'h40a0, 8'h20, 2,  8'h00, 8'h00, 8'h40, 8'hff, 1, 2);	
            				
		 	////////////////////////////////////////////////////////////////////
			// LDY : check z
			////////////////////////////////////////////////////////////////////
            resetState();
            setY(8'h20);
            $write("%40s", "LDY #$00 [check Z] :");
            //   (bytes,    ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'h00a0, 8'h22, 2,  8'h00, 8'h00, 8'h00, 8'hff, 1, 2);	
            				
		 	////////////////////////////////////////////////////////////////////
			// LDY : check n
			////////////////////////////////////////////////////////////////////
            resetState();
            $write("%40s", "LDY #$80 [check N] :");
            //   (bytes,    ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'h80a0, 8'ha0, 2,  8'h00, 8'h00, 8'h80, 8'hff, 1, 2);	
             	
            	
            				
		 	////////////////////////////////////////////////////////////////////
			// ORA : check no flags
			////////////////////////////////////////////////////////////////////
            $display("");
            resetState();
            setA(8'h40);
            $write("%40s", "ORA #$20 [check no flags] :");
            //   (bytes,    ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'h2009, 8'h20, 2,  8'h60, 8'h00, 8'h00, 8'hff, 1, 2);	
            				
		 	////////////////////////////////////////////////////////////////////
			// ORA : check z
			////////////////////////////////////////////////////////////////////
            resetState();
            $write("%40s", "ORA #$00 [check Z] :");
            //   (bytes,    ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'h0009, 8'h22, 2,  8'h00, 8'h00, 8'h00, 8'hff, 1, 2);	
            				
		 	////////////////////////////////////////////////////////////////////
			// ORA : check n
			////////////////////////////////////////////////////////////////////
            resetState();
            setA(8'h7f);
            $write("%40s", "ORA #$80 [check N] :");
            //   (bytes,    ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'h8009, 8'ha0, 2,  8'hff, 8'h00, 8'h00, 8'hff, 1, 2);	
           				
			

		 	////////////////////////////////////////////////////////////////////
			// SBC : check no flags
			////////////////////////////////////////////////////////////////////
            $display("");
            resetState();
            $write("%40s", "SBC #$85 [binary, check no flags] :");
            //   (bytes,    ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'h85e9, 8'h20, 2,  8'h7a, 8'h00, 8'h00, 8'hff, 1, 2);	

		 	////////////////////////////////////////////////////////////////////
			// SBC : check C
			////////////////////////////////////////////////////////////////////
            resetState();
            setA(8'h0b);
            $write("%40s", "SBC #$05 [binary, check C] :");
            //   (bytes,    ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'h05e9, 8'h21, 2,  8'h05, 8'h00, 8'h00, 8'hff, 1, 2);	
         				
		 	////////////////////////////////////////////////////////////////////
			// SBC : check V
			////////////////////////////////////////////////////////////////////
            resetState();
            setA(8'h80);
            $write("%40s", "SBC #$05 [binary, check V] :");
            //   (bytes,    ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'h05e9, 8'h61, 2,  8'h7a, 8'h00, 8'h00, 8'hff, 1, 2);	
           				
		 	////////////////////////////////////////////////////////////////////
			// SBC : check N
			////////////////////////////////////////////////////////////////////
            resetState();
            $write("%40s", "SBC #$00 [binary, check N] :");
            //   (bytes,    ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'h00e9, 8'ha0, 2,  8'hff, 8'h00, 8'h00, 8'hff, 1, 2);	   
                    				
		 	////////////////////////////////////////////////////////////////////
			// SBC : check Z
			////////////////////////////////////////////////////////////////////
            resetState();
            setA(1);
            $write("%40s", "SBC #$00 [binary, check Z] :");
            //   (bytes,    ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'h00e9, 8'h23, 2,  8'h00, 8'h00, 8'h00, 8'hff, 1, 2);	
            				
   
		 	////////////////////////////////////////////////////////////////////
			// SBC :  decimal, check no flags
			////////////////////////////////////////////////////////////////////
            $display("");
            resetState();
            setPS(8'h28);
            $write("%40s", "SBC #$85 [decimal, check no flags] :");
            //   (bytes,    ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'h85e9, 8'h28, 2,  8'h14, 8'h00, 8'h00, 8'hff, 1, 2);	
            				
		 	////////////////////////////////////////////////////////////////////
			// SBC :  decimal, check N
			////////////////////////////////////////////////////////////////////
            resetState();
            resetState();
            setPS(8'h28);
            $write("%40s", "SBC #5 [decimal, check N] :");
            //   (bytes,    ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'h05e9, 8'ha8, 2,  8'h94, 8'h00, 8'h00, 8'hff, 1, 2);	
             				           				
		 	////////////////////////////////////////////////////////////////////
			// SBC :  decimal, Z
			////////////////////////////////////////////////////////////////////
            resetState();
            setPS(8'h28);
            setA(1);
            $write("%40s", "SBC #$00 [decimal, check Z,C] :");
            //   (bytes,    ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'h00e9, 8'h2b, 2,  8'h00, 8'h00, 8'h00, 8'hff, 1, 2);	
				
		 	////////////////////////////////////////////////////////////////////
			// SBC : V flag not valid in decimal mode
			////////////////////////////////////////////////////////////////////
		end
	endtask

	
	
	////////////////////////////////////////////////////////////////////////////
	// Test all the zero-page-mode operations
	////////////////////////////////////////////////////////////////////////////
	task testAddrZp;
		begin
			$display("\nTest zero-page addressing:");
						
		 	////////////////////////////////////////////////////////////////////
			// ADC : check no flags
			////////////////////////////////////////////////////////////////////
            $display("");
            resetState();
            setMem(16'hc0, 8'h42);
            $write("%40s", "ADC $C0 [binary, check no flags] :");
            //   (bytes,    ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hc065, 8'h20, 2,  8'h42, 8'h00, 8'h00, 8'hff, 1, 3);	

            				
		 	////////////////////////////////////////////////////////////////////
			// ADC : check V,C
			////////////////////////////////////////////////////////////////////
            resetState();
            setA(8'h80);
            setMem(16'hc0, 8'h85);
            $write("%40s", "ADC $C0 [binary, check V,C] :");
            //   (bytes,    ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hc065, 8'h61, 2,  8'h05, 8'h00, 8'h00, 8'hff, 1, 3);	
            				
		 	////////////////////////////////////////////////////////////////////
			// ADC : check N
			////////////////////////////////////////////////////////////////////
            resetState();
            setA(8'h50);
            setMem(16'hc0, 8'h85);
            $write("%40s", "ADC $C0 [binary, check V,N] :");
            //   (bytes,    ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hC065, 8'ha0, 2,  8'hd5, 8'h00, 8'h00, 8'hff, 1, 3);	

		 	////////////////////////////////////////////////////////////////////
			// ADC : check Z
			////////////////////////////////////////////////////////////////////
            resetState();
            setMem(16'hc0, 8'h00);
            $write("%40s", "ADC $C0 [binary, check Z] :");
            //   (bytes,    ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hc065, 8'h22, 2,  8'h00, 8'h00, 8'h00, 8'hff, 1, 3);	


		 	////////////////////////////////////////////////////////////////////
			// ADC : check decimal, no flags
			////////////////////////////////////////////////////////////////////
            $display("");
            resetState();
            setMem(16'hc0, 8'h05);
            setPS(8'h28);
            $write("%40s", "ADC $C0 [decimal, check no flags] :");
            //   (bytes,    ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hc065, 8'h28, 2,  8'h05, 8'h00, 8'h00, 8'hff, 1, 3);	
            				
		 	////////////////////////////////////////////////////////////////////
			// ADC : check decimal, C
			////////////////////////////////////////////////////////////////////
            resetState();
            setPS(8'h28);
            setMem(16'hc0, 8'h85);
            setA(8'h80);
            $write("%40s", "ADC $C0 [decimal, check V,C] :");
            //   (bytes,    ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hC065, 8'h69, 2,  8'h65, 8'h00, 8'h00, 8'hff, 1, 3);	
            				
		 	////////////////////////////////////////////////////////////////////
			// ADC : check decimal, V,N
			////////////////////////////////////////////////////////////////////
            resetState();
            setPS(8'h28);
            setMem(16'hc0, 8'h40);
            setA(8'h40);
            $write("%40s", "ADC $C0 [decimal, check V,N] :");
            //   (bytes,    ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hC065, 8'he8, 2,  8'h80, 8'h00, 8'h00, 8'hff, 1, 3);	
            				
		 	////////////////////////////////////////////////////////////////////
			// ADC : check decimal, Z
			////////////////////////////////////////////////////////////////////
            resetState();
            setMem(16'hc0, 8'h00);
            setPS(8'h28);
            $write("%40s", "ADC $C0 [decimal, check Z] :");
            //   (bytes,    ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hC065, 8'h2a, 2,  8'h00, 8'h00, 8'h00, 8'hff, 1, 3);	
            	
            	
            				
		 	////////////////////////////////////////////////////////////////////
			// AND : check no flags
			////////////////////////////////////////////////////////////////////
            $display("");
            resetState();
            setMem(16'hc0, 8'h40);
            setA(8'h40);
            $write("%40s", "AND $C0 [check no flags] :");
            //   (bytes,    ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hC025, 8'h20, 2,  8'h40, 8'h00, 8'h00, 8'hff, 1, 3);	
            				
		 	////////////////////////////////////////////////////////////////////
			// AND : check z
			////////////////////////////////////////////////////////////////////
            resetState();
            setA(8'h20);
            setMem(16'hc0, 8'h40);
            $write("%40s", "AND $C0 [check Z] :");
            //   (bytes,    ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hC025, 8'h22, 2,  8'h00, 8'h00, 8'h00, 8'hff, 1, 3);	
            				
		 	////////////////////////////////////////////////////////////////////
			// AND : check n
			////////////////////////////////////////////////////////////////////
            resetState();
            setA(8'hff);
            setMem(16'hc0, 8'h80);
            $write("%40s", "AND $C0 [check N] :");
            //   (bytes,    ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hC025, 8'ha0, 2,  8'h80, 8'h00, 8'h00, 8'hff, 1, 3);	
		
			

		 	////////////////////////////////////////////////////////////////////
			// ASL : check no flags
			////////////////////////////////////////////////////////////////////
            $display("");
            resetState();
            setMem(16'hc0, 8'h02);
            $write("%40s", "ASL $C0 [check no flags] :");
            //      (bytes,    ps,    pc, a,     x,     y,     sp,    #, ck, addr,  #, val)
            checkMem(48'hc006, 8'h20, 2,  8'h00, 8'h00, 8'h00, 8'hff, 1, 5, 16'hc0, 1, 48'h04);

		 	////////////////////////////////////////////////////////////////////
			// ASL : check N
			////////////////////////////////////////////////////////////////////
            resetState();
            setMem(16'hc0, 8'h40);
            $write("%40s", "ASL $C0 [check N] :");
            //      (bytes,    ps,    pc, a,     x,     y,     sp,    #, ck, addr,  #, val)
            checkMem(48'hc006, 8'ha0, 2,  8'h00, 8'h00, 8'h00, 8'hff, 1, 5, 16'hc0, 1, 48'h80);

		 	////////////////////////////////////////////////////////////////////
			// ASL : check C, Z
			////////////////////////////////////////////////////////////////////
            resetState();
            setMem(16'hc0, 8'h80);
            $write("%40s", "ASL $C0 [check C, Z] :");
            //      (bytes,    ps,    pc, a,     x,     y,     sp,    #, ck, addr,  #, val)
            checkMem(48'hc006, 8'h23, 2,  8'h00, 8'h00, 8'h00, 8'hff, 1, 5, 16'hc0, 1, 48'h00);



		 	////////////////////////////////////////////////////////////////////
			// BIT : check no flags
			////////////////////////////////////////////////////////////////////
            $display("");
            resetState();
            setMem(16'hc0, 8'h20);
            setA(8'hff);
            $write("%40s", "BIT $C0 [check no flags] :");
            //   (bytes,    ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hC024, 8'h20, 2,  8'hff, 8'h00, 8'h00, 8'hff, 1, 3);	
            				
		 	////////////////////////////////////////////////////////////////////
			// BIT : check z
			////////////////////////////////////////////////////////////////////
            resetState();
            setA(8'h10);
            setMem(16'hc0, 8'h20);
            $write("%40s", "BIT $C0 [check Z] :");
            //   (bytes,    ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hC024, 8'h22, 2,  8'h10, 8'h00, 8'h00, 8'hff, 1, 3);	
            				
		 	////////////////////////////////////////////////////////////////////
			// BIT : check n
			////////////////////////////////////////////////////////////////////
            resetState();
            setA(8'hff);
            setMem(16'hc0, 8'h80);
            $write("%40s", "BIT $C0 [check N] :");
            //   (bytes,    ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hC024, 8'ha0, 2,  8'hff, 8'h00, 8'h00, 8'hff, 1, 3);	
            				
		 	////////////////////////////////////////////////////////////////////
			// BIT : check v
			////////////////////////////////////////////////////////////////////
            resetState();
            setA(8'hff);
            setMem(16'hc0, 8'hc0);
            $write("%40s", "BIT $C0 [check V] :");
            //   (bytes,    ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hC024, 8'he0, 2,  8'hff, 8'h00, 8'h00, 8'hff, 1, 3);	
			


		 	////////////////////////////////////////////////////////////////////
			// CMP : check no flags
			////////////////////////////////////////////////////////////////////
            $display("");
            resetState();
            setMem(16'hc0, 8'h00);
            setA(5);
            $write("%40s", "CMP $C0 [check C] :");
            //   (bytes,    ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hC0C5, 8'h21, 2,  8'h05, 8'h00, 8'h00, 8'hff, 1, 3);	
            				
		 	////////////////////////////////////////////////////////////////////
			// CMP : check N
			////////////////////////////////////////////////////////////////////
            resetState();
            setMem(16'hc0, 8'h05);
            $write("%40s", "CMP $C0 [check N] :");
            //   (bytes,    ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hC0C5, 8'ha0, 2,  8'h00, 8'h00, 8'h00, 8'hff, 1, 3);	
            				
		 	////////////////////////////////////////////////////////////////////
			// CMP : check Z
			////////////////////////////////////////////////////////////////////
            resetState();
            setMem(16'hc0, 8'h05);
            setA(5);
            $write("%40s", "CMP $C0 [check Z] :");
            //   (bytes,    ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hC0C5, 8'h23, 2,  8'h05, 8'h00, 8'h00, 8'hff, 1, 3);	
            	
            				
            				
		 	////////////////////////////////////////////////////////////////////
			// CPX : check no flags
			////////////////////////////////////////////////////////////////////
            $display("");
            resetState();
            setMem(16'hc0, 8'h00);
            setX(5);
            $write("%40s", "CPX $C0 [check C] :");
            //   (bytes,    ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hC0E4, 8'h21, 2,  8'h00, 8'h05, 8'h00, 8'hff, 1, 3);	
            				
		 	////////////////////////////////////////////////////////////////////
			// CPX : check N
			////////////////////////////////////////////////////////////////////
            resetState();
            setMem(16'hc0, 8'h05);
            $write("%40s", "CPX $C0 [check N] :");
            //   (bytes,    ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hC0E4, 8'ha0, 2,  8'h00, 8'h00, 8'h00, 8'hff, 1, 3);	
            				
		 	////////////////////////////////////////////////////////////////////
			// CPX : check Z
			////////////////////////////////////////////////////////////////////
            resetState();
            setMem(16'hc0, 8'h05);
            setX(5);
            $write("%40s", "CPX $C0 [check Z] :");
            //   (bytes,    ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hC0E4, 8'h23, 2,  8'h00, 8'h05, 8'h00, 8'hff, 1, 3);	
            	
            				

		 	////////////////////////////////////////////////////////////////////
			// CPY : check no flags
			////////////////////////////////////////////////////////////////////
            $display("");
            resetState();
            setMem(16'hc0, 8'h00);
            setY(5);
            $write("%40s", "CPY $C0 [check C] :");
            //   (bytes,    ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hC0C4, 8'h21, 2,  8'h00, 8'h00, 8'h05, 8'hff, 1, 3);	
            				
		 	////////////////////////////////////////////////////////////////////
			// CPY : check N
			////////////////////////////////////////////////////////////////////
            resetState();
            setMem(16'hc0, 8'h05);
            $write("%40s", "CPY $C0 [check N] :");
            //   (bytes,    ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hC0C4, 8'ha0, 2,  8'h00, 8'h00, 8'h00, 8'hff, 1, 3);	
            				
		 	////////////////////////////////////////////////////////////////////
			// CPY : check Z
			////////////////////////////////////////////////////////////////////
            resetState();
            setMem(16'hc0, 8'h05);
            setY(5);
            $write("%40s", "CPY $C0 [check Z] :");
            //   (bytes,    ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hC0C4, 8'h23, 2,  8'h00, 8'h00, 8'h05, 8'hff, 1, 3);	
             	


		 	////////////////////////////////////////////////////////////////////
			// DEC : check no flags
			////////////////////////////////////////////////////////////////////
            $display("");
            resetState();
            setMem(16'hc0, 8'h42);
            $write("%40s", "DEC $C0 [check no flags] :");
            //      (bytes,    ps,    pc, a,     x,     y,     sp,    #, ck, addr,   #, val)
            checkMem(48'hC0C6, 8'h20, 2,  8'h00, 8'h00, 8'h00, 8'hff, 1, 5,  16'hc0, 1, 48'h41);

		 	////////////////////////////////////////////////////////////////////
			// DEC : check Z
			////////////////////////////////////////////////////////////////////
            resetState();
            setMem(16'hc0, 8'h1);
            $write("%40s", "DEC $C0 [check Z] :");
            //      (bytes,    ps,    pc, a,     x,     y,     sp,    #, ck, addr,   #, val)
            checkMem(48'hC0C6, 8'h22, 2,  8'h00, 8'h00, 8'h00, 8'hff, 1, 5,  16'hc0, 1, 48'h00);

		 	////////////////////////////////////////////////////////////////////
			// DEC : check N
			////////////////////////////////////////////////////////////////////
            resetState();
            setMem(16'hc0, 8'hff);
            $write("%40s", "DEC $C0 [check C] :");
            //      (bytes,    ps,    pc, a,     x,     y,     sp,    #, ck, addr,   #, val)
            checkMem(48'hC0C6, 8'ha0, 2,  8'h00, 8'h00, 8'h00, 8'hff, 1, 5,  16'hc0, 1, 48'hfe);
            				
            	
            				
		 	////////////////////////////////////////////////////////////////////
			// EOR : check Z
			////////////////////////////////////////////////////////////////////
            $display("");
            resetState();
            setMem(16'hc0, 8'h40);
            setA(8'h40);
            $write("%40s", "EOR $C0 [check Z] :");
            //   (bytes,    ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hC045, 8'h22, 2,  8'h00, 8'h00, 8'h00, 8'hff, 1, 3);	
            				
		 	////////////////////////////////////////////////////////////////////
			// EOR : check no flags
			////////////////////////////////////////////////////////////////////
            resetState();
            setMem(16'hc0, 8'h40);
            setA(8'h20);
            $write("%40s", "EOR $C0 [check no flags] :");
            //   (bytes,    ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hC045, 8'h20, 2,  8'h60, 8'h00, 8'h00, 8'hff, 1, 3);	
            				
		 	////////////////////////////////////////////////////////////////////
			// EOR : check n
			////////////////////////////////////////////////////////////////////
            resetState();
            setMem(16'hc0, 8'h7f);
            setA(8'hff);
            $write("%40s", "EOR $C0 [check N] :");
            //   (bytes,    ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hC045, 8'ha0, 2,  8'h80, 8'h00, 8'h00, 8'hff, 1, 3);	
             	


		 	////////////////////////////////////////////////////////////////////
			// INC : check no flags
			////////////////////////////////////////////////////////////////////
            $display("");
            resetState();
            setMem(16'hc0, 8'h42);
            $write("%40s", "INC $C0 [check no flags] :");
            //      (bytes,    ps,    pc, a,     x,     y,     sp,    #, ck, addr,   #, val)
            checkMem(48'hC0E6, 8'h20, 2,  8'h00, 8'h00, 8'h00, 8'hff, 1, 5,  16'hc0, 1, 48'h43);

		 	////////////////////////////////////////////////////////////////////
			// INC : check Z
			////////////////////////////////////////////////////////////////////
            resetState();
            setMem(16'hc0, 8'hff);
            $write("%40s", "INC $C0 [check Z] :");
            //      (bytes,    ps,    pc, a,     x,     y,     sp,    #, ck, addr,   #, val)
            checkMem(48'hC0E6, 8'h22, 2,  8'h00, 8'h00, 8'h00, 8'hff, 1, 5,  16'hc0, 1, 48'h00);

		 	////////////////////////////////////////////////////////////////////
			// INC : check N
			////////////////////////////////////////////////////////////////////
            resetState();
            setMem(16'hc0, 8'h7f);
            $write("%40s", "INC $C0 [check C] :");
            //      (bytes,    ps,    pc, a,     x,     y,     sp,    #, ck, addr,   #, val)
            checkMem(48'hC0E6, 8'ha0, 2,  8'h00, 8'h00, 8'h00, 8'hff, 1, 5,  16'hc0, 1, 48'h80);

            	
            				
		 	////////////////////////////////////////////////////////////////////
			// LDA : check no flags
			////////////////////////////////////////////////////////////////////
            $display("");
            resetState();
            setMem(16'hc0, 8'h40);
            $write("%40s", "LDA $C0 [check no flags] :");
            //   (bytes,    ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hC0A5, 8'h20, 2,  8'h40, 8'h00, 8'h00, 8'hff, 1, 3);	
            				
		 	////////////////////////////////////////////////////////////////////
			// LDA : check z
			////////////////////////////////////////////////////////////////////
            resetState();
            setA(8'h20);
            setMem(16'hc0, 8'h00);
            $write("%40s", "LDA $C0 [check Z] :");
            //   (bytes,    ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hC0A5, 8'h22, 2,  8'h00, 8'h00, 8'h00, 8'hff, 1, 3);	
            				
		 	////////////////////////////////////////////////////////////////////
			// LDA : check n
			////////////////////////////////////////////////////////////////////
            resetState();
            setMem(16'hc0, 8'h80);
            $write("%40s", "LDA $C0 [check N] :");
            //   (bytes,    ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hC0A5, 8'ha0, 2,  8'h80, 8'h00, 8'h00, 8'hff, 1, 3);	
             	
            	
            				
		 	////////////////////////////////////////////////////////////////////
			// LDX : check no flags
			////////////////////////////////////////////////////////////////////
            $display("");
            resetState();
            setMem(16'hc0, 8'h40);
            $write("%40s", "LDX $C0 [check no flags] :");
            //   (bytes,    ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hC0A6, 8'h20, 2,  8'h00, 8'h40, 8'h00, 8'hff, 1, 3);	
            				
		 	////////////////////////////////////////////////////////////////////
			// LDX : check z
			////////////////////////////////////////////////////////////////////
            resetState();
            setMem(16'hc0, 8'h00);
            setX(8'h20);
            $write("%40s", "LDX $C0 [check Z] :");
            //   (bytes,    ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hC0A6, 8'h22, 2,  8'h00, 8'h00, 8'h00, 8'hff, 1, 3);	
            				
		 	////////////////////////////////////////////////////////////////////
			// LDX : check n
			////////////////////////////////////////////////////////////////////
            resetState();
            setMem(16'hc0, 8'h80);
            $write("%40s", "LDX $C0 [check N] :");
            //   (bytes,    ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hC0A6, 8'ha0, 2,  8'h00, 8'h80, 8'h00, 8'hff, 1, 3);	
             	
            	
            				
		 	////////////////////////////////////////////////////////////////////
			// LDY : check no flags
			////////////////////////////////////////////////////////////////////
            $display("");
            resetState();
            setMem(16'hc0, 8'h40);
            $write("%40s", "LDY #$40 [check no flags] :");
            //   (bytes,    ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hC0A4, 8'h20, 2,  8'h00, 8'h00, 8'h40, 8'hff, 1, 3);	
            				
		 	////////////////////////////////////////////////////////////////////
			// LDY : check z
			////////////////////////////////////////////////////////////////////
            resetState();
            setMem(16'hc0, 8'h00);
            setY(8'h20);
            $write("%40s", "LDY #$00 [check Z] :");
            //   (bytes,    ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hC0A4, 8'h22, 2,  8'h00, 8'h00, 8'h00, 8'hff, 1, 3);	
            				
		 	////////////////////////////////////////////////////////////////////
			// LDY : check n
			////////////////////////////////////////////////////////////////////
            resetState();
            setMem(16'hc0, 8'h80);
            $write("%40s", "LDY #$80 [check N] :");
            //   (bytes,    ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hC0A4, 8'ha0, 2,  8'h00, 8'h00, 8'h80, 8'hff, 1, 3);	
             	


		 	////////////////////////////////////////////////////////////////////
			// LSR : check no flags
			////////////////////////////////////////////////////////////////////
            $display("");
            resetState();
            setMem(16'hc0, 8'h02);
            $write("%40s", "LSR $C0 [check no flags] :");
            //      (bytes,    ps,    pc, a,     x,     y,     sp,    #, ck, addr,  #, val)
            checkMem(48'hc046, 8'h20, 2,  8'h00, 8'h00, 8'h00, 8'hff, 1, 5, 16'hc0, 1, 48'h01);

		 	////////////////////////////////////////////////////////////////////
			// LSR : check C, Z
			////////////////////////////////////////////////////////////////////
            resetState();
            setMem(16'hc0, 8'h1);
            $write("%40s", "LSR $C0 [check C, Z] :");
            //      (bytes,    ps,    pc, a,     x,     y,     sp,    #, ck, addr,  #, val)
            checkMem(48'hc046, 8'h23, 2,  8'h00, 8'h00, 8'h00, 8'hff, 1, 5, 16'hc0, 1, 48'h00);

            	
            				
		 	////////////////////////////////////////////////////////////////////
			// ORA : check no flags
			////////////////////////////////////////////////////////////////////
            $display("");
            resetState();
            setMem(16'hc0, 8'h20);
            setA(8'h40);
            $write("%40s", "ORA $C0 [check no flags] :");
            //   (bytes,    ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hC005, 8'h20, 2,  8'h60, 8'h00, 8'h00, 8'hff, 1, 3);	
            				
		 	////////////////////////////////////////////////////////////////////
			// ORA : check z
			////////////////////////////////////////////////////////////////////
            resetState();
            setMem(16'hc0, 8'h00);
            $write("%40s", "ORA $C0 [check Z] :");
            //   (bytes,    ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hC005, 8'h22, 2,  8'h00, 8'h00, 8'h00, 8'hff, 1, 3);	
            				
		 	////////////////////////////////////////////////////////////////////
			// ORA : check n
			////////////////////////////////////////////////////////////////////
            resetState();
            setMem(16'hc0, 8'h80);
            setA(8'h7f);
            $write("%40s", "ORA $C0 [check N] :");
            //   (bytes,    ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hC005, 8'ha0, 2,  8'hff, 8'h00, 8'h00, 8'hff, 1, 3);	            				
           				


		 	////////////////////////////////////////////////////////////////////
			// ROL : check no flags
			////////////////////////////////////////////////////////////////////
            $display("");
            resetState();
            setMem(16'hc0, 8'h02);
            $write("%40s", "ROL $C0 [check no flags] :");
            //      (bytes,    ps,    pc, a,     x,     y,     sp,    #, ck, addr,  #, val)
            checkMem(48'hc026, 8'h20, 2,  8'h00, 8'h00, 8'h00, 8'hff, 1, 5, 16'hc0, 1, 48'h04);

		 	////////////////////////////////////////////////////////////////////
			// ROL : check N, C
			////////////////////////////////////////////////////////////////////
            resetState();
            setMem(16'hc0, 8'hc0);
            $write("%40s", "ROL $C0 [check N, C] :");
            //      (bytes,    ps,    pc, a,     x,     y,     sp,    #, ck, addr,  #, val)
            checkMem(48'hc026, 8'ha1, 2,  8'h00, 8'h00, 8'h00, 8'hff, 1, 5, 16'hc0, 1, 48'h80);

		 	////////////////////////////////////////////////////////////////////
			// ROL : check C, Z
			////////////////////////////////////////////////////////////////////
            resetState();
            setMem(16'hc0, 8'h80);
            $write("%40s", "ROL $C0 [check C, Z] :");
            //      (bytes,    ps,    pc, a,     x,     y,     sp,    #, ck, addr,  #, val)
            checkMem(48'hc026, 8'h23, 2,  8'h00, 8'h00, 8'h00, 8'hff, 1, 5, 16'hc0, 1, 48'h00);



		 	////////////////////////////////////////////////////////////////////
			// ROR : check no flags
			////////////////////////////////////////////////////////////////////
            $display("");
            resetState();
            setMem(16'hc0, 8'h02);
            $write("%40s", "ROR $C0 [check no flags] :");
            //      (bytes,    ps,    pc, a,     x,     y,     sp,    #, ck, addr,  #, val)
            checkMem(48'hc066, 8'h20, 2,  8'h00, 8'h00, 8'h00, 8'hff, 1, 5, 16'hc0, 1, 48'h01);

		 	////////////////////////////////////////////////////////////////////
			// ROR : check N, C
			////////////////////////////////////////////////////////////////////
            resetState();
            setPS(8'h21);
            setMem(16'hc0, 8'hc0);
            $write("%40s", "ROR $C0 [check N, C] :");
            //      (bytes,    ps,    pc, a,     x,     y,     sp,    #, ck, addr,  #, val)
            checkMem(48'hc066, 8'ha0, 2,  8'h00, 8'h00, 8'h00, 8'hff, 1, 5, 16'hc0, 1, 48'he0);

		 	////////////////////////////////////////////////////////////////////
			// ROR : check C, Z
			////////////////////////////////////////////////////////////////////
            resetState();
            setMem(16'hc0, 8'h1);
            $write("%40s", "ROR $C0 [check C, Z] :");
            //      (bytes,    ps,    pc, a,     x,     y,     sp,    #, ck, addr,  #, val)
            checkMem(48'hc066, 8'h23, 2,  8'h00, 8'h00, 8'h00, 8'hff, 1, 5, 16'hc0, 1, 48'h00);

			

		 	////////////////////////////////////////////////////////////////////
			// SBC : check no flags
			////////////////////////////////////////////////////////////////////
            $display("");
            resetState();
            setMem(16'hc0, 8'h85);
            $write("%40s", "SBC $C0 [binary, check no flags] :");
            //   (bytes,    ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hC0E5, 8'h20, 2,  8'h7a, 8'h00, 8'h00, 8'hff, 1, 3);	

		 	////////////////////////////////////////////////////////////////////
			// SBC : check C
			////////////////////////////////////////////////////////////////////
            resetState();
            setA(8'h0b);
            setMem(16'hc0, 8'h05);
            $write("%40s", "SBC $C0 [binary, check C] :");
            //   (bytes,    ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hC0E5, 8'h21, 2,  8'h05, 8'h00, 8'h00, 8'hff, 1, 3);	
         				
		 	////////////////////////////////////////////////////////////////////
			// SBC : check V
			////////////////////////////////////////////////////////////////////
            resetState();
            setMem(16'hc0, 8'h05);
            setA(8'h80);
            $write("%40s", "SBC $C0 [binary, check V] :");
            //   (bytes,    ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hC0E5, 8'h61, 2,  8'h7a, 8'h00, 8'h00, 8'hff, 1, 3);	
           				
		 	////////////////////////////////////////////////////////////////////
			// SBC : check N
			////////////////////////////////////////////////////////////////////
            resetState();
            setMem(16'hc0, 8'h00);
            $write("%40s", "SBC $C0 [binary, check N] :");
            //   (bytes,    ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hC0E5, 8'ha0, 2,  8'hff, 8'h00, 8'h00, 8'hff, 1, 3);	   
                    				
		 	////////////////////////////////////////////////////////////////////
			// SBC : check Z
			////////////////////////////////////////////////////////////////////
            resetState();
            setMem(16'hc0, 8'h00);
            setA(1);
            $write("%40s", "SBC $C0 [binary, check Z] :");
            //   (bytes,    ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hC0E5, 8'h23, 2,  8'h00, 8'h00, 8'h00, 8'hff, 1, 3);            				
   
   
		 	////////////////////////////////////////////////////////////////////
			// SBC :  decimal, check no flags
			////////////////////////////////////////////////////////////////////
            $display("");
            resetState();
            setMem(16'hc0, 8'h85);
            setPS(8'h28);
            $write("%40s", "SBC $C0 [decimal, check no flags] :");
            //   (bytes,    ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hC0E5, 8'h28, 2,  8'h14, 8'h00, 8'h00, 8'hff, 1, 3);	
            				
		 	////////////////////////////////////////////////////////////////////
			// SBC :  decimal, check N
			////////////////////////////////////////////////////////////////////
            resetState();
            resetState();
            setPS(8'h28);
            setMem(16'hc0, 8'h05);
            $write("%40s", "SBC $C0 [decimal, check N] :");
            //   (bytes,    ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hC0E5, 8'ha8, 2,  8'h94, 8'h00, 8'h00, 8'hff, 1, 3);	
             				           				
		 	////////////////////////////////////////////////////////////////////
			// SBC :  decimal, Z
			////////////////////////////////////////////////////////////////////
            resetState();
            setPS(8'h28);
            setMem(16'hc0, 8'h00);
            setA(1);
            $write("%40s", "SBC $C0 [decimal, check Z,C] :");
            //   (bytes,    ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hC0E5, 8'h2b, 2,  8'h00, 8'h00, 8'h00, 8'hff, 1, 3);	
				
		 	////////////////////////////////////////////////////////////////////
			// SBC : V flag not valid in decimal mode
			////////////////////////////////////////////////////////////////////

            				
		 	////////////////////////////////////////////////////////////////////
			// STA : check memory
			////////////////////////////////////////////////////////////////////
            $display("");
            resetState();
            setMem(16'hc0, 8'h20);
            setA(8'h69);
            $write("%40s", "STA $C0 [check memory] :");
            //      (bytes,    ps,    pc, a,     x,     y,     sp,    #, ck, addr,   #, val)
            checkMem(48'hC085, 8'h20, 2,  8'h69, 8'h00, 8'h00, 8'hff, 1, 3,  16'hc0, 1, 48'h69);
            				
		 	////////////////////////////////////////////////////////////////////
			// STX : check memory
			////////////////////////////////////////////////////////////////////
            resetState();
            setMem(16'hc0, 8'h20);
            setX(8'h70);
            $write("%40s", "STX $C0 [check memory] :");
            //      (bytes,    ps,    pc, a,     x,     y,     sp,    #, ck, addr,   #, val)
            checkMem(48'hC086, 8'h20, 2,  8'h00, 8'h70, 8'h00, 8'hff, 1, 3,  16'hc0, 1, 48'h70);
            				
		 	////////////////////////////////////////////////////////////////////
			// STY : check memory
			////////////////////////////////////////////////////////////////////
            resetState();
            setMem(16'hc0, 8'h20);
            setY(8'h71);
            $write("%40s", "STY $C0 [check memory] :");
            //      (bytes,    ps,    pc, a,     x,     y,     sp,    #, ck, addr,   #, val)
            checkMem(48'hC084, 8'h20, 2,  8'h00, 8'h00, 8'h71, 8'hff, 1, 3,  16'hc0, 1, 48'h71);
		end
	endtask

	
	
	////////////////////////////////////////////////////////////////////////////
	// Test all the zero-page-mode operations
	////////////////////////////////////////////////////////////////////////////
	task testAddrZpX;
		begin
			$display("\nTest zero-page-indexed-with-X|Y addressing:");
						
		 	////////////////////////////////////////////////////////////////////
			// ADC : check no flags
			////////////////////////////////////////////////////////////////////
            $display("");
            resetState();
            setMem(16'hc4, 8'h42);
            setX(4);
            $write("%40s", "ADC $C0,X [binary, check no flags] :");
            //   (bytes,    ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hc075, 8'h20, 2,  8'h42, 8'h04, 8'h00, 8'hff, 1, 4);	

          				
		 	////////////////////////////////////////////////////////////////////
			// ADC : check V,C
			////////////////////////////////////////////////////////////////////
            resetState();
            setA(8'h80);
            setX(4);
            setMem(16'hc4, 8'h85);
            $write("%40s", "ADC $C0,X [binary, check V,C] :");
            //   (bytes,    ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hc075, 8'h61, 2,  8'h05, 8'h04, 8'h00, 8'hff, 1, 4);	
             				
		 	////////////////////////////////////////////////////////////////////
			// ADC : check N
			////////////////////////////////////////////////////////////////////
            resetState();
            setA(8'h50);
            setX(8'h60);
            setMem(16'h20, 8'h85);
            $write("%40s", "ADC $C0,X [binary, check V,N] :");
            //   (bytes,    ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hC075, 8'ha0, 2,  8'hd5, 8'h60, 8'h00, 8'hff, 1, 4);	

		 	////////////////////////////////////////////////////////////////////
			// ADC : check Z
			////////////////////////////////////////////////////////////////////
            resetState();
            setX(4);
            setMem(16'hc4, 8'h00);
            $write("%40s", "ADC $C0,X [binary, check Z] :");
            //   (bytes,    ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hc075, 8'h22, 2,  8'h00, 8'h04, 8'h00, 8'hff, 1, 4);	


		 	////////////////////////////////////////////////////////////////////
			// ADC : check decimal, no flags
			////////////////////////////////////////////////////////////////////
            $display("");
            resetState();
            setMem(16'hc4, 8'h05);
            setX(4);
            setPS(8'h28);
            $write("%40s", "ADC $C0,X [decimal, check no flags] :");
            //   (bytes,    ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hc075, 8'h28, 2,  8'h05, 8'h04, 8'h00, 8'hff, 1, 4);	
            				
		 	////////////////////////////////////////////////////////////////////
			// ADC : check decimal, C
			////////////////////////////////////////////////////////////////////
            resetState();
            setPS(8'h28);
            setMem(16'hc4, 8'h85);
            setX(4);
            setA(8'h80);
            $write("%40s", "ADC $C0,X [decimal, check V,C] :");
            //   (bytes,    ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hC075, 8'h69, 2,  8'h65, 8'h04, 8'h00, 8'hff, 1, 4);	
            				
		 	////////////////////////////////////////////////////////////////////
			// ADC : check decimal, V,N
			////////////////////////////////////////////////////////////////////
            resetState();
            setPS(8'h28);
            setMem(16'hc4, 8'h40);
            setX(4);
            setA(8'h40);
            $write("%40s", "ADC $C0,X [decimal, check V,N] :");
            //   (bytes,    ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hC075, 8'he8, 2,  8'h80, 8'h04, 8'h00, 8'hff, 1, 4);	
            				
		 	////////////////////////////////////////////////////////////////////
			// ADC : check decimal, Z
			////////////////////////////////////////////////////////////////////
            resetState();
            setMem(16'hc4, 8'h00);
            setX(4);
            setPS(8'h28);
            $write("%40s", "ADC $C0,X [decimal, check Z] :");
            //   (bytes,    ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hC075, 8'h2a, 2,  8'h00, 8'h04, 8'h00, 8'hff, 1, 4);	
            	
            	
           				
		 	////////////////////////////////////////////////////////////////////
			// AND : check no flags
			////////////////////////////////////////////////////////////////////
            $display("");
            resetState();
            setMem(16'hc4, 8'h40);
            setX(4);
            setA(8'h40);
            $write("%40s", "AND $C0,X [check no flags] :");
            //   (bytes,    ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hC035, 8'h20, 2,  8'h40, 8'h04, 8'h00, 8'hff, 1, 4);	
            				
		 	////////////////////////////////////////////////////////////////////
			// AND : check z
			////////////////////////////////////////////////////////////////////
            resetState();
            setA(8'h20);
            setMem(16'hc4, 8'h40);
            setX(4);
            $write("%40s", "AND $C0,X [check Z] :");
            //   (bytes,    ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hC035, 8'h22, 2,  8'h00, 8'h04, 8'h00, 8'hff, 1, 4);	
            				
		 	////////////////////////////////////////////////////////////////////
			// AND : check n
			////////////////////////////////////////////////////////////////////
            resetState();
            setA(8'hff);
            setMem(16'hc4, 8'h80);
            setX(4);
            $write("%40s", "AND $C0,X [check N] :");
            //   (bytes,    ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hC035, 8'ha0, 2,  8'h80, 8'h04, 8'h00, 8'hff, 1, 4);	
		
			

		 	////////////////////////////////////////////////////////////////////
			// ASL : check no flags
			////////////////////////////////////////////////////////////////////
            $display("");
            resetState();
            setMem(16'hc4, 8'h02);
            setX(4);
            $write("%40s", "ASL $C0,X [check no flags] :");
            //      (bytes,    ps,    pc, a,     x,     y,     sp,    #, ck, addr,  #, val)
            checkMem(48'hc016, 8'h20, 2,  8'h00, 8'h04, 8'h00, 8'hff, 1, 6, 16'hc4, 1, 48'h04);

		 	////////////////////////////////////////////////////////////////////
			// ASL : check N
			////////////////////////////////////////////////////////////////////
            resetState();
            setMem(16'hc4, 8'h40);
            setX(4);
            $write("%40s", "ASL $C0,X [check N] :");
            //      (bytes,    ps,    pc, a,     x,     y,     sp,    #, ck, addr,  #, val)
            checkMem(48'hc016, 8'ha0, 2,  8'h00, 8'h04, 8'h00, 8'hff, 1, 6, 16'hc4, 1, 48'h80);

		 	////////////////////////////////////////////////////////////////////
			// ASL : check C, Z
			////////////////////////////////////////////////////////////////////
            resetState();
            setMem(16'hc4, 8'h80);
            setX(4);
            $write("%40s", "ASL $C0,X [check C, Z] :");
            //      (bytes,    ps,    pc, a,     x,     y,     sp,    #, ck, addr,  #, val)
            checkMem(48'hc016, 8'h23, 2,  8'h00, 8'h04, 8'h00, 8'hff, 1, 6, 16'hc4, 1, 48'h00);
			


		 	////////////////////////////////////////////////////////////////////
			// CMP : check no flags
			////////////////////////////////////////////////////////////////////
            $display("");
            resetState();
            setMem(16'hc4, 8'h00);
            setX(4);
            setA(5);
            $write("%40s", "CMP $C0,X [check C] :");
            //   (bytes,    ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hC0D5, 8'h21, 2,  8'h05, 8'h04, 8'h00, 8'hff, 1, 4);	
            				
		 	////////////////////////////////////////////////////////////////////
			// CMP : check N
			////////////////////////////////////////////////////////////////////
            resetState();
            setMem(16'hc4, 8'h05);
            setX(4);
            $write("%40s", "CMP $C0,X [check N] :");
            //   (bytes,    ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hC0D5, 8'ha0, 2,  8'h00, 8'h04, 8'h00, 8'hff, 1, 4);	
            				
		 	////////////////////////////////////////////////////////////////////
			// CMP : check Z
			////////////////////////////////////////////////////////////////////
            resetState();
            setMem(16'hc4, 8'h05);
            setX(4);
            setA(5);
            $write("%40s", "CMP $C0,X [check Z] :");
            //   (bytes,    ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hC0D5, 8'h23, 2,  8'h05, 8'h04, 8'h00, 8'hff, 1, 4);	
             	


		 	////////////////////////////////////////////////////////////////////
			// DEC : check no flags
			////////////////////////////////////////////////////////////////////
            $display("");
            resetState();
            setMem(16'hc4, 8'h42);
            setX(4);
            $write("%40s", "DEC $C0,X [check no flags] :");
            //      (bytes,    ps,    pc, a,     x,     y,     sp,    #, ck, addr,   #, val)
            checkMem(48'hC0D6, 8'h20, 2,  8'h00, 8'h04, 8'h00, 8'hff, 1, 6,  16'hc4, 1, 48'h41);

		 	////////////////////////////////////////////////////////////////////
			// DEC : check Z
			////////////////////////////////////////////////////////////////////
            resetState();
            setMem(16'hc4, 8'h1);
            setX(4);
            $write("%40s", "DEC $C0,X [check Z] :");
            //      (bytes,    ps,    pc, a,     x,     y,     sp,    #, ck, addr,   #, val)
            checkMem(48'hC0D6, 8'h22, 2,  8'h00, 8'h04, 8'h00, 8'hff, 1, 6,  16'hc4, 1, 48'h00);

		 	////////////////////////////////////////////////////////////////////
			// DEC : check N
			////////////////////////////////////////////////////////////////////
            resetState();
            setMem(16'hc4, 8'hff);
            setX(4);
            $write("%40s", "DEC $C0,X [check C] :");
            //      (bytes,    ps,    pc, a,     x,     y,     sp,    #, ck, addr,   #, val)
            checkMem(48'hC0D6, 8'ha0, 2,  8'h00, 8'h04, 8'h00, 8'hff, 1, 6,  16'hc4, 1, 48'hfe);



		 	////////////////////////////////////////////////////////////////////
			// EOR : check Z
			////////////////////////////////////////////////////////////////////
            $display("");
            resetState();
            setMem(16'hc4, 8'h40);
            setX(4);
            setA(8'h40);
            $write("%40s", "EOR $C0,X [check Z] :");
            //   (bytes,    ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hC055, 8'h22, 2,  8'h00, 8'h04, 8'h00, 8'hff, 1, 4);	
            				
		 	////////////////////////////////////////////////////////////////////
			// EOR : check no flags
			////////////////////////////////////////////////////////////////////
            resetState();
            setMem(16'hc4, 8'h40);
            setX(4);
            setA(8'h20);
            $write("%40s", "EOR $C0,X [check no flags] :");
            //   (bytes,    ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hC055, 8'h20, 2,  8'h60, 8'h04, 8'h00, 8'hff, 1, 4);	
            				
		 	////////////////////////////////////////////////////////////////////
			// EOR : check n
			////////////////////////////////////////////////////////////////////
            resetState();
            setMem(16'hc4, 8'h7f);
            setX(4);
            setA(8'hff);
            $write("%40s", "EOR $C0,X [check N] :");
            //   (bytes,    ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hC055, 8'ha0, 2,  8'h80, 8'h04, 8'h00, 8'hff, 1, 4);	
             	


		 	////////////////////////////////////////////////////////////////////
			// INC : check no flags
			////////////////////////////////////////////////////////////////////
            $display("");
            resetState();
            setMem(16'hc4, 8'h42);
            setX(4);
            $write("%40s", "INC $C0,X [check no flags] :");
            //      (bytes,    ps,    pc, a,     x,     y,     sp,    #, ck, addr,   #, val)
            checkMem(48'hC0F6, 8'h20, 2,  8'h00, 8'h04, 8'h00, 8'hff, 1, 6,  16'hc4, 1, 48'h43);

		 	////////////////////////////////////////////////////////////////////
			// INC : check Z
			////////////////////////////////////////////////////////////////////
            resetState();
            setMem(16'hc4, 8'hff);
            setX(4);
            $write("%40s", "INC $C0,X [check Z] :");
            //      (bytes,    ps,    pc, a,     x,     y,     sp,    #, ck, addr,   #, val)
            checkMem(48'hC0F6, 8'h22, 2,  8'h00, 8'h04, 8'h00, 8'hff, 1, 6,  16'hc4, 1, 48'h00);

		 	////////////////////////////////////////////////////////////////////
			// INC : check N
			////////////////////////////////////////////////////////////////////
            resetState();
            setMem(16'hc4, 8'h7f);
            setX(4);
            $write("%40s", "INC $C0,X [check C] :");
            //      (bytes,    ps,    pc, a,     x,     y,     sp,    #, ck, addr,   #, val)
            checkMem(48'hC0F6, 8'ha0, 2,  8'h00, 8'h04, 8'h00, 8'hff, 1, 6,  16'hc4, 1, 48'h80);


            				
		 	////////////////////////////////////////////////////////////////////
			// LDA : check no flags
			////////////////////////////////////////////////////////////////////
            $display("");
            resetState();
            setX(4);
            setMem(16'hc4, 8'h40);
            $write("%40s", "LDA $C0,X [check no flags] :");
            //   (bytes,    ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hC0B5, 8'h20, 2,  8'h40, 8'h04, 8'h00, 8'hff, 1, 4);	
            				
		 	////////////////////////////////////////////////////////////////////
			// LDA : check z
			////////////////////////////////////////////////////////////////////
            resetState();
            setA(8'h20);
            setX(4);
            setMem(16'hc4, 8'h00);
            $write("%40s", "LDA $C0,X [check Z] :");
            //   (bytes,    ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hC0B5, 8'h22, 2,  8'h00, 8'h04, 8'h00, 8'hff, 1, 4);	
            				
		 	////////////////////////////////////////////////////////////////////
			// LDA : check n
			////////////////////////////////////////////////////////////////////
            resetState();
            setX(4);
            setMem(16'hc4, 8'h80);
            $write("%40s", "LDA $C0,X [check N] :");
            //   (bytes,    ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hC0B5, 8'ha0, 2,  8'h80, 8'h04, 8'h00, 8'hff, 1, 4);	
   			
   			
   			
		 	////////////////////////////////////////////////////////////////////
			// LDX,Y : check no flags
			////////////////////////////////////////////////////////////////////
            $display("");
            resetState();
            setMem(16'hc4, 8'h40);
            setY(4);
            $write("%40s", "LDX $C0,Y [check no flags] :");
            //   (bytes,    ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hC0B6, 8'h20, 2,  8'h00, 8'h40, 8'h04, 8'hff, 1, 4);	
			
		 	////////////////////////////////////////////////////////////////////
			// LDX,Y : check N
			////////////////////////////////////////////////////////////////////
            resetState();
            setMem(16'hc4, 8'h80);
            setY(4);
            $write("%40s", "LDX $C0,Y [check N] :");
            //   (bytes,    ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hC0B6, 8'ha0, 2,  8'h00, 8'h80, 8'h04, 8'hff, 1, 4);	
			
		 	////////////////////////////////////////////////////////////////////
			// LDX,Y : check Z
			////////////////////////////////////////////////////////////////////
            resetState();
            setMem(16'hc4, 8'h00);
            setY(4);
            $write("%40s", "LDX $C0,Y [check N] :");
            //   (bytes,    ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hC0B6, 8'h22, 2,  8'h00, 8'h00, 8'h04, 8'hff, 1, 4);	

			

		 	////////////////////////////////////////////////////////////////////
			// LDY : check no flags
			////////////////////////////////////////////////////////////////////
            $display("");
            resetState();
            setMem(16'hc4, 8'h40);
            setX(4);
            $write("%40s", "LDY $C0,X [check no flags] :");
            //   (bytes,    ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hC0B4, 8'h20, 2,  8'h00, 8'h04, 8'h40, 8'hff, 1, 4);	
            				
		 	////////////////////////////////////////////////////////////////////
			// LDY : check z
			////////////////////////////////////////////////////////////////////
            resetState();
            setMem(16'hc4, 8'h00);
            setY(8'h20);
            setX(4);
            $write("%40s", "LDY $C0,X [check Z] :");
            //   (bytes,    ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hC0B4, 8'h22, 2,  8'h00, 8'h04, 8'h00, 8'hff, 1, 4);	
            				
		 	////////////////////////////////////////////////////////////////////
			// LDY : check n
			////////////////////////////////////////////////////////////////////
            resetState();
            setMem(16'hc4, 8'h80);
            setX(4);
            $write("%40s", "LDY $C0,X [check N] :");
            //   (bytes,    ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hC0B4, 8'ha0, 2,  8'h00, 8'h04, 8'h80, 8'hff, 1, 4);	
             	


		 	////////////////////////////////////////////////////////////////////
			// LSR : check no flags
			////////////////////////////////////////////////////////////////////
            $display("");
            resetState();
            setX(4);
            setMem(16'hc4, 8'h02);
            $write("%40s", "LSR $C0,X [check no flags] :");
            //      (bytes,    ps,    pc, a,     x,     y,     sp,    #, ck, addr,  #, val)
            checkMem(48'hc056, 8'h20, 2,  8'h00, 8'h04, 8'h00, 8'hff, 1, 6, 16'hc4, 1, 48'h01);

		 	////////////////////////////////////////////////////////////////////
			// LSR : check C, Z
			////////////////////////////////////////////////////////////////////
            resetState();
            setX(4);
            setMem(16'hc4, 8'h1);
            $write("%40s", "LSR $C0,X [check C, Z] :");
            //      (bytes,    ps,    pc, a,     x,     y,     sp,    #, ck, addr,  #, val)
            checkMem(48'hc056, 8'h23, 2,  8'h00, 8'h04, 8'h00, 8'hff, 1, 6, 16'hc4, 1, 48'h00);

         	
            				
		 	////////////////////////////////////////////////////////////////////
			// ORA : check no flags
			////////////////////////////////////////////////////////////////////
            $display("");
            resetState();
            setMem(16'hc4, 8'h20);
            setX(4);
            setA(8'h40);
            $write("%40s", "ORA $C0,X [check no flags] :");
            //   (bytes,    ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hC015, 8'h20, 2,  8'h60, 8'h04, 8'h00, 8'hff, 1, 4);	
            				
		 	////////////////////////////////////////////////////////////////////
			// ORA : check z
			////////////////////////////////////////////////////////////////////
            resetState();
            setX(4);
            setMem(16'hc4, 8'h00);
            $write("%40s", "ORA $C0,X [check Z] :");
            //   (bytes,    ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hC015, 8'h22, 2,  8'h00, 8'h04, 8'h00, 8'hff, 1, 4);	
            				
		 	////////////////////////////////////////////////////////////////////
			// ORA : check n
			////////////////////////////////////////////////////////////////////
            resetState();
            setX(4);
            setMem(16'hc4, 8'h80);
            setA(8'h7f);
            $write("%40s", "ORA $C0,X [check N] :");
            //   (bytes,    ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hC015, 8'ha0, 2,  8'hff, 8'h04, 8'h00, 8'hff, 1, 4);	            				
           				
			
           				
		 	////////////////////////////////////////////////////////////////////
			// ROL : check no flags
			////////////////////////////////////////////////////////////////////
            $display("");
            resetState();
            setMem(16'hc4, 8'h02);
            setX(4);
            $write("%40s", "ROL $C0,X [check no flags] :");
            //      (bytes,    ps,    pc, a,     x,     y,     sp,    #, ck, addr,  #, val)
            checkMem(48'hc036, 8'h20, 2,  8'h00, 8'h04, 8'h00, 8'hff, 1, 6, 16'hc4, 1, 48'h04);

		 	////////////////////////////////////////////////////////////////////
			// ROL : check N, C
			////////////////////////////////////////////////////////////////////
            resetState();
            setX(4);
            setMem(16'hc4, 8'hc0);
            $write("%40s", "ROL $C0,X [check N, C] :");
            //      (bytes,    ps,    pc, a,     x,     y,     sp,    #, ck, addr,  #, val)
            checkMem(48'hc036, 8'ha1, 2,  8'h00, 8'h04, 8'h00, 8'hff, 1, 6, 16'hc4, 1, 48'h80);

		 	////////////////////////////////////////////////////////////////////
			// ROL : check C, Z
			////////////////////////////////////////////////////////////////////
            resetState();
            setX(4);
            setMem(16'hc4, 8'h80);
            $write("%40s", "ROL $C0,X [check C, Z] :");
            //      (bytes,    ps,    pc, a,     x,     y,     sp,    #, ck, addr,  #, val)
            checkMem(48'hc036, 8'h23, 2,  8'h00, 8'h04, 8'h00, 8'hff, 1, 6, 16'hc4, 1, 48'h00);



		 	////////////////////////////////////////////////////////////////////
			// ROR : check no flags
			////////////////////////////////////////////////////////////////////
            $display("");
            resetState();
            setMem(16'hc4, 8'h02);
            setX(4);
            $write("%40s", "ROR $C0,X [check no flags] :");
            //      (bytes,    ps,    pc, a,     x,     y,     sp,    #, ck, addr,  #, val)
            checkMem(48'hc076, 8'h20, 2,  8'h00, 8'h04, 8'h00, 8'hff, 1, 6, 16'hc4, 1, 48'h01);

		 	////////////////////////////////////////////////////////////////////
			// ROR : check N, C
			////////////////////////////////////////////////////////////////////
            resetState();
            setPS(8'h21);
            setMem(16'hc4, 8'hc0);
            setX(4);
            $write("%40s", "ROR $C0,X [check N, C] :");
            //      (bytes,    ps,    pc, a,     x,     y,     sp,    #, ck, addr,  #, val)
            checkMem(48'hc076, 8'ha0, 2,  8'h00, 8'h04, 8'h00, 8'hff, 1, 6, 16'hc4, 1, 48'he0);

		 	////////////////////////////////////////////////////////////////////
			// ROR : check C, Z
			////////////////////////////////////////////////////////////////////
            resetState();
            setMem(16'hc4, 8'h1);
            setX(4);
            $write("%40s", "ROR $C0,X [check C, Z] :");
            //      (bytes,    ps,    pc, a,     x,     y,     sp,    #, ck, addr,  #, val)
            checkMem(48'hc076, 8'h23, 2,  8'h00, 8'h04, 8'h00, 8'hff, 1, 6, 16'hc4, 1, 48'h00);




		 	////////////////////////////////////////////////////////////////////
			// SBC : check no flags
			////////////////////////////////////////////////////////////////////
            $display("");
            resetState();
            setX(4);
            setMem(16'hc4, 8'h85);
            $write("%40s", "SBC $C0 [binary, check no flags] :");
            //   (bytes,    ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hC0F5, 8'h20, 2,  8'h7a, 8'h04, 8'h00, 8'hff, 1, 4);	

		 	////////////////////////////////////////////////////////////////////
			// SBC : check C
			////////////////////////////////////////////////////////////////////
            resetState();
            setA(8'h0b);
            setX(4);
            setMem(16'hc4, 8'h05);
            $write("%40s", "SBC $C0 [binary, check C] :");
            //   (bytes,    ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hC0F5, 8'h21, 2,  8'h05, 8'h04, 8'h00, 8'hff, 1, 4);	
         				
		 	////////////////////////////////////////////////////////////////////
			// SBC : check V
			////////////////////////////////////////////////////////////////////
            resetState();
            setMem(16'hc4, 8'h05);
            setA(8'h80);
            setX(4);
            $write("%40s", "SBC $C0 [binary, check V] :");
            //   (bytes,    ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hC0F5, 8'h61, 2,  8'h7a, 8'h04, 8'h00, 8'hff, 1, 4);	
           				
		 	////////////////////////////////////////////////////////////////////
			// SBC : check N
			////////////////////////////////////////////////////////////////////
            resetState();
            setMem(16'hc4, 8'h00);
            setX(4);
            $write("%40s", "SBC $C0 [binary, check N] :");
            //   (bytes,    ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hC0F5, 8'ha0, 2,  8'hff, 8'h04, 8'h00, 8'hff, 1, 4);	   
                    				
		 	////////////////////////////////////////////////////////////////////
			// SBC : check Z
			////////////////////////////////////////////////////////////////////
            resetState();
            setMem(16'hc4, 8'h00);
            setA(1);
            setX(4);
            $write("%40s", "SBC $C0 [binary, check Z] :");
            //   (bytes,    ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hC0F5, 8'h23, 2,  8'h00, 8'h04, 8'h00, 8'hff, 1, 4);            				
   
   
		 	////////////////////////////////////////////////////////////////////
			// SBC :  decimal, check no flags
			////////////////////////////////////////////////////////////////////
            $display("");
            resetState();
            setMem(16'hc4, 8'h85);
            setPS(8'h28);
            setX(4);
            $write("%40s", "SBC #$85 [decimal, check no flags] :");
            //   (bytes,    ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hC0F5, 8'h28, 2,  8'h14, 8'h04, 8'h00, 8'hff, 1, 4);	
            				
		 	////////////////////////////////////////////////////////////////////
			// SBC :  decimal, check N
			////////////////////////////////////////////////////////////////////
            resetState();
            resetState();
            setPS(8'h28);
            setX(4);
            setMem(16'hc4, 8'h05);
            $write("%40s", "SBC #5 [decimal, check N] :");
            //   (bytes,    ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hC0F5, 8'ha8, 2,  8'h94, 8'h04, 8'h00, 8'hff, 1, 4);	
             				           				
		 	////////////////////////////////////////////////////////////////////
			// SBC :  decimal, Z
			////////////////////////////////////////////////////////////////////
            resetState();
            setPS(8'h28);
            setMem(16'hc4, 8'h00);
            setX(4);
            setA(1);
            $write("%40s", "SBC #$00 [decimal, check Z,C] :");
            //   (bytes,    ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hC0F5, 8'h2b, 2,  8'h00, 8'h04, 8'h00, 8'hff, 1, 4);	
				
		 	////////////////////////////////////////////////////////////////////
			// SBC : V flag not valid in decimal mode
			////////////////////////////////////////////////////////////////////


           				
		 	////////////////////////////////////////////////////////////////////
			// STA : check memory
			////////////////////////////////////////////////////////////////////
            $display("");
            resetState();
            setMem(16'hc4, 8'h20);
            setX(4);
            setA(8'h69);
            $write("%40s", "STA $C0,X [check memory] :");
            //      (bytes,    ps,    pc, a,     x,     y,     sp,    #, ck, addr,   #, val)
            checkMem(48'hC095, 8'h20, 2,  8'h69, 8'h04, 8'h00, 8'hff, 1, 4,  16'hc4, 1, 48'h69);


			
		 	////////////////////////////////////////////////////////////////////
			// STX,Y : check memory
			////////////////////////////////////////////////////////////////////
            $display("");
            resetState();
            setMem(16'hc4, 8'h40);
            setY(4);
            setX(8'h69);
            $write("%40s", "STX $C0,Y [check no flags] :");
            //      (bytes,    ps,    pc, a,     x,     y,     sp,    #, ck, addr,    #, data)
            checkMem(48'hC096, 8'h20, 2,  8'h00, 8'h69, 8'h04, 8'hff, 1, 4, 16'h00c4, 1, 48'h69);	

            				
            				
		 	////////////////////////////////////////////////////////////////////
			// STY : check memory
			////////////////////////////////////////////////////////////////////
            resetState();
            setMem(16'hc4, 8'h20);
            setX(4);
            setY(8'h71);
            $write("%40s", "STY $C0,X [check memory] :");
            //      (bytes,    ps,    pc, a,     x,     y,     sp,    #, ck, addr,   #, val)
            checkMem(48'hC094, 8'h20, 2,  8'h00, 8'h04, 8'h71, 8'hff, 1, 4,  16'hc4, 1, 48'h71);

		end
	endtask


	
	
	////////////////////////////////////////////////////////////////////////////
	// Test all the absolute addressing mode operations
	////////////////////////////////////////////////////////////////////////////
	task testAddrAbs;
		begin
			$display("\nTest absolute addressing:");
					
		 	////////////////////////////////////////////////////////////////////
			// ADC : check no flags
			////////////////////////////////////////////////////////////////////
            $display("");
            resetState();
            setMem(16'hc000, 8'h69);
            $write("%40s", "ADC $C000 [binary, check no flags] :");
            //   (bytes,      ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hc0006D, 8'h20, 3,  8'h69, 8'h00, 8'h00, 8'hff, 1, 4);	

            				
		 	////////////////////////////////////////////////////////////////////
			// ADC : check V,C
			////////////////////////////////////////////////////////////////////
            resetState();
            setA(8'h80);
            setMem(16'hc000, 8'h85);
            $write("%40s", "ADC $C000 [binary, check V,C] :");
            //   (bytes,      ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hc0006d, 8'h61, 3,  8'h05, 8'h00, 8'h00, 8'hff, 1, 4);	
            				
		 	////////////////////////////////////////////////////////////////////
			// ADC : check N
			////////////////////////////////////////////////////////////////////
            resetState();
            setA(8'h50);
            setMem(16'hc000, 8'h85);
            $write("%40s", "ADC $C000 [binary, check V,N] :");
            //   (bytes,      ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hC0006d, 8'ha0, 3,  8'hd5, 8'h00, 8'h00, 8'hff, 1, 4);	

		 	////////////////////////////////////////////////////////////////////
			// ADC : check Z
			////////////////////////////////////////////////////////////////////
            resetState();
            setMem(16'hc000, 8'h00);
            $write("%40s", "ADC $C000 [binary, check Z] :");
            //   (bytes,      ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hC0006d, 8'h22, 3,  8'h00, 8'h00, 8'h00, 8'hff, 1, 4);	


		 	////////////////////////////////////////////////////////////////////
			// ADC : check decimal, no flags
			////////////////////////////////////////////////////////////////////
            $display("");
            resetState();
            setMem(16'hc000, 8'h05);
            setPS(8'h28);
            $write("%40s", "ADC $C000 [decimal, check no flags] :");
            //   (bytes,      ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hC0006d, 8'h28, 3,  8'h05, 8'h00, 8'h00, 8'hff, 1, 4);	
            				
		 	////////////////////////////////////////////////////////////////////
			// ADC : check decimal, C
			////////////////////////////////////////////////////////////////////
            resetState();
            setPS(8'h28);
            setMem(16'hc000, 8'h85);
            setA(8'h80);
            $write("%40s", "ADC $C000 [decimal, check V,C] :");
            //   (bytes,      ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hC0006d, 8'h69, 3,  8'h65, 8'h00, 8'h00, 8'hff, 1, 4);	
            				
		 	////////////////////////////////////////////////////////////////////
			// ADC : check decimal, V,N
			////////////////////////////////////////////////////////////////////
            resetState();
            setPS(8'h28);
            setMem(16'hc000, 8'h40);
            setA(8'h40);
            $write("%40s", "ADC $C000 [decimal, check V,N] :");
            //   (bytes,      ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hC0006d, 8'he8, 3,  8'h80, 8'h00, 8'h00, 8'hff, 1, 4);	
            				
		 	////////////////////////////////////////////////////////////////////
			// ADC : check decimal, Z
			////////////////////////////////////////////////////////////////////
            resetState();
            setMem(16'hc000, 8'h00);
            setPS(8'h28);
            $write("%40s", "ADC $C000 [decimal, check Z] :");
            //   (bytes,      ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hC0006d, 8'h2a, 3,  8'h00, 8'h00, 8'h00, 8'hff, 1, 4);	
            	
            	
            				
		 	////////////////////////////////////////////////////////////////////
			// AND : check no flags
			////////////////////////////////////////////////////////////////////
            $display("");
            resetState();
            setMem(16'hc000, 8'h40);
            setA(8'h40);
            $write("%40s", "AND $C000 [check no flags] :");
            //   (bytes,      ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hC0002d, 8'h20, 3,  8'h40, 8'h00, 8'h00, 8'hff, 1, 4);	
            				
		 	////////////////////////////////////////////////////////////////////
			// AND : check z
			////////////////////////////////////////////////////////////////////
            resetState();
            setA(8'h20);
            setMem(16'hc000, 8'h40);
            $write("%40s", "AND $C000 [check Z] :");
            //   (bytes,      ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hC0002d, 8'h22, 3,  8'h00, 8'h00, 8'h00, 8'hff, 1, 4);	
            				
		 	////////////////////////////////////////////////////////////////////
			// AND : check n
			////////////////////////////////////////////////////////////////////
            resetState();
            setA(8'hff);
            setMem(16'hc000, 8'h80);
            $write("%40s", "AND $C000 [check N] :");
            //   (bytes,      ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hC0002d, 8'ha0, 3,  8'h80, 8'h00, 8'h00, 8'hff, 1, 4);	
		
		
			
		 	////////////////////////////////////////////////////////////////////
			// ASL : check no flags
			////////////////////////////////////////////////////////////////////
            $display("");
            resetState();
            setMem(16'hc000, 8'h02);
            $write("%40s", "ASL $C000 [check no flags] :");
            //      (bytes,      ps,    pc, a,     x,     y,     sp,    #, ck, addr,  #, val)
            checkMem(48'hc0000e, 8'h20, 3,  8'h00, 8'h00, 8'h00, 8'hff, 1, 6, 16'hc000, 1, 48'h04);

		 	////////////////////////////////////////////////////////////////////
			// ASL : check N
			////////////////////////////////////////////////////////////////////
            resetState();
            setMem(16'hc000, 8'h40);
            $write("%40s", "ASL $C000 [check N] :");
            //      (bytes,      ps,    pc, a,     x,     y,     sp,    #, ck, addr,  #, val)
            checkMem(48'hc0000e, 8'ha0, 3,  8'h00, 8'h00, 8'h00, 8'hff, 1, 6, 16'hc000, 1, 48'h80);

		 	////////////////////////////////////////////////////////////////////
			// ASL : check C, Z
			////////////////////////////////////////////////////////////////////
            resetState();
            setMem(16'hc000, 8'h80);
            $write("%40s", "ASL $C000 [check C, Z] :");
            //      (bytes,      ps,    pc, a,     x,     y,     sp,    #, ck, addr,  #, val)
            checkMem(48'hc0000e, 8'h23, 3,  8'h00, 8'h00, 8'h00, 8'hff, 1, 6, 16'hc000, 1, 48'h00);




		 	////////////////////////////////////////////////////////////////////
			// BIT : check no flags
			////////////////////////////////////////////////////////////////////
            $display("");
            resetState();
            setMem(16'hc000, 8'h20);
            setA(8'hff);
            $write("%40s", "BIT $C000 [check no flags] :");
            //   (bytes,      ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hC0002c, 8'h20, 3,  8'hff, 8'h00, 8'h00, 8'hff, 1, 4);	
            				
		 	////////////////////////////////////////////////////////////////////
			// BIT : check z
			////////////////////////////////////////////////////////////////////
            resetState();
            setA(8'h10);
            setMem(16'hc000, 8'h20);
            $write("%40s", "BIT $C000 [check Z] :");
            //   (bytes,      ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hC0002c, 8'h22, 3,  8'h10, 8'h00, 8'h00, 8'hff, 1, 4);	
            				
		 	////////////////////////////////////////////////////////////////////
			// BIT : check n
			////////////////////////////////////////////////////////////////////
            resetState();
            setA(8'hff);
            setMem(16'hc000, 8'h80);
            $write("%40s", "BIT $C000 [check N] :");
            //   (bytes,      ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hC0002c, 8'ha0, 3,  8'hff, 8'h00, 8'h00, 8'hff, 1, 4);	
            				
		 	////////////////////////////////////////////////////////////////////
			// BIT : check v
			////////////////////////////////////////////////////////////////////
            resetState();
            setA(8'hff);
            setMem(16'hc000, 8'hc0);
            $write("%40s", "BIT $C000 [check V] :");
            //   (bytes,      ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hC0002c, 8'he0, 3,  8'hff, 8'h00, 8'h00, 8'hff, 1, 4);	
			



		 	////////////////////////////////////////////////////////////////////
			// CMP : check no flags
			////////////////////////////////////////////////////////////////////
            $display("");
            resetState();
            setMem(16'hc000, 8'h00);
            setA(5);
            $write("%40s", "CMP $C000 [check C] :");
            //   (bytes,      ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hC000Cd, 8'h21, 3,  8'h05, 8'h00, 8'h00, 8'hff, 1, 4);	
            				
		 	////////////////////////////////////////////////////////////////////
			// CMP : check N
			////////////////////////////////////////////////////////////////////
            resetState();
            setMem(16'hc000, 8'h05);
            $write("%40s", "CMP $C000 [check N] :");
            //   (bytes,      ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hC000Cd, 8'ha0, 3,  8'h00, 8'h00, 8'h00, 8'hff, 1, 4);	
            				
		 	////////////////////////////////////////////////////////////////////
			// CMP : check Z
			////////////////////////////////////////////////////////////////////
            resetState();
            setMem(16'hc000, 8'h05);
            setA(5);
            $write("%40s", "CMP $C000 [check Z] :");
            //   (bytes,      ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hC000Cd, 8'h23, 3,  8'h05, 8'h00, 8'h00, 8'hff, 1, 4);	

            	
            				
            				
		 	////////////////////////////////////////////////////////////////////
			// CPX : check no flags
			////////////////////////////////////////////////////////////////////
            $display("");
            resetState();
            setMem(16'hc000, 8'h00);
            setX(5);
            $write("%40s", "CPX $C000 [check C] :");
            //   (bytes,      ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hC000Ec, 8'h21, 3,  8'h00, 8'h05, 8'h00, 8'hff, 1, 4);	
            				
		 	////////////////////////////////////////////////////////////////////
			// CPX : check N
			////////////////////////////////////////////////////////////////////
            resetState();
            setMem(16'hc000, 8'h05);
            $write("%40s", "CPX $C000 [check N] :");
            //   (bytes,      ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hC000Ec, 8'ha0, 3,  8'h00, 8'h00, 8'h00, 8'hff, 1, 4);	
            				
		 	////////////////////////////////////////////////////////////////////
			// CPX : check Z
			////////////////////////////////////////////////////////////////////
            resetState();
            setMem(16'hc000, 8'h05);
            setX(5);
            $write("%40s", "CPX $C000 [check Z] :");
            //   (bytes,      ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hC000Ec, 8'h23, 3,  8'h00, 8'h05, 8'h00, 8'hff, 1, 4);	

            	
            				

		 	////////////////////////////////////////////////////////////////////
			// CPY : check no flags
			////////////////////////////////////////////////////////////////////
            $display("");
            resetState();
            setMem(16'hc000, 8'h00);
            setY(5);
            $write("%40s", "CPY $C000 [check C] :");
            //   (bytes,      ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hC000CC, 8'h21, 3,  8'h00, 8'h00, 8'h05, 8'hff, 1, 4);	
            				
		 	////////////////////////////////////////////////////////////////////
			// CPY : check N
			////////////////////////////////////////////////////////////////////
            resetState();
            setMem(16'hc000, 8'h05);
            $write("%40s", "CPY $C000 [check N] :");
            //   (bytes,      ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hC000CC, 8'ha0, 3,  8'h00, 8'h00, 8'h00, 8'hff, 1, 4);	
            				
		 	////////////////////////////////////////////////////////////////////
			// CPY : check Z
			////////////////////////////////////////////////////////////////////
            resetState();
            setMem(16'hc000, 8'h05);
            setY(5);
            $write("%40s", "CPY $C000 [check Z] :");
            //   (bytes,      ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hC000CC, 8'h23, 3,  8'h00, 8'h00, 8'h05, 8'hff, 1, 4);	

             	


		 	////////////////////////////////////////////////////////////////////
			// DEC : check no flags
			////////////////////////////////////////////////////////////////////
            $display("");
            resetState();
            setMem(16'hc000, 8'h42);
            $write("%40s", "DEC $C000 [check no flags] :");
            //      (bytes,      ps,    pc, a,     x,     y,     sp,    #, ck, addr,     #, val)
            checkMem(48'hC000CE, 8'h20, 3,  8'h00, 8'h00, 8'h00, 8'hff, 1, 6,  16'hc000, 1, 48'h41);

		 	////////////////////////////////////////////////////////////////////
			// DEC : check Z
			////////////////////////////////////////////////////////////////////
            resetState();
            setMem(16'hc000, 8'h1);
            $write("%40s", "DEC $C000 [check Z] :");
            //      (bytes,      ps,    pc, a,     x,     y,     sp,    #, ck, addr,     #, val)
            checkMem(48'hC000CE, 8'h22, 3,  8'h00, 8'h00, 8'h00, 8'hff, 1, 6,  16'hc000, 1, 48'h00);

		 	////////////////////////////////////////////////////////////////////
			// DEC : check N
			////////////////////////////////////////////////////////////////////
            resetState();
            setMem(16'hc000, 8'hff);
            $write("%40s", "DEC $C000 [check C] :");
            //      (bytes,      ps,    pc, a,     x,     y,     sp,    #, ck, addr,     #, val)
            checkMem(48'hC000CE, 8'ha0, 3,  8'h00, 8'h00, 8'h00, 8'hff, 1, 6,  16'hc000, 1, 48'hfe);

            				
            	
            				
		 	////////////////////////////////////////////////////////////////////
			// EOR : check Z
			////////////////////////////////////////////////////////////////////
            $display("");
            resetState();
            setMem(16'hc000, 8'h40);
            setA(8'h40);
            $write("%40s", "EOR $C000 [check Z] :");
            //   (bytes,      ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hC0004D, 8'h22, 3,  8'h00, 8'h00, 8'h00, 8'hff, 1, 4);	
            				
		 	////////////////////////////////////////////////////////////////////
			// EOR : check no flags
			////////////////////////////////////////////////////////////////////
            resetState();
            setMem(16'hc000, 8'h40);
            setA(8'h20);
            $write("%40s", "EOR $C000 [check no flags] :");
            //   (bytes,      ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hC0004D, 8'h20, 3,  8'h60, 8'h00, 8'h00, 8'hff, 1, 4);	
            				
		 	////////////////////////////////////////////////////////////////////
			// EOR : check n
			////////////////////////////////////////////////////////////////////
            resetState();
            setMem(16'hc000, 8'h7f);
            setA(8'hff);
            $write("%40s", "EOR $C000 [check N] :");
            //   (bytes,      ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hC0004D, 8'ha0, 3,  8'h80, 8'h00, 8'h00, 8'hff, 1, 4);	


             	

		 	////////////////////////////////////////////////////////////////////
			// INC : check no flags
			////////////////////////////////////////////////////////////////////
            $display("");
            resetState();
            setMem(16'hc000, 8'h42);
            $write("%40s", "INC $C000 [check no flags] :");
            //      (bytes,      ps,    pc, a,     x,     y,     sp,    #, ck, addr,     #, val)
            checkMem(48'hC000EE, 8'h20, 3,  8'h00, 8'h00, 8'h00, 8'hff, 1, 6,  16'hc000, 1, 48'h43);

		 	////////////////////////////////////////////////////////////////////
			// INC : check Z
			////////////////////////////////////////////////////////////////////
            resetState();
            setMem(16'hc000, 8'hff);
            $write("%40s", "INC $C000 [check Z] :");
            //      (bytes,      ps,    pc, a,     x,     y,     sp,    #, ck, addr,     #, val)
            checkMem(48'hC000EE, 8'h22, 3,  8'h00, 8'h00, 8'h00, 8'hff, 1, 6,  16'hc000, 1, 48'h00);

		 	////////////////////////////////////////////////////////////////////
			// INC : check N
			////////////////////////////////////////////////////////////////////
            resetState();
            setMem(16'hc000, 8'h7f);
            $write("%40s", "INC $C000 [check C] :");
            //      (bytes,      ps,    pc, a,     x,     y,     sp,    #, ck, addr,     #, val)
            checkMem(48'hC000EE, 8'ha0, 3,  8'h00, 8'h00, 8'h00, 8'hff, 1, 6,  16'hc000, 1, 48'h80);




		 	////////////////////////////////////////////////////////////////////
			// JMP : check PC
			////////////////////////////////////////////////////////////////////
            $display("");
            resetState();
            $write("%40s", "JMP $A000 [check PC] :");
            //   (bytes,      ps,    pc,        a,     x,     y,     sp,    #, ck)
            check(48'hA0004C, 8'h20, 16'hA000,  8'h00, 8'h00, 8'h00, 8'hff, 1, 3);

		 	////////////////////////////////////////////////////////////////////
			// JMP : check PC
			////////////////////////////////////////////////////////////////////
            resetState();
            setMem(16'ha000, 8'h00);
            setMem(16'ha001, 8'hd0);
            $write("%40s", "JMP ($A000) [check PC] :");
            //   (bytes,      ps,    pc,        a,     x,     y,     sp,    #, ck)
            check(48'hA0006C, 8'h20, 16'hD000,  8'h00, 8'h00, 8'h00, 8'hff, 1, 5);

		 	////////////////////////////////////////////////////////////////////
			// JSR : check stack,PC
			////////////////////////////////////////////////////////////////////
            resetState();
            $write("%40s", "JSR $A000 [check PC] :");
            //      (bytes,      ps,    pc,        a,     x,     y,     sp,    #, ck)
            checkMem(48'hA00020, 8'h20, 16'hA000,  8'h00, 8'h00, 8'h00, 8'hfd, 1, 6, 16'h01fe, 2, 16'h0001);



            				
		 	////////////////////////////////////////////////////////////////////
			// LDA : check no flags
			////////////////////////////////////////////////////////////////////
            $display("");
            resetState();
            setMem(16'hc000, 8'h40);
            $write("%40s", "LDA $C000 [check no flags] :");
            //   (bytes,      ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hC000AD, 8'h20, 3,  8'h40, 8'h00, 8'h00, 8'hff, 1, 4);	
            				
		 	////////////////////////////////////////////////////////////////////
			// LDA : check z
			////////////////////////////////////////////////////////////////////
            resetState();
            setA(8'h20);
            setMem(16'hc000, 8'h00);
            $write("%40s", "LDA $C000 [check Z] :");
            //   (bytes,      ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hC000AD, 8'h22, 3,  8'h00, 8'h00, 8'h00, 8'hff, 1, 4);	
            				
		 	////////////////////////////////////////////////////////////////////
			// LDA : check n
			////////////////////////////////////////////////////////////////////
            resetState();
            setMem(16'hc000, 8'h80);
            $write("%40s", "LDA $C000 [check N] :");
            //   (bytes,      ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hC000AD, 8'ha0, 3,  8'h80, 8'h00, 8'h00, 8'hff, 1, 4);	
             	
            	
            				
		 	////////////////////////////////////////////////////////////////////
			// LDX : check no flags
			////////////////////////////////////////////////////////////////////
            $display("");
            resetState();
            setMem(16'hc000, 8'h40);
            $write("%40s", "LDX $C000 [check no flags] :");
            //   (bytes,      ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hC000AE, 8'h20, 3,  8'h00, 8'h40, 8'h00, 8'hff, 1, 4);	
            				
		 	////////////////////////////////////////////////////////////////////
			// LDX : check z
			////////////////////////////////////////////////////////////////////
            resetState();
            setMem(16'hc000, 8'h00);
            setX(8'h20);
            $write("%40s", "LDX $C000 [check Z] :");
            //   (bytes,      ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hC000AE, 8'h22, 3,  8'h00, 8'h00, 8'h00, 8'hff, 1, 4);	
            				
		 	////////////////////////////////////////////////////////////////////
			// LDX : check n
			////////////////////////////////////////////////////////////////////
            resetState();
            setMem(16'hc000, 8'h80);
            $write("%40s", "LDX $C000 [check N] :");
            //   (bytes,      ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hC000AE, 8'ha0, 3,  8'h00, 8'h80, 8'h00, 8'hff, 1, 4);	
             	
            	
            				
		 	////////////////////////////////////////////////////////////////////
			// LDY : check no flags
			////////////////////////////////////////////////////////////////////
            $display("");
            resetState();
            setMem(16'hc000, 8'h40);
            $write("%40s", "LDY $C000 [check no flags] :");
            //   (bytes,      ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hC000AC, 8'h20, 3,  8'h00, 8'h00, 8'h40, 8'hff, 1, 4);	
            				
		 	////////////////////////////////////////////////////////////////////
			// LDY : check z
			////////////////////////////////////////////////////////////////////
            resetState();
            setMem(16'hc000, 8'h00);
            setY(8'h20);
            $write("%40s", "LDY $C000 [check Z] :");
            //   (bytes,      ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hC000AC, 8'h22, 3,  8'h00, 8'h00, 8'h00, 8'hff, 1, 4);	
            				
		 	////////////////////////////////////////////////////////////////////
			// LDY : check n
			////////////////////////////////////////////////////////////////////
            resetState();
            setMem(16'hc000, 8'h80);
            $write("%40s", "LDY $C000 [check N] :");
            //   (bytes,      ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hC000AC, 8'ha0, 3,  8'h00, 8'h00, 8'h80, 8'hff, 1, 4);	
             	


		 	////////////////////////////////////////////////////////////////////
			// LSR : check no flags
			////////////////////////////////////////////////////////////////////
            $display("");
            resetState();
            setMem(16'hc000, 8'h02);
            $write("%40s", "LSR $C000 [check no flags] :");
            //      (bytes,      ps,    pc, a,     x,     y,     sp,    #, ck, addr,    #, val)
            checkMem(48'hc0004e, 8'h20, 3,  8'h00, 8'h00, 8'h00, 8'hff, 1, 6, 16'hc000, 1, 48'h01);

		 	////////////////////////////////////////////////////////////////////
			// LSR : check C, Z
			////////////////////////////////////////////////////////////////////
            resetState();
            setMem(16'hc000, 8'h1);
            $write("%40s", "LSR $C000 [check C, Z] :");
            //      (bytes,      ps,    pc, a,     x,     y,     sp,    #, ck, addr,    #, val)
            checkMem(48'hc0004e, 8'h23, 3,  8'h00, 8'h00, 8'h00, 8'hff, 1, 6, 16'hc000, 1, 48'h00);

            	
            				
		 	////////////////////////////////////////////////////////////////////
			// ORA : check no flags
			////////////////////////////////////////////////////////////////////
            $display("");
            resetState();
            setMem(16'hc000, 8'h20);
            setA(8'h40);
            $write("%40s", "ORA $C000 [check no flags] :");
            //   (bytes,      ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hC0000D, 8'h20, 3,  8'h60, 8'h00, 8'h00, 8'hff, 1, 4);	
            				
		 	////////////////////////////////////////////////////////////////////
			// ORA : check z
			////////////////////////////////////////////////////////////////////
            resetState();
            setMem(16'hc000, 8'h00);
            $write("%40s", "ORA $C000 [check Z] :");
            //   (bytes,      ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hC0000D, 8'h22, 3,  8'h00, 8'h00, 8'h00, 8'hff, 1, 4);	
            				
		 	////////////////////////////////////////////////////////////////////
			// ORA : check n
			////////////////////////////////////////////////////////////////////
            resetState();
            setMem(16'hc000, 8'h80);
            setA(8'h7f);
            $write("%40s", "ORA $C000 [check N] :");
            //   (bytes,      ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hC0000D, 8'ha0, 3,  8'hff, 8'h00, 8'h00, 8'hff, 1, 4);	            				
           				


		 	////////////////////////////////////////////////////////////////////
			// ROL : check no flags
			////////////////////////////////////////////////////////////////////
            $display("");
            resetState();
            setMem(16'hc000, 8'h02);
            $write("%40s", "ROL $C000 [check no flags] :");
            //      (bytes,      ps,    pc, a,     x,     y,     sp,    #, ck, addr,    #, val)
            checkMem(48'hc0002E, 8'h20, 3,  8'h00, 8'h00, 8'h00, 8'hff, 1, 6, 16'hc000, 1, 48'h04);

		 	////////////////////////////////////////////////////////////////////
			// ROL : check N, C
			////////////////////////////////////////////////////////////////////
            resetState();
            setMem(16'hc000, 8'hc0);
            $write("%40s", "ROL $C000 [check N, C] :");
            //      (bytes,      ps,    pc, a,     x,     y,     sp,    #, ck, addr,    #, val)
            checkMem(48'hc0002E, 8'ha1, 3,  8'h00, 8'h00, 8'h00, 8'hff, 1, 6, 16'hc000, 1, 48'h80);

		 	////////////////////////////////////////////////////////////////////
			// ROL : check C, Z
			////////////////////////////////////////////////////////////////////
            resetState();
            setMem(16'hc000, 8'h80);
            $write("%40s", "ROL $C000 [check C, Z] :");
            //      (bytes,      ps,    pc, a,     x,     y,     sp,    #, ck, addr,    #, val)
            checkMem(48'hc0002E, 8'h23, 3,  8'h00, 8'h00, 8'h00, 8'hff, 1, 6, 16'hc000, 1, 48'h00);




		 	////////////////////////////////////////////////////////////////////
			// ROR : check no flags
			////////////////////////////////////////////////////////////////////
            $display("");
            resetState();
            setMem(16'hc000, 8'h02);
            $write("%40s", "ROR $C000 [check no flags] :");
            //      (bytes,      ps,    pc, a,     x,     y,     sp,    #, ck, addr,    #, val)
            checkMem(48'hc0006E, 8'h20, 3,  8'h00, 8'h00, 8'h00, 8'hff, 1, 6, 16'hc000, 1, 48'h01);

		 	////////////////////////////////////////////////////////////////////
			// ROR : check N, C
			////////////////////////////////////////////////////////////////////
            resetState();
            setPS(8'h21);
            setMem(16'hc000, 8'hc0);
            $write("%40s", "ROR $C000 [check N, C] :");
            //      (bytes,      ps,    pc, a,     x,     y,     sp,    #, ck, addr,    #, val)
            checkMem(48'hc0006E, 8'ha0, 3, 8'h00, 8'h00, 8'h00, 8'hff, 1, 6, 16'hc000, 1, 48'he0);

		 	////////////////////////////////////////////////////////////////////
			// ROR : check C, Z
			////////////////////////////////////////////////////////////////////
            resetState();
            setMem(16'hc000, 8'h1);
            $write("%40s", "ROR $C000 [check C, Z] :");
            //      (bytes,      ps,    pc, a,     x,     y,     sp,    #, ck, addr,    #, val)
            checkMem(48'hc0006E, 8'h23, 3,  8'h00, 8'h00, 8'h00, 8'hff, 1, 6, 16'hc000, 1, 48'h00);


			

		 	////////////////////////////////////////////////////////////////////
			// SBC : check no flags
			////////////////////////////////////////////////////////////////////
            $display("");
            resetState();
            setMem(16'hc000, 8'h85);
            $write("%40s", "SBC $C000 [binary, check no flags] :");
            //   (bytes,      ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hC000ED, 8'h20, 3,  8'h7a, 8'h00, 8'h00, 8'hff, 1, 4);	

		 	////////////////////////////////////////////////////////////////////
			// SBC : check C
			////////////////////////////////////////////////////////////////////
            resetState();
            setA(8'h0b);
            setMem(16'hc000, 8'h05);
            $write("%40s", "SBC $C000 [binary, check C] :");
            //   (bytes,      ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hC000ED, 8'h21, 3,  8'h05, 8'h00, 8'h00, 8'hff, 1, 4);	
         				
		 	////////////////////////////////////////////////////////////////////
			// SBC : check V
			////////////////////////////////////////////////////////////////////
            resetState();
            setMem(16'hc000, 8'h05);
            setA(8'h80);
            $write("%40s", "SBC $C000 [binary, check V] :");
            //   (bytes,      ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hC000ED, 8'h61, 3,  8'h7a, 8'h00, 8'h00, 8'hff, 1, 4);	
           				
		 	////////////////////////////////////////////////////////////////////
			// SBC : check N
			////////////////////////////////////////////////////////////////////
            resetState();
            setMem(16'hc000, 8'h00);
            $write("%40s", "SBC $C000 [binary, check N] :");
            //   (bytes,      ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hC000ED, 8'ha0, 3,  8'hff, 8'h00, 8'h00, 8'hff, 1, 4);	   
                    				
		 	////////////////////////////////////////////////////////////////////
			// SBC : check Z
			////////////////////////////////////////////////////////////////////
            resetState();
            setMem(16'hc000, 8'h00);
            setA(1);
            $write("%40s", "SBC $C000 [binary, check Z] :");
            //   (bytes,      ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hC000ED, 8'h23, 3,  8'h00, 8'h00, 8'h00, 8'hff, 1, 4);            				
   
   
		 	////////////////////////////////////////////////////////////////////
			// SBC :  decimal, check no flags
			////////////////////////////////////////////////////////////////////
            $display("");
            resetState();
            setMem(16'hc000, 8'h85);
            setPS(8'h28);
            $write("%40s", "SBC $C000 [decimal, check no flags] :");
            //   (bytes,      ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hC000Ed, 8'h28, 3,  8'h14, 8'h00, 8'h00, 8'hff, 1, 4);	
            				
		 	////////////////////////////////////////////////////////////////////
			// SBC :  decimal, check N
			////////////////////////////////////////////////////////////////////
            resetState();
            resetState();
            setPS(8'h28);
            setMem(16'hc000, 8'h05);
            $write("%40s", "SBC $C000 [decimal, check N] :");
            //   (bytes,      ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hC000Ed, 8'ha8, 3,  8'h94, 8'h00, 8'h00, 8'hff, 1, 4);	
             				           				
		 	////////////////////////////////////////////////////////////////////
			// SBC :  decimal, Z
			////////////////////////////////////////////////////////////////////
            resetState();
            setPS(8'h28);
            setMem(16'hc000, 8'h00);
            setA(1);
            $write("%40s", "SBC $C000 [decimal, check Z,C] :");
            //   (bytes,      ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hC000Ed, 8'h2b, 3,  8'h00, 8'h00, 8'h00, 8'hff, 1, 4);	
				
		 	////////////////////////////////////////////////////////////////////
			// SBC : V flag not valid in decimal mode
			////////////////////////////////////////////////////////////////////


            				
		 	////////////////////////////////////////////////////////////////////
			// STA : check memory
			////////////////////////////////////////////////////////////////////
            $display("");
            resetState();
            setMem(16'hc000, 8'h20);
            setA(8'h69);
            $write("%40s", "STA $C000 [check memory] :");
            //      (bytes,      ps,    pc, a,     x,     y,     sp,    #, ck, addr,     #, val)
            checkMem(48'hC0008D, 8'h20, 3,  8'h69, 8'h00, 8'h00, 8'hff, 1, 4,  16'hc000, 1, 48'h69);
            				
		 	////////////////////////////////////////////////////////////////////
			// STX : check memory
			////////////////////////////////////////////////////////////////////
            resetState();
            setMem(16'hc000, 8'h20);
            setX(8'h70);
            $write("%40s", "STX $C000 [check memory] :");
            //      (bytes,      ps,    pc, a,     x,     y,     sp,    #, ck, addr,     #, val)
            checkMem(48'hC0008E, 8'h20, 3,  8'h00, 8'h70, 8'h00, 8'hff, 1, 4,  16'hc000, 1, 48'h70);
            				
		 	////////////////////////////////////////////////////////////////////
			// STY : check memory
			////////////////////////////////////////////////////////////////////
            resetState();
            setMem(16'hc000, 8'h20);
            setY(8'h71);
            $write("%40s", "STY $C000 [check memory] :");
            //      (bytes,      ps,    pc, a,     x,     y,     sp,    #, ck, addr,     #, val)
            checkMem(48'hC0008C, 8'h20, 3,  8'h00, 8'h00, 8'h71, 8'hff, 1, 4,  16'hc000, 1, 48'h71);

						
		end
	endtask


	
	
	////////////////////////////////////////////////////////////////////////////
	// Test all the absolute-indexed-with-X operations
	////////////////////////////////////////////////////////////////////////////
	task testAddrAbsX;
		begin
			$display("\nTest absolute addressing indexed with X:");
								
		 	////////////////////////////////////////////////////////////////////
			// ADC : check no flags
			////////////////////////////////////////////////////////////////////
            $display("");
            resetState();
            setMem(16'hc004, 8'h69);
            setX(4);
            $write("%40s", "ADC $C000,X [bin, no flags] :");
            //   (bytes,      ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hc0007D, 8'h20, 3,  8'h69, 8'h04, 8'h00, 8'hff, 1, 4);	
								
		 	////////////////////////////////////////////////////////////////////
			// ADC : check no flags, x-page
			////////////////////////////////////////////////////////////////////
            resetState();
            setMem(16'hD003, 8'h36);
            setX(4);
            $write("%40s", "ADC $CFFF,X [bin, no flags, x-page] :");
            //   (bytes,      ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hcfff7D, 8'h20, 3,  8'h36, 8'h04, 8'h00, 8'hff, 1, 5);	

            				
		 	////////////////////////////////////////////////////////////////////
			// ADC : check V,C
			////////////////////////////////////////////////////////////////////
            resetState();
            setA(8'h80);
            setX(4);
            setMem(16'hc004, 8'h85);
            $write("%40s", "ADC $C000,X [bin, check V,C] :");
            //   (bytes,      ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hc0007d, 8'h61, 3,  8'h05, 8'h04, 8'h00, 8'hff, 1, 4);	
            				
		 	////////////////////////////////////////////////////////////////////
			// ADC : check N
			////////////////////////////////////////////////////////////////////
            resetState();
            setX(4);
            setA(8'h50);
            setMem(16'hc004, 8'h85);
            $write("%40s", "ADC $C000,X [bin, check V,N] :");
            //   (bytes,      ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hC0007d, 8'ha0, 3,  8'hd5, 8'h04, 8'h00, 8'hff, 1, 4);	

		 	////////////////////////////////////////////////////////////////////
			// ADC : check Z
			////////////////////////////////////////////////////////////////////
            resetState();
            setX(4);
            setMem(16'hc004, 8'h00);
            $write("%40s", "ADC $C000,X [bin, check Z] :");
            //   (bytes,      ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hC0007d, 8'h22, 3,  8'h00, 8'h04, 8'h00, 8'hff, 1, 4);	


		 	////////////////////////////////////////////////////////////////////
			// ADC : check decimal, no flags
			////////////////////////////////////////////////////////////////////
            $display("");
            resetState();
            setMem(16'hc004, 8'h05);
            setX(4);
            setPS(8'h28);
            $write("%40s", "ADC $C000,X [decimal, check no flags] :");
            //   (bytes,      ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hC0007d, 8'h28, 3,  8'h05, 8'h04, 8'h00, 8'hff, 1, 4);	
            				
		 	////////////////////////////////////////////////////////////////////
			// ADC : check decimal, C
			////////////////////////////////////////////////////////////////////
            resetState();
            setPS(8'h28);
            setMem(16'hc004, 8'h85);
            setX(4);
            setA(8'h80);
            $write("%40s", "ADC $C000,X [decimal, check V,C] :");
            //   (bytes,      ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hC0007d, 8'h69, 3,  8'h65, 8'h04, 8'h00, 8'hff, 1, 4);	
            				
		 	////////////////////////////////////////////////////////////////////
			// ADC : check decimal, V,N
			////////////////////////////////////////////////////////////////////
            resetState();
            setPS(8'h28);
            setMem(16'hc004, 8'h40);
            setX(4);
            setA(8'h40);
            $write("%40s", "ADC $C000,X [decimal, check V,N] :");
            //   (bytes,      ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hC0007d, 8'he8, 3,  8'h80, 8'h04, 8'h00, 8'hff, 1, 4);	
            				
		 	////////////////////////////////////////////////////////////////////
			// ADC : check decimal, Z
			////////////////////////////////////////////////////////////////////
            resetState();
            setMem(16'hc004, 8'h00);
            setX(4);
            setPS(8'h28);
            $write("%40s", "ADC $C000,X [decimal, check Z] :");
            //   (bytes,      ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hC0007d, 8'h2a, 3,  8'h00, 8'h04, 8'h00, 8'hff, 1, 4);	
            	
            	
            				
		 	////////////////////////////////////////////////////////////////////
			// AND : check no flags
			////////////////////////////////////////////////////////////////////
            $display("");
            resetState();
            setMem(16'hc004, 8'h40);
            setX(4);
            setA(8'h40);
            $write("%40s", "AND $C000,X [check no flags] :");
            //   (bytes,      ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hC0003d, 8'h20, 3,  8'h40, 8'h04, 8'h00, 8'hff, 1, 4);	
            				
		 	////////////////////////////////////////////////////////////////////
			// AND : check z
			////////////////////////////////////////////////////////////////////
            resetState();
            setA(8'h20);
            setX(4);
            setMem(16'hc004, 8'h40);
            $write("%40s", "AND $C000,X [check Z] :");
            //   (bytes,      ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hC0003d, 8'h22, 3,  8'h00, 8'h04, 8'h00, 8'hff, 1, 4);	
            				
		 	////////////////////////////////////////////////////////////////////
			// AND : check n
			////////////////////////////////////////////////////////////////////
            resetState();
            setA(8'hff);
            setX(4);
            setMem(16'hc004, 8'h80);
            $write("%40s", "AND $C000,X [check N] :");
            //   (bytes,      ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hC0003d, 8'ha0, 3,  8'h80, 8'h04, 8'h00, 8'hff, 1, 4);	
		

			
		 	////////////////////////////////////////////////////////////////////
			// ASL : check no flags
			////////////////////////////////////////////////////////////////////
            $display("");
            resetState();
            setX(4);
            setMem(16'hc004, 8'h02);
            $write("%40s", "ASL $C000,X [check no flags] :");
            //      (bytes,      ps,    pc, a,     x,     y,     sp,    #, ck, addr,  #, val)
            checkMem(48'hc0001e, 8'h20, 3,  8'h00, 8'h04, 8'h00, 8'hff, 1, 7, 16'hc004, 1, 48'h04);

		 	////////////////////////////////////////////////////////////////////
			// ASL : check N
			////////////////////////////////////////////////////////////////////
            resetState();
            setMem(16'hc004, 8'h40);
            setX(4);
            $write("%40s", "ASL $C000,X [check N] :");
            //      (bytes,      ps,    pc, a,     x,     y,     sp,    #, ck, addr,  #, val)
            checkMem(48'hc0001e, 8'ha0, 3,  8'h00, 8'h04, 8'h00, 8'hff, 1, 7, 16'hc004, 1, 48'h80);

		 	////////////////////////////////////////////////////////////////////
			// ASL : check C, Z
			////////////////////////////////////////////////////////////////////
            resetState();
            setMem(16'hc004, 8'h80);
            setX(4);
            $write("%40s", "ASL $C000,X [check C, Z] :");
            //      (bytes,      ps,    pc, a,     x,     y,     sp,    #, ck, addr,  #, val)
            checkMem(48'hc0001e, 8'h23, 3,  8'h00, 8'h04, 8'h00, 8'hff, 1, 7, 16'hc004, 1, 48'h00);
			
			



		 	////////////////////////////////////////////////////////////////////
			// CMP : check no flags
			////////////////////////////////////////////////////////////////////
            $display("");
            resetState();
            setMem(16'hc004, 8'h00);
            setX(4);
            setA(5);
            $write("%40s", "CMP $C000,X [check C] :");
            //   (bytes,      ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hC000DD, 8'h21, 3,  8'h05, 8'h04, 8'h00, 8'hff, 1, 4);	
 								
		 	////////////////////////////////////////////////////////////////////
			// CMP : check no flags, x-page
			////////////////////////////////////////////////////////////////////
            resetState();
            setMem(16'hD003, 8'h00);
            setX(4);
            setA(5);
            $write("%40s", "CMP $C000,X [check C, xpage] :");
            //   (bytes,      ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hcfffDD, 8'h21, 3,  8'h05, 8'h04, 8'h00, 8'hff, 1, 5);	
           				
		 	////////////////////////////////////////////////////////////////////
			// CMP : check N
			////////////////////////////////////////////////////////////////////
            resetState();
            setMem(16'hc004, 8'h05);
            setX(4);
            $write("%40s", "CMP $C000,X [check N] :");
            //   (bytes,      ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hC000DD, 8'ha0, 3,  8'h00, 8'h04, 8'h00, 8'hff, 1, 4);	
            				
		 	////////////////////////////////////////////////////////////////////
			// CMP : check Z
			////////////////////////////////////////////////////////////////////
            resetState();
            setMem(16'hc004, 8'h05);
            setX(4);
            setA(5);
            $write("%40s", "CMP $C000,X [check Z] :");
            //   (bytes,      ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hC000DD, 8'h23, 3,  8'h05, 8'h04, 8'h00, 8'hff, 1, 4);	
             	


		 	////////////////////////////////////////////////////////////////////
			// DEC : check no flags
			////////////////////////////////////////////////////////////////////
            $display("");
            resetState();
            setX(4);
            setMem(16'hc004, 8'h42);
            $write("%40s", "DEC $C000,X [check no flags] :");
            //      (bytes,      ps,    pc, a,     x,     y,     sp,    #, ck, addr,     #, val)
            checkMem(48'hC000DE, 8'h20, 3,  8'h00, 8'h04, 8'h00, 8'hff, 1, 7,  16'hc004, 1, 48'h41);

		 	////////////////////////////////////////////////////////////////////
			// DEC : check Z
			////////////////////////////////////////////////////////////////////
            resetState();
            setMem(16'hc004, 8'h1);
            setX(4);
            $write("%40s", "DEC $C000,X [check Z] :");
            //      (bytes,      ps,    pc, a,     x,     y,     sp,    #, ck, addr,     #, val)
            checkMem(48'hC000DE, 8'h22, 3,  8'h00, 8'h04, 8'h00, 8'hff, 1, 7,  16'hc004, 1, 48'h00);

		 	////////////////////////////////////////////////////////////////////
			// DEC : check N
			////////////////////////////////////////////////////////////////////
            resetState();
            setMem(16'hc004, 8'hff);
            setX(4);
            $write("%40s", "DEC $C000,X [check C] :");
            //      (bytes,      ps,    pc, a,     x,     y,     sp,    #, ck, addr,     #, val)
            checkMem(48'hC000DE, 8'ha0, 3,  8'h00, 8'h04, 8'h00, 8'hff, 1, 7,  16'hc004, 1, 48'hfe);

            				
            	
            				
		 	////////////////////////////////////////////////////////////////////
			// EOR : check Z
			////////////////////////////////////////////////////////////////////
            $display("");
            resetState();
            setMem(16'hc004, 8'h40);
            setX(4);
            setA(8'h40);
            $write("%40s", "EOR $C000,X [check Z] :");
            //   (bytes,      ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hC0005D, 8'h22, 3,  8'h00, 8'h04, 8'h00, 8'hff, 1, 4);	
 								
		 	////////////////////////////////////////////////////////////////////
			// ADC : check Z, x-page
			////////////////////////////////////////////////////////////////////
            resetState();
            setMem(16'hD003, 8'h40);
            setX(4);
            setA(8'h40);
            $write("%40s", "EOR $CFFF,X [check Z, x-page] :");
            //   (bytes,      ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hcfff5D, 8'h22, 3,  8'h00, 8'h04, 8'h00, 8'hff, 1, 5);	
           				
		 	////////////////////////////////////////////////////////////////////
			// EOR : check no flags
			////////////////////////////////////////////////////////////////////
            resetState();
            setMem(16'hc004, 8'h40);
            setX(4);
            setA(8'h20);
            $write("%40s", "EOR $C000,X [check no flags] :");
            //   (bytes,      ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hC0005D, 8'h20, 3,  8'h60, 8'h04, 8'h00, 8'hff, 1, 4);	
            				
		 	////////////////////////////////////////////////////////////////////
			// EOR : check n
			////////////////////////////////////////////////////////////////////
            resetState();
            setMem(16'hc004, 8'h7f);
            setX(4);
            setA(8'hff);
            $write("%40s", "EOR $C000,X [check N] :");
            //   (bytes,      ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hC0005D, 8'ha0, 3,  8'h80, 8'h04, 8'h00, 8'hff, 1, 4);	


             	

		 	////////////////////////////////////////////////////////////////////
			// INC : check no flags
			////////////////////////////////////////////////////////////////////
            $display("");
            resetState();
            setX(4);
            setMem(16'hc004, 8'h42);
            $write("%40s", "INC $C000,X [check no flags] :");
            //      (bytes,      ps,    pc, a,     x,     y,     sp,    #, ck, addr,     #, val)
            checkMem(48'hC000FE, 8'h20, 3,  8'h00, 8'h04, 8'h00, 8'hff, 1, 7,  16'hc004, 1, 48'h43);

		 	////////////////////////////////////////////////////////////////////
			// INC : check Z
			////////////////////////////////////////////////////////////////////
            resetState();
            setX(4);
            setMem(16'hc004, 8'hff);
            $write("%40s", "INC $C000,X [check Z] :");
            //      (bytes,      ps,    pc, a,     x,     y,     sp,    #, ck, addr,     #, val)
            checkMem(48'hC000FE, 8'h22, 3,  8'h00, 8'h04, 8'h00, 8'hff, 1, 7,  16'hc004, 1, 48'h00);

		 	////////////////////////////////////////////////////////////////////
			// INC : check N
			////////////////////////////////////////////////////////////////////
            resetState();
            setX(4);
            setMem(16'hc004, 8'h7f);
            $write("%40s", "INC $C000,X [check C] :");
            //      (bytes,      ps,    pc, a,     x,     y,     sp,    #, ck, addr,     #, val)
            checkMem(48'hC000FE, 8'ha0, 3,  8'h00, 8'h04, 8'h00, 8'hff, 1, 7,  16'hc004, 1, 48'h80);



            				
		 	////////////////////////////////////////////////////////////////////
			// LDA : check no flags
			////////////////////////////////////////////////////////////////////
            $display("");
            resetState();
            setX(4);
            setMem(16'hc004, 8'h40);
            $write("%40s", "LDA $C000,X [check no flags] :");
            //   (bytes,      ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hC000BD, 8'h20, 3,  8'h40, 8'h04, 8'h00, 8'hff, 1, 4);	
  								
		 	////////////////////////////////////////////////////////////////////
			// LDA : check no flags, x-page
			////////////////////////////////////////////////////////////////////
            resetState();
            setMem(16'hD003, 8'h40);
            setX(4);
            $write("%40s", "LDA $CFFF,X [check no flags, x-page] :");
            //   (bytes,      ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hcfffBD, 8'h20, 3,  8'h40, 8'h04, 8'h00, 8'hff, 1, 5);	
           				
		 	////////////////////////////////////////////////////////////////////
			// LDA : check z
			////////////////////////////////////////////////////////////////////
            resetState();
            setA(8'h20);
            setX(4);
            setMem(16'hc004, 8'h00);
            $write("%40s", "LDA $C000,X [check Z] :");
            //   (bytes,      ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hC000BD, 8'h22, 3,  8'h00, 8'h04, 8'h00, 8'hff, 1, 4);	
            				
		 	////////////////////////////////////////////////////////////////////
			// LDA : check n
			////////////////////////////////////////////////////////////////////
            resetState();
            setMem(16'hc004, 8'h80);
            setX(4);
            $write("%40s", "LDA $C000,X [check N] :");
            //   (bytes,      ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hC000BD, 8'ha0, 3,  8'h80, 8'h04, 8'h00, 8'hff, 1, 4);	

             	
            	
            				
		 	////////////////////////////////////////////////////////////////////
			// LDY : check no flags
			////////////////////////////////////////////////////////////////////
            $display("");
            resetState();
            setX(4);
            setMem(16'hc004, 8'h40);
            $write("%40s", "LDY $C000,X [check no flags] :");
            //   (bytes,      ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hC000BC, 8'h20, 3,  8'h00, 8'h04, 8'h40, 8'hff, 1, 4);	
   								
		 	////////////////////////////////////////////////////////////////////
			// LDY : check no flags, x-page
			////////////////////////////////////////////////////////////////////
            resetState();
            setMem(16'hD003, 8'h40);
            setX(4);
            $write("%40s", "LDY $CFFF,X [check no flags, x-page] :");
            //   (bytes,      ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hcfffBC, 8'h20, 3,  8'h00, 8'h04, 8'h40, 8'hff, 1, 5);	
           				
		 	////////////////////////////////////////////////////////////////////
			// LDY : check z
			////////////////////////////////////////////////////////////////////
            resetState();
            setMem(16'hc004, 8'h00);
            setX(4);
            setY(8'h20);
            $write("%40s", "LDY $C000,X [check Z] :");
            //   (bytes,      ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hC000BC, 8'h22, 3,  8'h00, 8'h04, 8'h00, 8'hff, 1, 4);	
            				
		 	////////////////////////////////////////////////////////////////////
			// LDY : check n
			////////////////////////////////////////////////////////////////////
            resetState();
            setX(4);
            setMem(16'hc004, 8'h80);
            $write("%40s", "LDY $C000,X [check N] :");
            //   (bytes,      ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hC000BC, 8'ha0, 3,  8'h00, 8'h04, 8'h80, 8'hff, 1, 4);	
             	


		 	////////////////////////////////////////////////////////////////////
			// LSR : check no flags
			////////////////////////////////////////////////////////////////////
            $display("");
            resetState();
            setX(4);
            setMem(16'hc004, 8'h02);
            $write("%40s", "LSR $C000,X [check no flags] :");
            //      (bytes,      ps,    pc, a,     x,     y,     sp,    #, ck, addr,    #, val)
            checkMem(48'hc0005e, 8'h20, 3,  8'h00, 8'h04, 8'h00, 8'hff, 1, 7, 16'hc004, 1, 48'h01);

		 	////////////////////////////////////////////////////////////////////
			// LSR : check C, Z
			////////////////////////////////////////////////////////////////////
            resetState();
            setX(4);
            setMem(16'hc004, 8'h1);
            $write("%40s", "LSR $C000,X [check C, Z] :");
            //      (bytes,      ps,    pc, a,     x,     y,     sp,    #, ck, addr,    #, val)
            checkMem(48'hc0005e, 8'h23, 3,  8'h00, 8'h04, 8'h00, 8'hff, 1, 7, 16'hc004, 1, 48'h00);

            	
            				
		 	////////////////////////////////////////////////////////////////////
			// ORA : check no flags
			////////////////////////////////////////////////////////////////////
            $display("");
            resetState();
            setMem(16'hc004, 8'h20);
            setX(4);
            setA(8'h40);
            $write("%40s", "ORA $C000,X [check no flags] :");
            //   (bytes,      ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hC0001D, 8'h20, 3,  8'h60, 8'h04, 8'h00, 8'hff, 1, 4);	
   								
		 	////////////////////////////////////////////////////////////////////
			// ORA : check no flags, x-page
			////////////////////////////////////////////////////////////////////
            resetState();
            setMem(16'hD003, 8'h20);
            setX(4);
            setA(8'h40);
            $write("%40s", "ORA $CFFF,X [check no flags, x-page] :");
            //   (bytes,      ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hcfff1D, 8'h20, 3,  8'h60, 8'h04, 8'h00, 8'hff, 1, 5);	
            				
		 	////////////////////////////////////////////////////////////////////
			// ORA : check z
			////////////////////////////////////////////////////////////////////
            resetState();
            setMem(16'hc004, 8'h00);
            setX(4);
            $write("%40s", "ORA $C000,X [check Z] :");
            //   (bytes,      ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hC0001D, 8'h22, 3,  8'h00, 8'h04, 8'h00, 8'hff, 1, 4);	
            				
		 	////////////////////////////////////////////////////////////////////
			// ORA : check n
			////////////////////////////////////////////////////////////////////
            resetState();
            setMem(16'hc004, 8'h80);
            setA(8'h7f);
            setX(4);
            $write("%40s", "ORA $C000,X [check N] :");
            //   (bytes,      ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hC0001D, 8'ha0, 3,  8'hff, 8'h04, 8'h00, 8'hff, 1, 4);	            				
           				


		 	////////////////////////////////////////////////////////////////////
			// ROL : check no flags
			////////////////////////////////////////////////////////////////////
            $display("");
            resetState();
            setX(4);
            setMem(16'hc004, 8'h02);
            $write("%40s", "ROL $C000,X [check no flags] :");
            //      (bytes,      ps,    pc, a,     x,     y,     sp,    #, ck, addr,    #, val)
            checkMem(48'hc0003E, 8'h20, 3,  8'h00, 8'h04, 8'h00, 8'hff, 1, 7, 16'hc004, 1, 48'h04);

		 	////////////////////////////////////////////////////////////////////
			// ROL : check N, C
			////////////////////////////////////////////////////////////////////
            resetState();
            setX(4);
            setMem(16'hc004, 8'hc0);
            $write("%40s", "ROL $C000,X [check N, C] :");
            //      (bytes,      ps,    pc, a,     x,     y,     sp,    #, ck, addr,    #, val)
            checkMem(48'hc0003E, 8'ha1, 3,  8'h00, 8'h04, 8'h00, 8'hff, 1, 7, 16'hc004, 1, 48'h80);

		 	////////////////////////////////////////////////////////////////////
			// ROL : check C, Z
			////////////////////////////////////////////////////////////////////
            resetState();
            setX(4);
            setMem(16'hc004, 8'h80);
            $write("%40s", "ROL $C000,X [check C, Z] :");
            //      (bytes,      ps,    pc, a,     x,     y,     sp,    #, ck, addr,    #, val)
            checkMem(48'hc0003E, 8'h23, 3,  8'h00, 8'h04, 8'h00, 8'hff, 1, 7, 16'hc004, 1, 48'h00);




		 	////////////////////////////////////////////////////////////////////
			// ROR : check no flags
			////////////////////////////////////////////////////////////////////
            $display("");
            resetState();
            setX(4);
            setMem(16'hc004, 8'h02);
            $write("%40s", "ROR $C000,X [check no flags] :");
            //      (bytes,      ps,    pc, a,     x,     y,     sp,    #, ck, addr,    #, val)
            checkMem(48'hc0007E, 8'h20, 3,  8'h00, 8'h04, 8'h00, 8'hff, 1, 7, 16'hc004, 1, 48'h01);

		 	////////////////////////////////////////////////////////////////////
			// ROR : check N, C
			////////////////////////////////////////////////////////////////////
            resetState();
            setPS(8'h21);
            setX(4);
            setMem(16'hc004, 8'hc0);
            $write("%40s", "ROR $C000,X [check N, C] :");
            //      (bytes,      ps,    pc, a,     x,     y,     sp,    #, ck, addr,    #, val)
            checkMem(48'hc0007E, 8'ha0, 3,  8'h00, 8'h04, 8'h00, 8'hff, 1, 7, 16'hc004, 1, 48'he0);

		 	////////////////////////////////////////////////////////////////////
			// ROR : check C, Z
			////////////////////////////////////////////////////////////////////
            resetState();
            setX(4);
	        setMem(16'hc004, 8'h1);
            $write("%40s", "ROR $C000,X [check C, Z] :");
            //      (bytes,      ps,    pc, a,     x,     y,     sp,    #, ck, addr,    #, val)
            checkMem(48'hc0007E, 8'h23, 3,  8'h00, 8'h04, 8'h00, 8'hff, 1, 7, 16'hc004, 1, 48'h00);


			

		 	////////////////////////////////////////////////////////////////////
			// SBC : check no flags
			////////////////////////////////////////////////////////////////////
            $display("");
            resetState();
            setX(4);
            setMem(16'hc004, 8'h85);
            $write("%40s", "SBC $C000,X [binary, check no flags] :");
            //   (bytes,      ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hC000FD, 8'h20, 3,  8'h7a, 8'h04, 8'h00, 8'hff, 1, 4);	
								
		 	////////////////////////////////////////////////////////////////////
			// SBC : check no flags, x-page
			////////////////////////////////////////////////////////////////////
            resetState();
            setMem(16'hD003, 8'h85);
            setX(4);
            $write("%40s", "SBC $CFFF,X [bin, no flags, x-page] :");
            //   (bytes,      ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hcfffFD, 8'h20, 3,  8'h7a, 8'h04, 8'h00, 8'hff, 1, 5);	

		 	////////////////////////////////////////////////////////////////////
			// SBC : check C
			////////////////////////////////////////////////////////////////////
            resetState();
            setA(8'h0b);
            setX(4);
            setMem(16'hc004, 8'h05);
            $write("%40s", "SBC $C000,X [binary, check C] :");
            //   (bytes,      ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hC000FD, 8'h21, 3,  8'h05, 8'h04, 8'h00, 8'hff, 1, 4);	
         				
		 	////////////////////////////////////////////////////////////////////
			// SBC : check V
			////////////////////////////////////////////////////////////////////
            resetState();
            setMem(16'hc004, 8'h05);
            setX(4);
            setA(8'h80);
            $write("%40s", "SBC $C000,X [binary, check V] :");
            //   (bytes,      ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hC000FD, 8'h61, 3,  8'h7a, 8'h04, 8'h00, 8'hff, 1, 4);	
           				
		 	////////////////////////////////////////////////////////////////////
			// SBC : check N
			////////////////////////////////////////////////////////////////////
            resetState();
            setX(4);
            setMem(16'hc004, 8'h00);
            $write("%40s", "SBC $C000,X [binary, check N] :");
            //   (bytes,      ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hC000FD, 8'ha0, 3,  8'hff, 8'h04, 8'h00, 8'hff, 1, 4);	   
                    				
		 	////////////////////////////////////////////////////////////////////
			// SBC : check Z
			////////////////////////////////////////////////////////////////////
            resetState();
            setX(4);
            setMem(16'hc004, 8'h00);
            setA(1);
            $write("%40s", "SBC $C000,X [binary, check Z] :");
            //   (bytes,      ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hC000FD, 8'h23, 3,  8'h00, 8'h04, 8'h00, 8'hff, 1, 4);            				
   
   
		 	////////////////////////////////////////////////////////////////////
			// SBC :  decimal, check no flags
			////////////////////////////////////////////////////////////////////
            $display("");
            resetState();
            setX(4);
            setMem(16'hc004, 8'h85);
            setPS(8'h28);
            $write("%40s", "SBC $C000,X [decimal, check no flags] :");
            //   (bytes,      ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hC000FD, 8'h28, 3,  8'h14, 8'h04, 8'h00, 8'hff, 1, 4);	
            				
		 	////////////////////////////////////////////////////////////////////
			// SBC :  decimal, check N
			////////////////////////////////////////////////////////////////////
            resetState();
            resetState();
            setX(4);
            setPS(8'h28);
            setMem(16'hc004, 8'h05);
            $write("%40s", "SBC $C000,X [decimal, check N] :");
            //   (bytes,      ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hC000FD, 8'ha8, 3,  8'h94, 8'h04, 8'h00, 8'hff, 1, 4);	
             				           				
		 	////////////////////////////////////////////////////////////////////
			// SBC :  decimal, Z
			////////////////////////////////////////////////////////////////////
            resetState();
            setPS(8'h28);
            setX(4);
            setMem(16'hc004, 8'h00);
            setA(1);
            $write("%40s", "SBC $C000,X [decimal, check Z,C] :");
            //   (bytes,      ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hC000FD, 8'h2b, 3,  8'h00, 8'h04, 8'h00, 8'hff, 1, 4);	
				
		 	////////////////////////////////////////////////////////////////////
			// SBC : V flag not valid in decimal mode
			////////////////////////////////////////////////////////////////////


            				
		 	////////////////////////////////////////////////////////////////////
			// STA : check memory
			////////////////////////////////////////////////////////////////////
            $display("");
            resetState();
            setMem(16'hc004, 8'h20);
            setX(4);
            setA(8'h69);
            $write("%40s", "STA $C000,X [check memory] :");
            //      (bytes,      ps,    pc, a,     x,     y,     sp,    #, ck, addr,     #, val)
            checkMem(48'hC0009D, 8'h20, 3,  8'h69, 8'h04, 8'h00, 8'hff, 1, 4,  16'hc000, 1, 48'h69);
							
		end
	endtask

	
	
	////////////////////////////////////////////////////////////////////////////
	// Test all the absolute-indexed-with-Y operations
	////////////////////////////////////////////////////////////////////////////
	task testAddrAbsY;
		begin
			$display("\nTest absolute addressing indexed with Y:");
								
								
		 	////////////////////////////////////////////////////////////////////
			// ADC : check no flags
			////////////////////////////////////////////////////////////////////
            $display("");
            resetState();
            setMem(16'hc004, 8'h69);
            setY(4);
            $write("%40s", "ADC $C000,Y [bin, no flags] :");
            //   (bytes,      ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hc00079, 8'h20, 3,  8'h69, 8'h00, 8'h04, 8'hff, 1, 4);	
								
		 	////////////////////////////////////////////////////////////////////
			// ADC : check no flags, x-page
			////////////////////////////////////////////////////////////////////
            resetState();
            setMem(16'hD003, 8'h36);
            setY(4);
            $write("%40s", "ADC $CFFF,Y [bin, no flags, x-page] :");
            //   (bytes,      ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hcfff79, 8'h20, 3,  8'h36, 8'h00, 8'h04, 8'hff, 1, 5);	

            				
		 	////////////////////////////////////////////////////////////////////
			// ADC : check V,C
			////////////////////////////////////////////////////////////////////
            resetState();
            setA(8'h80);
            setY(4);
            setMem(16'hc004, 8'h85);
            $write("%40s", "ADC $C000,Y [bin, check V,C] :");
            //   (bytes,      ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hc00079, 8'h61, 3,  8'h05, 8'h00, 8'h04, 8'hff, 1, 4);	
            				
		 	////////////////////////////////////////////////////////////////////
			// ADC : check N
			////////////////////////////////////////////////////////////////////
            resetState();
            setY(4);
            setA(8'h50);
            setMem(16'hc004, 8'h85);
            $write("%40s", "ADC $C000,Y [bin, check V,N] :");
            //   (bytes,      ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hc00079, 8'ha0, 3,  8'hd5, 8'h00, 8'h04, 8'hff, 1, 4);	

		 	////////////////////////////////////////////////////////////////////
			// ADC : check Z
			////////////////////////////////////////////////////////////////////
            resetState();
            setY(4);
            setMem(16'hc004, 8'h00);
            $write("%40s", "ADC $C000,Y [bin, check Z] :");
            //   (bytes,      ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hc00079, 8'h22, 3,  8'h00, 8'h00, 8'h04, 8'hff, 1, 4);	


		 	////////////////////////////////////////////////////////////////////
			// ADC : check decimal, no flags
			////////////////////////////////////////////////////////////////////
            $display("");
            resetState();
            setMem(16'hc004, 8'h05);
            setY(4);
            setPS(8'h28);
            $write("%40s", "ADC $C000,Y [decimal, check no flags] :");
            //   (bytes,      ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hc00079, 8'h28, 3,  8'h05, 8'h00, 8'h04, 8'hff, 1, 4);	
            				
		 	////////////////////////////////////////////////////////////////////
			// ADC : check decimal, C
			////////////////////////////////////////////////////////////////////
            resetState();
            setPS(8'h28);
            setMem(16'hc004, 8'h85);
            setY(4);
            setA(8'h80);
            $write("%40s", "ADC $C000,Y [decimal, check V,C] :");
            //   (bytes,      ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hc00079, 8'h69, 3,  8'h65, 8'h00, 8'h04, 8'hff, 1, 4);	
            				
		 	////////////////////////////////////////////////////////////////////
			// ADC : check decimal, V,N
			////////////////////////////////////////////////////////////////////
            resetState();
            setPS(8'h28);
            setMem(16'hc004, 8'h40);
            setY(4);
            setA(8'h40);
            $write("%40s", "ADC $C000,Y [decimal, check V,N] :");
            //   (bytes,      ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hc00079, 8'he8, 3,  8'h80, 8'h00, 8'h04, 8'hff, 1, 4);	
            				
		 	////////////////////////////////////////////////////////////////////
			// ADC : check decimal, Z
			////////////////////////////////////////////////////////////////////
            resetState();
            setMem(16'hc004, 8'h00);
            setY(4);
            setPS(8'h28);
            $write("%40s", "ADC $C000,Y [decimal, check Z] :");
            //   (bytes,      ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hc00079, 8'h2a, 3,  8'h00, 8'h00, 8'h04, 8'hff, 1, 4);	

            	
            	
            				
		 	////////////////////////////////////////////////////////////////////
			// AND : check no flags
			////////////////////////////////////////////////////////////////////
            $display("");
            resetState();
            setMem(16'hc004, 8'h40);
            setY(4);
            setA(8'h40);
            $write("%40s", "AND $C000,Y [check no flags] :");
            //   (bytes,      ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hC00039, 8'h20, 3,  8'h40, 8'h00, 8'h04, 8'hff, 1, 4);	
            				
		 	////////////////////////////////////////////////////////////////////
			// AND : check z
			////////////////////////////////////////////////////////////////////
            resetState();
            setA(8'h20);
            setY(4);
            setMem(16'hc004, 8'h40);
            $write("%40s", "AND $C000,Y [check Z] :");
            //   (bytes,      ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hC00039, 8'h22, 3,  8'h00, 8'h00, 8'h04, 8'hff, 1, 4);	
            				
		 	////////////////////////////////////////////////////////////////////
			// AND : check n
			////////////////////////////////////////////////////////////////////
            resetState();
            setA(8'hff);
            setY(4);
            setMem(16'hc004, 8'h80);
            $write("%40s", "AND $C000,Y [check N] :");
            //   (bytes,      ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hC00039, 8'ha0, 3,  8'h80, 8'h00, 8'h04, 8'hff, 1, 4);	
			
			



		 	////////////////////////////////////////////////////////////////////
			// CMP : check no flags
			////////////////////////////////////////////////////////////////////
            $display("");
            resetState();
            setMem(16'hc004, 8'h00);
            setY(4);
            setA(5);
            $write("%40s", "CMP $C000,Y [check C] :");
            //   (bytes,      ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hC000D9, 8'h21, 3,  8'h05, 8'h00, 8'h04, 8'hff, 1, 4);	
 								
		 	////////////////////////////////////////////////////////////////////
			// CMP : check no flags, x-page
			////////////////////////////////////////////////////////////////////
            resetState();
            setMem(16'hD003, 8'h00);
            setY(4);
            setA(5);
            $write("%40s", "CMP $C000,Y [check C, xpage] :");
            //   (bytes,      ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hcfffD9, 8'h21, 3,  8'h05, 8'h00, 8'h04, 8'hff, 1, 5);	
           				
		 	////////////////////////////////////////////////////////////////////
			// CMP : check N
			////////////////////////////////////////////////////////////////////
            resetState();
            setMem(16'hc004, 8'h05);
            setY(4);
            $write("%40s", "CMP $C000,Y [check N] :");
            //   (bytes,      ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hC000D9, 8'ha0, 3,  8'h00, 8'h00, 8'h04, 8'hff, 1, 4);	
            				
		 	////////////////////////////////////////////////////////////////////
			// CMP : check Z
			////////////////////////////////////////////////////////////////////
            resetState();
            setMem(16'hc004, 8'h05);
            setY(4);
            setA(5);
            $write("%40s", "CMP $C000,Y [check Z] :");
            //   (bytes,      ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hC000D9, 8'h23, 3,  8'h05, 8'h00, 8'h04, 8'hff, 1, 4);	

            				
            	
            				
		 	////////////////////////////////////////////////////////////////////
			// EOR : check Z
			////////////////////////////////////////////////////////////////////
            $display("");
            resetState();
            setMem(16'hc004, 8'h40);
            setY(4);
            setA(8'h40);
            $write("%40s", "EOR $C000,Y [check Z] :");
            //   (bytes,      ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hC00059, 8'h22, 3,  8'h00, 8'h00, 8'h04, 8'hff, 1, 4);	
 								
		 	////////////////////////////////////////////////////////////////////
			// ADC : check Z, x-page
			////////////////////////////////////////////////////////////////////
            resetState();
            setMem(16'hD003, 8'h40);
            setY(4);
            setA(8'h40);
            $write("%40s", "EOR $CFFF,Y [check Z, x-page] :");
            //   (bytes,      ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hcfff59, 8'h22, 3,  8'h00, 8'h00, 8'h04, 8'hff, 1, 5);	
           				
		 	////////////////////////////////////////////////////////////////////
			// EOR : check no flags
			////////////////////////////////////////////////////////////////////
            resetState();
            setMem(16'hc004, 8'h40);
            setY(4);
            setA(8'h20);
            $write("%40s", "EOR $C000,Y [check no flags] :");
            //   (bytes,      ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hC00059, 8'h20, 3,  8'h60, 8'h00, 8'h04, 8'hff, 1, 4);	
            				
		 	////////////////////////////////////////////////////////////////////
			// EOR : check n
			////////////////////////////////////////////////////////////////////
            resetState();
            setMem(16'hc004, 8'h7f);
            setY(4);
            setA(8'hff);
            $write("%40s", "EOR $C000,Y [check N] :");
            //   (bytes,      ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hC00059, 8'ha0, 3,  8'h80, 8'h00, 8'h04, 8'hff, 1, 4);	



            				
		 	////////////////////////////////////////////////////////////////////
			// LDA : check no flags
			////////////////////////////////////////////////////////////////////
            $display("");
            resetState();
            setY(4);
            setMem(16'hc004, 8'h40);
            $write("%40s", "LDA $C000,Y [check no flags] :");
            //   (bytes,      ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hC000B9, 8'h20, 3,  8'h40, 8'h00, 8'h04, 8'hff, 1, 4);	
  								
		 	////////////////////////////////////////////////////////////////////
			// LDA : check no flags, x-page
			////////////////////////////////////////////////////////////////////
            resetState();
            setMem(16'hD003, 8'h40);
            setY(4);
            $write("%40s", "LDA $CFFF,Y [check no flags, x-page] :");
            //   (bytes,      ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hcfffB9, 8'h20, 3,  8'h40, 8'h00, 8'h04, 8'hff, 1, 5);	
           				
		 	////////////////////////////////////////////////////////////////////
			// LDA : check z
			////////////////////////////////////////////////////////////////////
            resetState();
            setA(8'h20);
            setY(4);
            setMem(16'hc004, 8'h00);
            $write("%40s", "LDA $C000,Y [check Z] :");
            //   (bytes,      ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hC000B9, 8'h22, 3,  8'h00, 8'h00, 8'h04, 8'hff, 1, 4);	
            				
		 	////////////////////////////////////////////////////////////////////
			// LDA : check n
			////////////////////////////////////////////////////////////////////
            resetState();
            setMem(16'hc004, 8'h80);
            setY(4);
            $write("%40s", "LDA $C000,Y [check N] :");
            //   (bytes,      ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hC000B9, 8'ha0, 3,  8'h80, 8'h00, 8'h04, 8'hff, 1, 4);	

             	
            	
            				
		 	////////////////////////////////////////////////////////////////////
			// LDX : check no flags
			////////////////////////////////////////////////////////////////////
            $display("");
            resetState();
            setY(4);
            setMem(16'hc004, 8'h40);
            $write("%40s", "LDX $C000,Y [check no flags] :");
            //   (bytes,      ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hC000Be, 8'h20, 3,  8'h00, 8'h40, 8'h04, 8'hff, 1, 4);	
   								
		 	////////////////////////////////////////////////////////////////////
			// LDX : check no flags, x-page
			////////////////////////////////////////////////////////////////////
            resetState();
            setMem(16'hD003, 8'h40);
            setY(4);
            $write("%40s", "LDX $CFFF,Y [check no flags, x-page] :");
            //   (bytes,      ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hcfffBe, 8'h20, 3,  8'h00, 8'h40, 8'h04, 8'hff, 1, 5);	
           				
		 	////////////////////////////////////////////////////////////////////
			// LDX : check z
			////////////////////////////////////////////////////////////////////
            resetState();
            setMem(16'hc004, 8'h00);
            setY(4);
            setX(8'h20);
            $write("%40s", "LDX $C000,Y [check Z] :");
            //   (bytes,      ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hC000Be, 8'h22, 3,  8'h00, 8'h00, 8'h04, 8'hff, 1, 4);	
            				
		 	////////////////////////////////////////////////////////////////////
			// LDX : check n
			////////////////////////////////////////////////////////////////////
            resetState();
            setY(4);
            setMem(16'hc004, 8'h80);
            $write("%40s", "LDX $C000,Y [check N] :");
            //   (bytes,      ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hC000Be, 8'ha0, 3,  8'h00, 8'h80, 8'h04, 8'hff, 1, 4);	

            	
            			

		 	////////////////////////////////////////////////////////////////////
			// ORA : check no flags
			////////////////////////////////////////////////////////////////////
            $display("");
            resetState();
            setMem(16'hc004, 8'h20);
            setY(4);
            setA(8'h40);
            $write("%40s", "ORA $C000,Y [check no flags] :");
            //   (bytes,      ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hC00019, 8'h20, 3,  8'h60, 8'h00, 8'h04, 8'hff, 1, 4);	
   								
		 	////////////////////////////////////////////////////////////////////
			// ORA : check no flags, x-page
			////////////////////////////////////////////////////////////////////
            resetState();
            setMem(16'hD003, 8'h20);
            setY(4);
            setA(8'h40);
            $write("%40s", "ORA $CFFF,Y [check no flags, x-page] :");
            //   (bytes,      ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hcfff19, 8'h20, 3,  8'h60, 8'h00, 8'h04, 8'hff, 1, 5);	
            				
		 	////////////////////////////////////////////////////////////////////
			// ORA : check z
			////////////////////////////////////////////////////////////////////
            resetState();
            setMem(16'hc004, 8'h00);
            setY(4);
            $write("%40s", "ORA $C000,Y [check Z] :");
            //   (bytes,      ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hC00019, 8'h22, 3,  8'h00, 8'h00, 8'h04, 8'hff, 1, 4);	
            				
		 	////////////////////////////////////////////////////////////////////
			// ORA : check n
			////////////////////////////////////////////////////////////////////
            resetState();
            setMem(16'hc004, 8'h80);
            setA(8'h7f);
            setY(4);
            $write("%40s", "ORA $C000,Y [check N] :");
            //   (bytes,      ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hC00019, 8'ha0, 3,  8'hff, 8'h00, 8'h04, 8'hff, 1, 4);	            				



			

		 	////////////////////////////////////////////////////////////////////
			// SBC : check no flags
			////////////////////////////////////////////////////////////////////
            $display("");
            resetState();
            setY(4);
            setMem(16'hc004, 8'h85);
            $write("%40s", "SBC $C000,Y [binary, check no flags] :");
            //   (bytes,      ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hC000F9, 8'h20, 3,  8'h7a, 8'h00, 8'h04, 8'hff, 1, 4);	
								
		 	////////////////////////////////////////////////////////////////////
			// SBC : check no flags, x-page
			////////////////////////////////////////////////////////////////////
            resetState();
            setMem(16'hD003, 8'h85);
            setY(4);
            $write("%40s", "SBC $CFFF,Y [bin, no flags, x-page] :");
            //   (bytes,      ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hcfffF9, 8'h20, 3,  8'h7a, 8'h00, 8'h04, 8'hff, 1, 5);	

		 	////////////////////////////////////////////////////////////////////
			// SBC : check C
			////////////////////////////////////////////////////////////////////
            resetState();
            setA(8'h0b);
            setY(4);
            setMem(16'hc004, 8'h05);
            $write("%40s", "SBC $C000,Y [binary, check C] :");
            //   (bytes,      ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hC000F9, 8'h21, 3,  8'h05, 8'h00, 8'h04, 8'hff, 1, 4);	
         				
		 	////////////////////////////////////////////////////////////////////
			// SBC : check V
			////////////////////////////////////////////////////////////////////
            resetState();
            setMem(16'hc004, 8'h05);
            setY(4);
            setA(8'h80);
            $write("%40s", "SBC $C000,Y [binary, check V] :");
            //   (bytes,      ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hC000F9, 8'h61, 3,  8'h7a, 8'h00, 8'h04, 8'hff, 1, 4);	
           				
		 	////////////////////////////////////////////////////////////////////
			// SBC : check N
			////////////////////////////////////////////////////////////////////
            resetState();
            setY(4);
            setMem(16'hc004, 8'h00);
            $write("%40s", "SBC $C000,Y [binary, check N] :");
            //   (bytes,      ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hC000F9, 8'ha0, 3,  8'hff, 8'h00, 8'h04, 8'hff, 1, 4);	   
                    				
		 	////////////////////////////////////////////////////////////////////
			// SBC : check Z
			////////////////////////////////////////////////////////////////////
            resetState();
            setY(4);
            setMem(16'hc004, 8'h00);
            setA(1);
            $write("%40s", "SBC $C000,Y [binary, check Z] :");
            //   (bytes,      ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hC000F9, 8'h23, 3,  8'h00, 8'h00, 8'h04, 8'hff, 1, 4);            				
   
   
		 	////////////////////////////////////////////////////////////////////
			// SBC :  decimal, check no flags
			////////////////////////////////////////////////////////////////////
            $display("");
            resetState();
            setY(4);
            setMem(16'hc004, 8'h85);
            setPS(8'h28);
            $write("%40s", "SBC $C000,Y [decimal, check no flags] :");
            //   (bytes,      ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hC000F9, 8'h28, 3,  8'h14, 8'h00, 8'h04, 8'hff, 1, 4);	
            				
		 	////////////////////////////////////////////////////////////////////
			// SBC :  decimal, check N
			////////////////////////////////////////////////////////////////////
            resetState();
            resetState();
            setY(4);
            setPS(8'h28);
            setMem(16'hc004, 8'h05);
            $write("%40s", "SBC $C000,Y [decimal, check N] :");
            //   (bytes,      ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hC000F9, 8'ha8, 3,  8'h94, 8'h00, 8'h04, 8'hff, 1, 4);	
             				           				
		 	////////////////////////////////////////////////////////////////////
			// SBC :  decimal, Z
			////////////////////////////////////////////////////////////////////
            resetState();
            setPS(8'h28);
            setY(4);
            setMem(16'hc004, 8'h00);
            setA(1);
            $write("%40s", "SBC $C000,Y [decimal, check Z,C] :");
            //   (bytes,      ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hC000F9, 8'h2b, 3,  8'h00, 8'h00, 8'h04, 8'hff, 1, 4);	
				
		 	////////////////////////////////////////////////////////////////////
			// SBC : V flag not valid in decimal mode
			////////////////////////////////////////////////////////////////////


            				
		 	////////////////////////////////////////////////////////////////////
			// STA : check memory
			////////////////////////////////////////////////////////////////////
            $display("");
            resetState();
            setMem(16'hc004, 8'h20);
            setY(4);
            setA(8'h69);
            $write("%40s", "STA $C000,Y [check memory] :");
            //      (bytes,      ps,    pc, a,     x,     y,     sp,    #, ck, addr,     #, val)
            checkMem(48'hC00099, 8'h20, 3,  8'h69, 8'h00, 8'h04, 8'hff, 1, 4,  16'hc000, 1, 48'h69);
	
		end
	endtask
					
					
	
	
	////////////////////////////////////////////////////////////////////////////
	// Test all the zero-page-indexed-with-X operations
	////////////////////////////////////////////////////////////////////////////
	task testZpIndxX;
		begin
			$display("\nTest zero-page indexed indirect with X addressing:");

            				
		 	////////////////////////////////////////////////////////////////////
			// ADC : check no flags
			////////////////////////////////////////////////////////////////////
            $display("");
            resetState();
            setMem(16'hc5, 8'h40);
            setMem(16'hc4, 8'h00);
            setX(4);
            setMem(16'h4000, 8'h42);
            $write("%40s", "ADC ($C0,X) [binary, check no flags] :");
            //   (bytes,    ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hc061, 8'h20, 2,  8'h42, 8'h04, 8'h00, 8'hff, 1, 6);	

		 	////////////////////////////////////////////////////////////////////
			// ADC : check V,C
			////////////////////////////////////////////////////////////////////
            resetState();
            setA(8'h80);
            setMem(16'hc5, 8'h40);
            setMem(16'hc4, 8'h00);
            setX(4);
            setMem(16'h4000, 8'h85);
            $write("%40s", "ADC ($C0,X) [binary, check V,C] :");
            //   (bytes,    ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hc061, 8'h61, 2,  8'h05, 8'h04, 8'h00, 8'hff, 1, 6);	
            				
		 	////////////////////////////////////////////////////////////////////
			// ADC : check N
			////////////////////////////////////////////////////////////////////
            resetState();
            setA(8'h50);
            setMem(16'hc5, 8'h40);
            setMem(16'hc4, 8'h00);
            setX(4);
            setMem(16'h4000, 8'h85);

            $write("%40s", "ADC ($C0,X) [binary, check V,N] :");
            //   (bytes,    ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hC061, 8'ha0, 2,  8'hd5, 8'h04, 8'h00, 8'hff, 1, 6);	

		 	////////////////////////////////////////////////////////////////////
			// ADC : check Z
			////////////////////////////////////////////////////////////////////
            resetState();
            setMem(16'hc5, 8'h40);
            setMem(16'hc4, 8'h00);
            setX(4);
            setMem(16'h4000, 8'h00);
            $write("%40s", "ADC ($C0,X) [binary, check Z] :");
            //   (bytes,    ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hc061, 8'h22, 2,  8'h00, 8'h04, 8'h00, 8'hff, 1, 6);	


		 	////////////////////////////////////////////////////////////////////
			// ADC : check decimal, no flags
			////////////////////////////////////////////////////////////////////
            $display("");
            resetState();
            setMem(16'hc5, 8'h40);
            setMem(16'hc4, 8'h00);
            setX(4);
            setMem(16'h4000, 8'h05);
            setPS(8'h28);
            $write("%40s", "ADC ($C0,X) [decimal, check no flags] :");
            //   (bytes,    ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hc061, 8'h28, 2,  8'h05, 8'h04, 8'h00, 8'hff, 1, 6);	
            				
		 	////////////////////////////////////////////////////////////////////
			// ADC : check decimal, C
			////////////////////////////////////////////////////////////////////
            resetState();
            setPS(8'h28);
            setMem(16'hc5, 8'h40);
            setMem(16'hc4, 8'h00);
            setX(4);
            setMem(16'h4000, 8'h85);
            setA(8'h80);
            $write("%40s", "ADC ($C0,X) [decimal, check V,C] :");
            //   (bytes,    ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hC061, 8'h69, 2,  8'h65, 8'h04, 8'h00, 8'hff, 1, 6);	
            				
		 	////////////////////////////////////////////////////////////////////
			// ADC : check decimal, V,N
			////////////////////////////////////////////////////////////////////
            resetState();
            setPS(8'h28);
            setMem(16'hc5, 8'h40);
            setMem(16'hc4, 8'h00);
            setX(4);
            setMem(16'h4000, 8'h40);
            setA(8'h40);
            $write("%40s", "ADC ($C0,X) [decimal, check V,N] :");
            //   (bytes,    ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hC061, 8'he8, 2,  8'h80, 8'h04, 8'h00, 8'hff, 1, 6);	
            				
		 	////////////////////////////////////////////////////////////////////
			// ADC : check decimal, Z
			////////////////////////////////////////////////////////////////////
            resetState();
            setMem(16'hc5, 8'h40);
            setMem(16'hc4, 8'h00);
            setX(4);
            setMem(16'h4000, 8'h00);
            setPS(8'h28);
            $write("%40s", "ADC ($C0,X) [decimal, check Z] :");
            //   (bytes,    ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hC061, 8'h2a, 2,  8'h00, 8'h04, 8'h00, 8'hff, 1, 6);	

            	
            	
            				
		 	////////////////////////////////////////////////////////////////////
			// AND : check no flags
			////////////////////////////////////////////////////////////////////
            $display("");
            resetState();
            setMem(16'hc5, 8'h40);
            setMem(16'hc4, 8'h00);
            setX(4);
            setMem(16'h4000, 8'h40);
            setA(8'h40);
            $write("%40s", "AND ($C0,X) [check no flags] :");
            //   (bytes,    ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hC021, 8'h20, 2,  8'h40, 8'h04, 8'h00, 8'hff, 1, 6);	
            				
		 	////////////////////////////////////////////////////////////////////
			// AND : check z
			////////////////////////////////////////////////////////////////////
            resetState();
            setA(8'h20);
            setMem(16'hc5, 8'h40);
            setMem(16'hc4, 8'h00);
            setX(4);
            setMem(16'h4000, 8'h40);
            $write("%40s", "AND ($C0,X) [check Z] :");
            //   (bytes,    ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hC021, 8'h22, 2,  8'h00, 8'h04, 8'h00, 8'hff, 1, 6);	
            				
		 	////////////////////////////////////////////////////////////////////
			// AND : check n
			////////////////////////////////////////////////////////////////////
            resetState();
            setA(8'hff);
            setMem(16'hc5, 8'h40);
            setMem(16'hc4, 8'h00);
            setX(4);
            setMem(16'h4000, 8'h80);
            $write("%40s", "AND ($C0,X) [check N] :");
            //   (bytes,    ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hC021, 8'ha0, 2,  8'h80, 8'h04, 8'h00, 8'hff, 1, 6);	

			


		 	////////////////////////////////////////////////////////////////////
			// CMP : check no flags
			////////////////////////////////////////////////////////////////////
            $display("");
            resetState();
            setMem(16'hc5, 8'h40);
            setMem(16'hc4, 8'h00);
            setX(4);
            setMem(16'h4000, 8'h00);
            setA(5);
            $write("%40s", "CMP ($C0,X) [check C] :");
            //   (bytes,    ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hC0C1, 8'h21, 2,  8'h05, 8'h04, 8'h00, 8'hff, 1, 6);	
            				
		 	////////////////////////////////////////////////////////////////////
			// CMP : check N
			////////////////////////////////////////////////////////////////////
            resetState();
            setMem(16'hc5, 8'h40);
            setMem(16'hc4, 8'h00);
            setX(4);
            setMem(16'h4000, 8'h05);
            $write("%40s", "CMP ($C0,X) [check N] :");
            //   (bytes,    ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hC0C1, 8'ha0, 2,  8'h00, 8'h04, 8'h00, 8'hff, 1, 6);	
            				
		 	////////////////////////////////////////////////////////////////////
			// CMP : check Z
			////////////////////////////////////////////////////////////////////
            resetState();
            setMem(16'hc5, 8'h40);
            setMem(16'hc4, 8'h00);
            setX(4);
            setMem(16'h4000, 8'h05);
            setA(5);
            $write("%40s", "CMP ($C0,X) [check Z] :");
            //   (bytes,    ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hC0C1, 8'h23, 2,  8'h05, 8'h04, 8'h00, 8'hff, 1, 6);	
 
            				
            	
            				
		 	////////////////////////////////////////////////////////////////////
			// EOR : check Z
			////////////////////////////////////////////////////////////////////
            $display("");
            resetState();
            setMem(16'hc5, 8'h40);
            setMem(16'hc4, 8'h00);
            setX(4);
            setMem(16'h4000, 8'h40);
            setA(8'h40);
            $write("%40s", "EOR ($C0,X) [check Z] :");
            //   (bytes,    ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hC041, 8'h22, 2,  8'h00, 8'h04, 8'h00, 8'hff, 1, 6);	
            				
		 	////////////////////////////////////////////////////////////////////
			// EOR : check no flags
			////////////////////////////////////////////////////////////////////
            resetState();
            setMem(16'hc5, 8'h40);
            setMem(16'hc4, 8'h00);
            setX(4);
            setMem(16'h4000, 8'h40);
            setA(8'h20);
            $write("%40s", "EOR ($C0,X) [check no flags] :");
            //   (bytes,    ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hC041, 8'h20, 2,  8'h60, 8'h04, 8'h00, 8'hff, 1, 6);	
            				
		 	////////////////////////////////////////////////////////////////////
			// EOR : check n
			////////////////////////////////////////////////////////////////////
            resetState();
            setMem(16'hc5, 8'h40);
            setMem(16'hc4, 8'h00);
            setX(4);
            setMem(16'h4000, 8'h7f);
            setA(8'hff);
            $write("%40s", "EOR ($C0,X) [check N] :");
            //   (bytes,    ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hC041, 8'ha0, 2,  8'h80, 8'h04, 8'h00, 8'hff, 1, 6);	


            	
            				
		 	////////////////////////////////////////////////////////////////////
			// LDA : check no flags
			////////////////////////////////////////////////////////////////////
            $display("");
            resetState();
            setMem(16'hc5, 8'h40);
            setMem(16'hc4, 8'h00);
            setX(4);
            setMem(16'h4000, 8'h40);
            $write("%40s", "LDA ($C0,X) [check no flags] :");
            //   (bytes,    ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hC0A1, 8'h20, 2,  8'h40, 8'h04, 8'h00, 8'hff, 1, 6);	
            				
		 	////////////////////////////////////////////////////////////////////
			// LDA : check z
			////////////////////////////////////////////////////////////////////
            resetState();
            setA(8'h20);
            setMem(16'hc5, 8'h40);
            setMem(16'hc4, 8'h00);
            setX(4);
            setMem(16'h4000, 8'h00);
            $write("%40s", "LDA ($C0,X) [check Z] :");
            //   (bytes,    ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hC0A1, 8'h22, 2,  8'h00, 8'h04, 8'h00, 8'hff, 1, 6);	
            				
		 	////////////////////////////////////////////////////////////////////
			// LDA : check n
			////////////////////////////////////////////////////////////////////
            resetState();
            setMem(16'hc5, 8'h40);
            setMem(16'hc4, 8'h00);
            setX(4);
            setMem(16'h4000, 8'h80);
            $write("%40s", "LDA ($C0,X) [check N] :");
            //   (bytes,    ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hC0A1, 8'ha0, 2,  8'h80, 8'h04, 8'h00, 8'hff, 1, 6);	

            	
            				
		 	////////////////////////////////////////////////////////////////////
			// ORA : check no flags
			////////////////////////////////////////////////////////////////////
            $display("");
            resetState();
            setMem(16'hc5, 8'h40);
            setMem(16'hc4, 8'h00);
            setX(4);
            setMem(16'h4000, 8'h20);
            setA(8'h40);
            $write("%40s", "ORA ($C0,X) [check no flags] :");
            //   (bytes,    ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hC001, 8'h20, 2,  8'h60, 8'h04, 8'h00, 8'hff, 1, 6);	
            				
		 	////////////////////////////////////////////////////////////////////
			// ORA : check z
			////////////////////////////////////////////////////////////////////
            resetState();
            setMem(16'hc5, 8'h40);
            setMem(16'hc4, 8'h00);
            setX(4);
            setMem(16'h4000, 8'h00);
            $write("%40s", "ORA ($C0,X) [check Z] :");
            //   (bytes,    ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hC001, 8'h22, 2,  8'h00, 8'h04, 8'h00, 8'hff, 1, 6);	
            				
		 	////////////////////////////////////////////////////////////////////
			// ORA : check n
			////////////////////////////////////////////////////////////////////
            resetState();
            setMem(16'hc5, 8'h40);
            setMem(16'hc4, 8'h00);
            setX(4);
            setMem(16'h4000, 8'h80);
            setA(8'h7f);
            $write("%40s", "ORA ($C0,X) [check N] :");
            //   (bytes,    ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hC001, 8'ha0, 2,  8'hff, 8'h04, 8'h00, 8'hff, 1, 6);	            				


			

		 	////////////////////////////////////////////////////////////////////
			// SBC : check no flags
			////////////////////////////////////////////////////////////////////
            $display("");
            resetState();
            setMem(16'hc5, 8'h40);
            setMem(16'hc4, 8'h00);
            setX(4);
            setMem(16'h4000, 8'h85);
            $write("%40s", "SBC ($C0,X) [binary, check no flags] :");
            //   (bytes,    ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hC0E1, 8'h20, 2,  8'h7a, 8'h04, 8'h00, 8'hff, 1, 6);	

		 	////////////////////////////////////////////////////////////////////
			// SBC : check C
			////////////////////////////////////////////////////////////////////
            resetState();
            setA(8'h0b);
            setMem(16'hc5, 8'h40);
            setMem(16'hc4, 8'h00);
            setX(4);
            setMem(16'h4000, 8'h05);
            $write("%40s", "SBC ($C0,X) [binary, check C] :");
            //   (bytes,    ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hC0E1, 8'h21, 2,  8'h05, 8'h04, 8'h00, 8'hff, 1, 6);	
         				
		 	////////////////////////////////////////////////////////////////////
			// SBC : check V
			////////////////////////////////////////////////////////////////////
            resetState();
            setMem(16'hc5, 8'h40);
            setMem(16'hc4, 8'h00);
            setX(4);
            setMem(16'h4000, 8'h05);
            setA(8'h80);
            $write("%40s", "SBC ($C0,X) [binary, check V] :");
            //   (bytes,    ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hC0E1, 8'h61, 2,  8'h7a, 8'h04, 8'h00, 8'hff, 1, 6);	
           				
		 	////////////////////////////////////////////////////////////////////
			// SBC : check N
			////////////////////////////////////////////////////////////////////
            resetState();
            setMem(16'hc5, 8'h40);
            setMem(16'hc4, 8'h00);
            setX(4);
            setMem(16'h4000, 8'h00);
            $write("%40s", "SBC ($C0,X) [binary, check N] :");
            //   (bytes,    ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hC0E1, 8'ha0, 2,  8'hff, 8'h04, 8'h00, 8'hff, 1, 6);	   
                    				
		 	////////////////////////////////////////////////////////////////////
			// SBC : check Z
			////////////////////////////////////////////////////////////////////
            resetState();
            setMem(16'hc5, 8'h40);
            setMem(16'hc4, 8'h00);
            setX(4);
            setMem(16'h4000, 8'h00);
            setA(1);
            $write("%40s", "SBC ($C0,X) [binary, check Z] :");
            //   (bytes,    ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hC0E1, 8'h23, 2,  8'h00, 8'h04, 8'h00, 8'hff, 1, 6);            				
   
   
		 	////////////////////////////////////////////////////////////////////
			// SBC :  decimal, check no flags
			////////////////////////////////////////////////////////////////////
            $display("");
            resetState();
            setMem(16'hc5, 8'h40);
            setMem(16'hc4, 8'h00);
            setX(4);
            setMem(16'h4000, 8'h85);
            setPS(8'h28);
            $write("%40s", "SBC ($C0,X) [decimal, check no flags] :");
            //   (bytes,    ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hC0E1, 8'h28, 2,  8'h14, 8'h04, 8'h00, 8'hff, 1, 6);	
            				
		 	////////////////////////////////////////////////////////////////////
			// SBC :  decimal, check N
			////////////////////////////////////////////////////////////////////
            resetState();
            resetState();
            setPS(8'h28);
            setMem(16'hc5, 8'h40);
            setMem(16'hc4, 8'h00);
            setX(4);
            setMem(16'h4000, 8'h05);
            $write("%40s", "SBC ($C0,X) [decimal, check N] :");
            //   (bytes,    ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hC0E1, 8'ha8, 2,  8'h94, 8'h04, 8'h00, 8'hff, 1, 6);	
             				           				
		 	////////////////////////////////////////////////////////////////////
			// SBC :  decimal, Z
			////////////////////////////////////////////////////////////////////
            resetState();
            setPS(8'h28);
            setMem(16'hc5, 8'h40);
            setMem(16'hc4, 8'h00);
            setX(4);
            setMem(16'h4000, 8'h00);
            setA(1);
            $write("%40s", "SBC ($C0,X) [decimal, check Z,C] :");
            //   (bytes,    ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hC0E1, 8'h2b, 2,  8'h00, 8'h04, 8'h00, 8'hff, 1, 6);	
				
		 	////////////////////////////////////////////////////////////////////
			// SBC : V flag not valid in decimal mode
			////////////////////////////////////////////////////////////////////


            				
		 	////////////////////////////////////////////////////////////////////
			// STA : check memory
			////////////////////////////////////////////////////////////////////
            $display("");
            resetState();
            setMem(16'hc5, 8'h40);
            setMem(16'hc4, 8'h00);
            setX(4);
            setMem(16'h4000, 8'h20);
            setA(8'h69);
            $write("%40s", "STA ($C0,X) [check memory] :");
            //      (bytes,    ps,    pc, a,     x,     y,     sp,    #, ck, addr,   #, val)
            checkMem(48'hC081, 8'h20, 2,  8'h69, 8'h04, 8'h00, 8'hff, 1, 6,  16'h4000, 1, 48'h69);
            				
  
		end
	endtask
								
					
					
	
	
	////////////////////////////////////////////////////////////////////////////
	// Test all the zero-page-indexed-with-Y operations
	////////////////////////////////////////////////////////////////////////////
	task testZpIndxY;
		begin
			$display("\nTest zero-page indirect indexed with Y addressing:");

            				
		 	////////////////////////////////////////////////////////////////////
			// ADC : check no flags
			////////////////////////////////////////////////////////////////////
            $display("");
            resetState();
            setMem(16'hc1, 8'h40);
            setMem(16'hc0, 8'h00);
            setY(4);
            setMem(16'h4004, 8'h42);
            $write("%40s", "ADC ($C0),Y [binary, check no flags] :");
            //   (bytes,    ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hc071, 8'h20, 2,  8'h42, 8'h00, 8'h04, 8'hff, 1, 5);	
            				
		 	////////////////////////////////////////////////////////////////////
			// ADC : check no flags, x-page
			////////////////////////////////////////////////////////////////////
            resetState();
            setMem(16'hc1, 8'h40);
            setMem(16'hc0, 8'hF0);
            setY(8'h20);
            setMem(16'h4110, 8'h43);
            $write("%40s", "ADC ($C0),Y [bin, no flags, x-page] :");
            //   (bytes,    ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hc071, 8'h20, 2,  8'h43, 8'h00, 8'h20, 8'hff, 1, 6);	
            				
		 	////////////////////////////////////////////////////////////////////
			// ADC : check N
			////////////////////////////////////////////////////////////////////
            resetState();
            setA(8'h50);
            setMem(16'hc1, 8'h40);
            setMem(16'hc0, 8'h00);
            setY(4);
            setMem(16'h4004, 8'h85);
            $write("%40s", "ADC ($C0),Y [binary, check V,N] :");
            //   (bytes,    ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hC071, 8'ha0, 2,  8'hd5, 8'h00, 8'h04, 8'hff, 1, 5);	

		 	////////////////////////////////////////////////////////////////////
			// ADC : check Z
			////////////////////////////////////////////////////////////////////
            resetState();
            setMem(16'hc1, 8'h40);
            setMem(16'hc0, 8'h00);
            setY(4);
            setMem(16'h4004, 8'h00);
            $write("%40s", "ADC ($C0),Y [binary, check Z] :");
            //   (bytes,    ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hc071, 8'h22, 2,  8'h00, 8'h00, 8'h04, 8'hff, 1, 5);	


		 	////////////////////////////////////////////////////////////////////
			// ADC : check decimal, no flags
			////////////////////////////////////////////////////////////////////
            $display("");
            resetState();
            setMem(16'hc1, 8'h40);
            setMem(16'hc0, 8'h00);
            setY(4);
            setMem(16'h4004, 8'h05);
            setPS(8'h28);
            $write("%40s", "ADC ($C0),Y [decimal, check no flags] :");
            //   (bytes,    ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hc071, 8'h28, 2,  8'h05, 8'h00, 8'h04, 8'hff, 1, 5);	
            				
		 	////////////////////////////////////////////////////////////////////
			// ADC : check decimal, C
			////////////////////////////////////////////////////////////////////
            resetState();
            setPS(8'h28);
            setMem(16'hc1, 8'h40);
            setMem(16'hc0, 8'h00);
            setY(4);
            setMem(16'h4004, 8'h85);
            setA(8'h80);
            $write("%40s", "ADC ($C0),Y [decimal, check V,C] :");
            //   (bytes,    ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hC071, 8'h69, 2,  8'h65, 8'h00, 8'h04, 8'hff, 1, 5);	
            				
		 	////////////////////////////////////////////////////////////////////
			// ADC : check decimal, V,N
			////////////////////////////////////////////////////////////////////
            resetState();
            setPS(8'h28);
            setMem(16'hc1, 8'h40);
            setMem(16'hc0, 8'h00);
            setY(4);
            setMem(16'h4004, 8'h40);
            setA(8'h40);
            $write("%40s", "ADC ($C0),Y [decimal, check V,N] :");
            //   (bytes,    ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hC071, 8'he8, 2,  8'h80, 8'h00, 8'h04, 8'hff, 1, 5);	
            				
		 	////////////////////////////////////////////////////////////////////
			// ADC : check decimal, Z
			////////////////////////////////////////////////////////////////////
            resetState();
            setMem(16'hc1, 8'h40);
            setMem(16'hc0, 8'h00);
            setY(4);
            setMem(16'h4004, 8'h00);
            setPS(8'h28);
            $write("%40s", "ADC ($C0),Y [decimal, check Z] :");
            //   (bytes,    ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hC071, 8'h2a, 2,  8'h00, 8'h00, 8'h04, 8'hff, 1, 5);	

            	
            	
            				
		 	////////////////////////////////////////////////////////////////////
			// AND : check no flags
			////////////////////////////////////////////////////////////////////
            $display("");
            resetState();
            setMem(16'hc1, 8'h40);
            setMem(16'hc0, 8'h00);
            setY(4);
            setMem(16'h4004, 8'h40);
            setA(8'h40);
            $write("%40s", "AND ($C0),Y [check no flags] :");
            //   (bytes,    ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hC031, 8'h20, 2,  8'h40, 8'h00, 8'h04, 8'hff, 1, 5);	
            				
		 	////////////////////////////////////////////////////////////////////
			// AND : check z
			////////////////////////////////////////////////////////////////////
            resetState();
            setA(8'h20);
            setMem(16'hc1, 8'h40);
            setMem(16'hc0, 8'h00);
            setY(4);
            setMem(16'h4004, 8'h40);
            $write("%40s", "AND ($C0),Y [check Z] :");
            //   (bytes,    ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hC031, 8'h22, 2,  8'h00, 8'h00, 8'h04, 8'hff, 1, 5);	
            				
		 	////////////////////////////////////////////////////////////////////
			// AND : check n
			////////////////////////////////////////////////////////////////////
            resetState();
            setA(8'hff);
            setMem(16'hc1, 8'h40);
            setMem(16'hc0, 8'h00);
            setY(4);
            setMem(16'h4004, 8'h80);
            $write("%40s", "AND ($C0),Y [check N] :");
            //   (bytes,    ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hC031, 8'ha0, 2,  8'h80, 8'h00, 8'h04, 8'hff, 1, 5);	

			


		 	////////////////////////////////////////////////////////////////////
			// CMP : check no flags
			////////////////////////////////////////////////////////////////////
            $display("");
            resetState();
            setMem(16'hc1, 8'h40);
            setMem(16'hc0, 8'h00);
            setY(4);
            setMem(16'h4004, 8'h00);
            setA(5);
            $write("%40s", "CMP ($C0),Y [check C] :");
            //   (bytes,    ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hC0D1, 8'h21, 2,  8'h05, 8'h00, 8'h04, 8'hff, 1, 5);	
            				
		 	////////////////////////////////////////////////////////////////////
			// CMP : check N
			////////////////////////////////////////////////////////////////////
            resetState();
            setMem(16'hc1, 8'h40);
            setMem(16'hc0, 8'h00);
            setY(4);
            setMem(16'h4004, 8'h05);
            $write("%40s", "CMP ($C0),Y [check N] :");
            //   (bytes,    ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hC0D1, 8'ha0, 2,  8'h00, 8'h00, 8'h04, 8'hff, 1, 5);	
            				
		 	////////////////////////////////////////////////////////////////////
			// CMP : check Z
			////////////////////////////////////////////////////////////////////
            resetState();
            setMem(16'hc1, 8'h40);
            setMem(16'hc0, 8'h00);
            setY(4);
            setMem(16'h4004, 8'h05);
            setA(5);
            $write("%40s", "CMP ($C0),Y [check Z] :");
            //   (bytes,    ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hC0D1, 8'h23, 2,  8'h05, 8'h00, 8'h04, 8'hff, 1, 5);	
 
            				
            	
            				
		 	////////////////////////////////////////////////////////////////////
			// EOR : check Z
			////////////////////////////////////////////////////////////////////
            $display("");
            resetState();
            setMem(16'hc1, 8'h40);
            setMem(16'hc0, 8'h00);
            setY(4);
            setMem(16'h4004, 8'h40);
            setA(8'h40);
            $write("%40s", "EOR ($C0),Y [check Z] :");
            //   (bytes,    ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hC051, 8'h22, 2,  8'h00, 8'h00, 8'h04, 8'hff, 1, 5);	
            				
		 	////////////////////////////////////////////////////////////////////
			// EOR : check no flags
			////////////////////////////////////////////////////////////////////
            resetState();
            setMem(16'hc1, 8'h40);
            setMem(16'hc0, 8'h00);
            setY(4);
            setMem(16'h4004, 8'h40);
            setA(8'h20);
            $write("%40s", "EOR ($C0),Y [check no flags] :");
            //   (bytes,    ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hC051, 8'h20, 2,  8'h60, 8'h00, 8'h04, 8'hff, 1, 5);	
            				
		 	////////////////////////////////////////////////////////////////////
			// EOR : check n
			////////////////////////////////////////////////////////////////////
            resetState();
            setMem(16'hc1, 8'h40);
            setMem(16'hc0, 8'h00);
            setY(4);
            setMem(16'h4004, 8'h7f);
            setA(8'hff);
            $write("%40s", "EOR ($C0),Y [check N] :");
            //   (bytes,    ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hC051, 8'ha0, 2,  8'h80, 8'h00, 8'h04, 8'hff, 1, 5);	


            	
            				
		 	////////////////////////////////////////////////////////////////////
			// LDA : check no flags
			////////////////////////////////////////////////////////////////////
            $display("");
            resetState();
            setMem(16'hc1, 8'h40);
            setMem(16'hc0, 8'h00);
            setY(4);
            setMem(16'h4004, 8'h40);
            $write("%40s", "LDA ($C0),Y [check no flags] :");
            //   (bytes,    ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hC0B1, 8'h20, 2,  8'h40, 8'h00, 8'h04, 8'hff, 1, 5);	
            				
		 	////////////////////////////////////////////////////////////////////
			// LDA : check z
			////////////////////////////////////////////////////////////////////
            resetState();
            setA(8'h20);
            setMem(16'hc1, 8'h40);
            setMem(16'hc0, 8'h00);
            setY(4);
            setMem(16'h4004, 8'h00);
            $write("%40s", "LDA ($C0),Y [check Z] :");
            //   (bytes,    ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hC0B1, 8'h22, 2,  8'h00, 8'h00, 8'h04, 8'hff, 1, 5);	
            				
		 	////////////////////////////////////////////////////////////////////
			// LDA : check n
			////////////////////////////////////////////////////////////////////
            resetState();
            setMem(16'hc1, 8'h40);
            setMem(16'hc0, 8'h00);
            setY(4);
            setMem(16'h4004, 8'h80);
            $write("%40s", "LDA ($C0),Y [check N] :");
            //   (bytes,    ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hC0B1, 8'ha0, 2,  8'h80, 8'h00, 8'h04, 8'hff, 1, 5);	


            	
            				
		 	////////////////////////////////////////////////////////////////////
			// ORA : check no flags
			////////////////////////////////////////////////////////////////////
            $display("");
            resetState();
            setMem(16'hc1, 8'h40);
            setMem(16'hc0, 8'h00);
            setY(4);
            setMem(16'h4004, 8'h20);
            setA(8'h40);
            $write("%40s", "ORA ($C0),Y [check no flags] :");
            //   (bytes,    ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hC011, 8'h20, 2,  8'h60, 8'h00, 8'h04, 8'hff, 1, 5);	
            				
		 	////////////////////////////////////////////////////////////////////
			// ORA : check z
			////////////////////////////////////////////////////////////////////
            resetState();
            setMem(16'hc1, 8'h40);
            setMem(16'hc0, 8'h00);
            setY(4);
            setMem(16'h4004, 8'h00);
            $write("%40s", "ORA ($C0),Y [check Z] :");
            //   (bytes,    ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hC011, 8'h22, 2,  8'h00, 8'h00, 8'h04, 8'hff, 1, 5);	
            				
		 	////////////////////////////////////////////////////////////////////
			// ORA : check n
			////////////////////////////////////////////////////////////////////
            resetState();
            setMem(16'hc1, 8'h40);
            setMem(16'hc0, 8'h00);
            setY(4);
            setMem(16'h4004, 8'h80);
            setA(8'h7f);
            $write("%40s", "ORA ($C0),Y [check N] :");
            //   (bytes,    ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hC011, 8'ha0, 2,  8'hff, 8'h00, 8'h04, 8'hff, 1, 5);


			

		 	////////////////////////////////////////////////////////////////////
			// SBC : check no flags
			////////////////////////////////////////////////////////////////////
            $display("");
            resetState();
            setMem(16'hc1, 8'h40);
            setMem(16'hc0, 8'h00);
            setY(4);
            setMem(16'h4004, 8'h85);
            $write("%40s", "SBC ($C0),Y [binary, check no flags] :");
            //   (bytes,    ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hC0F1, 8'h20, 2,  8'h7a, 8'h00, 8'h04, 8'hff, 1, 5);

		 	////////////////////////////////////////////////////////////////////
			// SBC : check C
			////////////////////////////////////////////////////////////////////
            resetState();
            setA(8'h0b);
            setMem(16'hc1, 8'h40);
            setMem(16'hc0, 8'h00);
            setY(4);
            setMem(16'h4004, 8'h05);
            $write("%40s", "SBC ($C0),Y [binary, check C] :");
            //   (bytes,    ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hC0F1, 8'h21, 2,  8'h05, 8'h00, 8'h04, 8'hff, 1, 5);
         				
		 	////////////////////////////////////////////////////////////////////
			// SBC : check V
			////////////////////////////////////////////////////////////////////
            resetState();
            setMem(16'hc1, 8'h40);
            setMem(16'hc0, 8'h00);
            setY(4);
            setMem(16'h4004, 8'h05);
            setA(8'h80);
            $write("%40s", "SBC ($C0),Y [binary, check V] :");
            //   (bytes,    ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hC0F1, 8'h61, 2,  8'h7a, 8'h00, 8'h04, 8'hff, 1, 5);
           				
		 	////////////////////////////////////////////////////////////////////
			// SBC : check N
			////////////////////////////////////////////////////////////////////
            resetState();
            setMem(16'hc1, 8'h40);
            setMem(16'hc0, 8'h00);
            setY(4);
            setMem(16'h4004, 8'h00);
            $write("%40s", "SBC ($C0),Y [binary, check N] :");
            //   (bytes,    ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hC0F1, 8'ha0, 2,  8'hff, 8'h00, 8'h04, 8'hff, 1, 5);
                    				
		 	////////////////////////////////////////////////////////////////////
			// SBC : check Z
			////////////////////////////////////////////////////////////////////
            resetState();
            setMem(16'hc1, 8'h40);
            setMem(16'hc0, 8'h00);
            setY(4);
            setMem(16'h4004, 8'h00);
            setA(1);
            $write("%40s", "SBC ($C0),Y [binary, check Z] :");
            //   (bytes,    ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hC0F1, 8'h23, 2,  8'h00, 8'h00, 8'h04, 8'hff, 1, 5);
   
   
		 	////////////////////////////////////////////////////////////////////
			// SBC :  decimal, check no flags
			////////////////////////////////////////////////////////////////////
            $display("");
            resetState();
            setMem(16'hc1, 8'h40);
            setMem(16'hc0, 8'h00);
            setY(4);
            setMem(16'h4004, 8'h85);
            setPS(8'h28);
            $write("%40s", "SBC ($C0),Y [decimal, check no flags] :");
            //   (bytes,    ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hC0F1, 8'h28, 2,  8'h14, 8'h00, 8'h04, 8'hff, 1, 5);
            				
		 	////////////////////////////////////////////////////////////////////
			// SBC :  decimal, check N
			////////////////////////////////////////////////////////////////////
            resetState();
            setPS(8'h28);
            setMem(16'hc1, 8'h40);
            setMem(16'hc0, 8'h00);
            setY(4);
            setMem(16'h4004, 8'h05);
            $write("%40s", "SBC ($C0),Y [decimal, check N] :");
            //   (bytes,    ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hC0F1, 8'ha8, 2,  8'h94, 8'h00, 8'h04, 8'hff, 1, 5);
             				           				
		 	////////////////////////////////////////////////////////////////////
			// SBC :  decimal, Z
			////////////////////////////////////////////////////////////////////
            resetState();
            setPS(8'h28);
            setMem(16'hc1, 8'h40);
            setMem(16'hc0, 8'h00);
            setY(4);
            setMem(16'h4004, 8'h00);
            setA(1);
            $write("%40s", "SBC ($C0),Y [decimal, check Z,C] :");
            //   (bytes,    ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'hC0F1, 8'h2b, 2,  8'h00, 8'h00, 8'h04, 8'hff, 1, 5);
				
		 	////////////////////////////////////////////////////////////////////
			// SBC : V flag not valid in decimal mode
			////////////////////////////////////////////////////////////////////


            				
		 	////////////////////////////////////////////////////////////////////
			// STA : check memory
			////////////////////////////////////////////////////////////////////
            $display("");
            resetState();
            setMem(16'hc1, 8'h40);
            setMem(16'hc0, 8'h00);
            setY(4);
            setMem(16'h4004, 8'h00);
            setA(8'h69);
            $write("%40s", "STA ($C0),Y [check memory] :");
            //      (bytes,    ps,    pc, a,     x,     y,     sp,    #, ck, addr,   #, val)
            checkMem(48'hC091, 8'h20, 2,  8'h69, 8'h00, 8'h04, 8'hff, 1, 5,  16'h4004, 1, 48'h69);
		end
	endtask

					
					
	
	
	////////////////////////////////////////////////////////////////////////////
	// Test all the branch-relative instructions
	////////////////////////////////////////////////////////////////////////////
	task testBranch;
		begin
			$display("\nTest branch-relative:");
			
		 	////////////////////////////////////////////////////////////////////
			// BCC : check not taken
			////////////////////////////////////////////////////////////////////
            $display("");
            resetState();
            setPS(8'h21);
            $write("%40s", "BCC $40 [check not taken] :");
            //   (bytes,    ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'h4090, 8'h21, 2,  8'h00, 8'h00, 8'h00, 8'hff, 1, 2);	   
			
		 	////////////////////////////////////////////////////////////////////
			// BCC : check taken
			////////////////////////////////////////////////////////////////////
            resetState();
            $write("%40s", "BCC $40 [check taken] :");
            //   (bytes,    ps,    pc,     a,     x,     y,     sp,    #, clocks)
            check(48'h4090, 8'h20, 8'h42,  8'h00, 8'h00, 8'h00, 8'hff, 1, 3);	                       				
			
		 	////////////////////////////////////////////////////////////////////
			// BCC : check taken, x-page
			////////////////////////////////////////////////////////////////////
            resetState();
            setPC(16'hf0);
            $write("%40s", "BCC $40 [check taken, x-page] :");
            //   (bytes,    ps,    pc,        a,     x,     y,     sp,    #, clocks)
            check(48'h4090, 8'h20, 16'h0132,  8'h00, 8'h00, 8'h00, 8'hff, 1, 4);	                       				
			
			
			
		 	////////////////////////////////////////////////////////////////////
			// BCS : check not taken
			////////////////////////////////////////////////////////////////////
            $display("");
            resetState();
            $write("%40s", "BCS $40 [check not taken] :");
            //   (bytes,    ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'h40B0, 8'h20, 2,  8'h00, 8'h00, 8'h00, 8'hff, 1, 2);	   
			
		 	////////////////////////////////////////////////////////////////////
			// BCS : check taken
			////////////////////////////////////////////////////////////////////
            resetState();
            setPS(8'h21);
            $write("%40s", "BCS $40 [check taken] :");
            //   (bytes,    ps,    pc,     a,     x,     y,     sp,    #, clocks)
            check(48'h40B0, 8'h21, 8'h42,  8'h00, 8'h00, 8'h00, 8'hff, 1, 3);	                       				
			
		 	////////////////////////////////////////////////////////////////////
			// BCS : check taken, x-page
			////////////////////////////////////////////////////////////////////
            resetState();
            setPS(8'h21);
            setPC(16'hf0);
            $write("%40s", "BCS $40 [check taken, x-page] :");
            //   (bytes,    ps,    pc,        a,     x,     y,     sp,    #, clocks)
            check(48'h40B0, 8'h21, 16'h0132,  8'h00, 8'h00, 8'h00, 8'hff, 1, 4);	                       				
			
			
			
		 	////////////////////////////////////////////////////////////////////
			// BEQ : check not taken
			////////////////////////////////////////////////////////////////////
            $display("");
            resetState();
            $write("%40s", "BEQ $40 [check not taken] :");
            //   (bytes,    ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'h40F0, 8'h20, 2,  8'h00, 8'h00, 8'h00, 8'hff, 1, 2);	   
			
		 	////////////////////////////////////////////////////////////////////
			// BEQ : check taken
			////////////////////////////////////////////////////////////////////
            resetState();
            setPS(8'h22);
            $write("%40s", "BEQ $40 [check taken] :");
            //   (bytes,    ps,    pc,     a,     x,     y,     sp,    #, clocks)
            check(48'h40F0, 8'h22, 8'h42,  8'h00, 8'h00, 8'h00, 8'hff, 1, 3);	                       				
			
		 	////////////////////////////////////////////////////////////////////
			// BEQ : check taken, x-page
			////////////////////////////////////////////////////////////////////
            resetState();
            setPS(8'h22);
            setPC(16'hf0);
            $write("%40s", "BEQ $40 [check taken, x-page] :");
            //   (bytes,    ps,    pc,        a,     x,     y,     sp,    #, clocks)
            check(48'h40F0, 8'h22, 16'h0132,  8'h00, 8'h00, 8'h00, 8'hff, 1, 4);	                       				
			
			
			
		 	////////////////////////////////////////////////////////////////////
			// BMI : check not taken
			////////////////////////////////////////////////////////////////////
            $display("");
            resetState();
            $write("%40s", "BMI $40 [check not taken] :");
            //   (bytes,    ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'h4030, 8'h20, 2,  8'h00, 8'h00, 8'h00, 8'hff, 1, 2);	   
			
		 	////////////////////////////////////////////////////////////////////
			// BMI : check taken
			////////////////////////////////////////////////////////////////////
            resetState();
            setPS(8'ha0);
            $write("%40s", "BMI $40 [check taken] :");
            //   (bytes,    ps,    pc,     a,     x,     y,     sp,    #, clocks)
            check(48'h4030, 8'ha0, 8'h42,  8'h00, 8'h00, 8'h00, 8'hff, 1, 3);	                       				
			
		 	////////////////////////////////////////////////////////////////////
			// BMI : check taken, x-page
			////////////////////////////////////////////////////////////////////
            resetState();
            setPS(8'ha0);
            setPC(16'hf0);
            $write("%40s", "BMI $40 [check taken, x-page] :");
            //   (bytes,    ps,    pc,        a,     x,     y,     sp,    #, clocks)
            check(48'h4030, 8'ha0, 16'h0132,  8'h00, 8'h00, 8'h00, 8'hff, 1, 4);	                       				

			
			
		 	////////////////////////////////////////////////////////////////////
			// BNE : check not taken
			////////////////////////////////////////////////////////////////////
            $display("");
            resetState();
            setPS(8'h22);
            $write("%40s", "BNE $40 [check not taken] :");
            //   (bytes,    ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'h40D0, 8'h22, 2,  8'h00, 8'h00, 8'h00, 8'hff, 1, 2);	   
			
		 	////////////////////////////////////////////////////////////////////
			// BNE : check taken
			////////////////////////////////////////////////////////////////////
            resetState();
            $write("%40s", "BNE $40 [check taken] :");
            //   (bytes,    ps,    pc,     a,     x,     y,     sp,    #, clocks)
            check(48'h40D0, 8'h20, 8'h42,  8'h00, 8'h00, 8'h00, 8'hff, 1, 3);	                       				
			
		 	////////////////////////////////////////////////////////////////////
			// BNE : check taken, x-page
			////////////////////////////////////////////////////////////////////
            resetState();
            setPC(16'hf0);
            $write("%40s", "BNE $40 [check taken, x-page] :");
            //   (bytes,    ps,    pc,        a,     x,     y,     sp,    #, clocks)
            check(48'h40D0, 8'h20, 16'h0132,  8'h00, 8'h00, 8'h00, 8'hff, 1, 4);	                       				

			
			
		 	////////////////////////////////////////////////////////////////////
			// BPL : check not taken
			////////////////////////////////////////////////////////////////////
            $display("");
            resetState();
            setPS(8'ha0);
            $write("%40s", "BPL $40 [check not taken] :");
            //   (bytes,    ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'h4010, 8'ha0, 2,  8'h00, 8'h00, 8'h00, 8'hff, 1, 2);	   
			
		 	////////////////////////////////////////////////////////////////////
			// BPL : check taken
			////////////////////////////////////////////////////////////////////
            resetState();
            $write("%40s", "BPL $40 [check taken] :");
            //   (bytes,    ps,    pc,     a,     x,     y,     sp,    #, clocks)
            check(48'h4010, 8'h20, 8'h42,  8'h00, 8'h00, 8'h00, 8'hff, 1, 3);	                       				
			
		 	////////////////////////////////////////////////////////////////////
			// BPL : check taken, x-page
			////////////////////////////////////////////////////////////////////
            resetState();
            setPC(16'hf0);
            $write("%40s", "BPL $40 [check taken, x-page] :");
            //   (bytes,    ps,    pc,        a,     x,     y,     sp,    #, clocks)
            check(48'h4010, 8'h20, 16'h0132,  8'h00, 8'h00, 8'h00, 8'hff, 1, 4);	                       				

			
			
		 	////////////////////////////////////////////////////////////////////
			// BVC : check not taken
			////////////////////////////////////////////////////////////////////
            $display("");
            resetState();
            setPS(8'h60);
            $write("%40s", "BVC $40 [check not taken] :");
            //   (bytes,    ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'h4050, 8'h60, 2,  8'h00, 8'h00, 8'h00, 8'hff, 1, 2);	   
			
		 	////////////////////////////////////////////////////////////////////
			// BVC : check taken
			////////////////////////////////////////////////////////////////////
            resetState();
            $write("%40s", "BVC $40 [check taken] :");
            //   (bytes,    ps,    pc,     a,     x,     y,     sp,    #, clocks)
            check(48'h4050, 8'h20, 8'h42,  8'h00, 8'h00, 8'h00, 8'hff, 1, 3);	                       				
			
		 	////////////////////////////////////////////////////////////////////
			// BVC : check taken, x-page
			////////////////////////////////////////////////////////////////////
            resetState();
            setPC(16'hf0);
            $write("%40s", "BVC $40 [check taken, x-page] :");
            //   (bytes,    ps,    pc,        a,     x,     y,     sp,    #, clocks)
            check(48'h4050, 8'h20, 16'h0132,  8'h00, 8'h00, 8'h00, 8'hff, 1, 4);	                       				
			
			
			
		 	////////////////////////////////////////////////////////////////////
			// BVS : check not taken
			////////////////////////////////////////////////////////////////////
            $display("");
            resetState();
            $write("%40s", "BVS $40 [check not taken] :");
            //   (bytes,    ps,    pc, a,     x,     y,     sp,    #, clocks)
            check(48'h4070, 8'h20, 2,  8'h00, 8'h00, 8'h00, 8'hff, 1, 2);	   
			
		 	////////////////////////////////////////////////////////////////////
			// BVS : check taken
			////////////////////////////////////////////////////////////////////
            resetState();
            setPS(8'h60);
            $write("%40s", "BVS $40 [check taken] :");
            //   (bytes,    ps,    pc,     a,     x,     y,     sp,    #, clocks)
            check(48'h4070, 8'h60, 8'h42,  8'h00, 8'h00, 8'h00, 8'hff, 1, 3);	                       				
			
		 	////////////////////////////////////////////////////////////////////
			// BVS : check taken, x-page
			////////////////////////////////////////////////////////////////////
            resetState();
            setPS(8'h60);
            setPC(16'hf0);
            $write("%40s", "BVS $40 [check taken, x-page] :");
            //   (bytes,    ps,    pc,        a,     x,     y,     sp,    #, clocks)
            check(48'h4070, 8'h60, 16'h0132,  8'h00, 8'h00, 8'h00, 8'hff, 1, 4);	                       				
		end
	endtask

            				
